/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.Apprentissage;

import java.awt.*;
import java.io.*;
import visuAnalor.analyses.*;
import visuAnalor.analyses.DetectionPeriodes.*;
import visuAnalor.donnees.*;
import visuAnalor.fichiers.FichiersJava;
import visuAnalor.parametres.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class ApprendPeriodes implements CalculApprentissage.Evaluateur {
  private Parametres2 parametres;
  private double[] vals, perfs;
  private int nbExemples;
  private CalculApprentissage calcul;
  private VisuApprentissage visu;
  private CandidatCalcule[][] candidats;
  private boolean[][] references;
  // Marge d'acceptation d'un écart entre la borne d'une période calculée 
  // et la borne d'une période de référence
  static final double MARGE = 0.01;
  public static void newApprentissage(Frame fenetre, Parametres2 pars) {
    File[] fics = Fichiers.choisirCorpus("Choix du corpus",
            Fichiers.TypeFichier.CORPUS_ANALOR);
    if (fics.length==0) return;
    ApprendPeriodes apprend = new ApprendPeriodes(fenetre, pars);
    apprend.initApprend(fics);
  }
  private ApprendPeriodes(Frame fenetrePrincipale, Parametres2 pars) {
    parametres = pars;
    // Attention à l'ordre des paramètres à apprendre
    vals = new double[]{
      pars.getSeuilPausePeriodes(),
      pars.getSeuilGestePeriodes(),
      pars.getSeuilSautPeriodes(),
      pars.getPoidsPausePeriodes(),
      pars.getPoidsGestePeriodes(),
      pars.getPoidsSautPeriodes(),};
    String titre = "Apprentissage des paramètres de détection de périodes";
    String[][] labels = {
      {"Seuil de durée", "des pauses (secondes)"},
      {"Seuil de hauteur", "des gestes (demi-tons)"},
      {"Seuil de hauteur", "des sauts (demi-tons)"},
      {"Poids des durées", "des pauses"},
      {"Poids des hauteurs", "des gestes"},
      {"Poids des hauteurs", "des sauts"},};
    float[] formats = {100, 1200, 1200, 100, 100, 100};
    visu = new VisuApprentissage(fenetrePrincipale, this, titre, labels, formats);
  }
  private void initApprend(File[] fics) {
    if (!initCandidats(fics)) return;
    visu.setNbExemples(nbExemples);
    perfs = evaluer(vals);
    visu.setValsInitiales(vals, perfs);
    visu.setValsCourantes(vals, perfs);
    visu.setVisible(true);
  }
// pour passer les valeurs courantes à CalculApprentissage
  @Override
  public double[] getVals() {
    return vals;
  }
// pour passer les performances courantes à CalculApprentissage
  @Override
  public double[] getPerfs() {
    return perfs;
  }
  // à utiliser après initSyllabes
  @Override
  public double[] evaluer(double[] vals) {
    int nbPersCalc = 0, nbPersRef = 0, nbPersOk = 0;
    CandidatCalcule.setVals(vals);
    for (int i = 0; i<candidats.length; i++) {
      int nbptscumule = 0;
      double fqmoycumulee = 0;
      for (int j = 0; j<candidats[i].length; j++) {
        boolean isPerCalc = candidats[i][j].calculerSiPeriode(nbptscumule, fqmoycumulee);
        if (isPerCalc) {
          nbptscumule = 0;
          fqmoycumulee = 0;
        } else {
          nbptscumule = candidats[i][j].getNewnbpoints();
          fqmoycumulee = candidats[i][j].getNewfqMoyenne();
        }
        boolean isPerRef = references[i][j];
        if (isPerCalc) nbPersCalc++;
        if (isPerRef) nbPersRef++;
        if (isPerCalc&&isPerRef) nbPersOk++;
//        System.out.println(j+" : calc "+isPromCalc+" ; ref "+isPromRef);
      }
    }
    double nbDoubleOk = (double) nbPersOk;
    return new double[]{nbDoubleOk/nbPersCalc, nbDoubleOk/nbPersRef,
              2*nbDoubleOk/(nbPersCalc+nbPersRef)};
  }
  private boolean initCandidats(File[] fics) {
    DetectionPeriodes detect = new DetectionPeriodes(parametres);
    references = new boolean[fics.length][];
    candidats = new CandidatCalcule[fics.length][];
    if (parametres.getSeuilPausePeriodes()==0
            ||parametres.getSeuilGestePeriodes()==0
            ||parametres.getSeuilSautPeriodes()==0) {
      GMessages.impossibiliteAction(
              "Paramètres de seuil des critères incorrects");
      return false;
    }
    if (parametres.getPoidsPausePeriodes()==0
            &&parametres.getPoidsGestePeriodes()==0
            &&parametres.getPoidsSautPeriodes()==0) {
      GMessages.impossibiliteAction(
              "Paramètres de pondération des critères incorrects");
      return false;
    }
    Donnees donnees = new Donnees();
    for (int i = 0; i<fics.length; i++) {
      String nomDonnees = Fichiers.extraireNom(fics[i]);
      FichiersJava.ouvrirDonnees(donnees, fics[i], false);
      Tire tireRef = donnees.trouverTireParNom(
              parametres.getNomTirePeriodesReference());
      if (tireRef==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les périodes de référence\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          candidats[i] = new CandidatCalcule[0];
          continue;
        }
      }
      CourbeF0 f0 = donnees.getCourbeF0();
      if (f0==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé de fondamentale\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          candidats[i] = new CandidatCalcule[0];
          continue;
        }
      }
      double[] bornesTirePauses, bornesLoc, bornesTireEuh, bornesTireRef;
      boolean[] isPause, isEuh;
      bornesTireRef = tireRef.getBornes();
      Tire tirePauses = donnees.trouverTireParNom(parametres.getNomTirePausesPeriodes());
      if (tirePauses==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les pauses\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          candidats[i] = new CandidatCalcule[0];
          continue;
        }
      } else if (tirePauses.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : La tire indiquant les pauses\n"
                +"doit être une tire de type intervalle\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          candidats[i] = new CandidatCalcule[0];
          continue;
        }
      } else {
        bornesTirePauses = tirePauses.getBornes();
        isPause = CalculsAnalyses.reperer(tirePauses.getMots(),
                parametres.getMarquesPausePeriodes());
      }
      Tire tireEuh = donnees.trouverTireParNom(parametres.getNomTireEuh());
      if (tireEuh==null) {
        if (!GMessages.confirmationAction(nomDonnees+"Pas trouvé la tire "
                +"indiquant les \"euh\"\nAucun \"euh\" ne sera pris  en compte"))
          return false;
        bornesTireEuh = new double[0];
        isEuh = null;
      } else if (tireEuh.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees+"La tire indiquant les \"euh\"\n"
                +"doit être une tire de type intervalle"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          candidats[i] = new CandidatCalcule[0];
          continue;
        }
      } else {
        bornesTireEuh = tireEuh.getBornes();
        isEuh = CalculsAnalyses.reperer(tireEuh.getMots(), parametres.getMarquesEuh());
      }
      Tire tireLoc = donnees.trouverTireParNom(parametres.getNomTireLocuteurs());
      if (tireLoc==null) bornesLoc = new double[0];
      else bornesLoc = tireLoc.getBornes();
      CourbeF0 f0joint = new CourbeF0(f0);
      CalculsAnalyses.joindre(f0joint, bornesTirePauses, isPause, bornesLoc);
      double moyenneFq = f0joint.calculerMoyenne();
      detect.determinerCandidats(f0joint, moyenneFq, bornesTirePauses, isPause,
              bornesLoc, bornesTireEuh, isEuh);
      Candidat[] cands = detect.getCandidats();
      candidats[i] = new CandidatCalcule[cands.length];
      references[i] = new boolean[cands.length];
      for (int j = 0; j<cands.length; j++) {
        double bsup = cands[j].bornes[1];
        references[i][j] = CalculsAnalyses.trouverBorneDansIntervalle(
                bornesTireRef, bsup-MARGE, bsup+MARGE)>=0;
        candidats[i][j] = new CandidatCalcule();
        candidats[i][j].init(cands[j]);
//               System.out.println(j+" : calc "+cands[j].decision.isPeriode()+
//                       " ; ref "+references[i][j]);

      }
    }
    nbExemples = 0;
    for (int i = 0; i<fics.length; i++) nbExemples += candidats[i].length;
    if (nbExemples>0) return true;
    GMessages.impossibiliteAction(
            "Aucune donnée pertinente dans les documents choisis\n"
            +"Apprentissage impossible");
    return false;
  }
  @Override
  public void appliquer() {
    parametres.setSeuilPausePeriodes(vals[0]);
    parametres.setSeuilGestePeriodes(vals[1]);
    parametres.setSeuilSautPeriodes(vals[2]);
    parametres.setPoidsPausePeriodes(vals[3]);
    parametres.setPoidsGestePeriodes(vals[4]);
    parametres.setPoidsSautPeriodes(vals[5]);
    parametres.modifApprendPeriodes();
  }
  @Override
  public void demarrerReprendre() {
    visu.setValsInitiales(vals, perfs);
    calcul = new CalculApprentissage(this, visu);
    calcul.start();
  }
  @Override
  public void stopper() {
    calcul.stopper();
  }
}
