/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.Apprentissage;

import java.awt.*;
import java.io.*;
import visuAnalor.analyses.*;
import visuAnalor.analyses.DetectionProeminences.*;
import visuAnalor.donnees.*;
import visuAnalor.fichiers.FichiersJava;
import visuAnalor.parametres.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class ApprendProeminences implements CalculApprentissage.Evaluateur {
  private Parametres2 parametres;
  private double[] vals, perfs;
  private int nbExemples;
  private CalculApprentissage calcul;
  private VisuApprentissage visu;
  private SyllabeCalculee[][] syllabes;
  private boolean[][] references;
  public static void newApprentissage(Frame fenetre, Parametres2 pars) {
    File[] fics = Fichiers.choisirCorpus("Choix du corpus",
            Fichiers.TypeFichier.CORPUS_ANALOR);
    if (fics.length==0) return;
    ApprendProeminences apprend = new ApprendProeminences(fenetre, pars);
    apprend.initApprend(fics);
  }
  private ApprendProeminences(Frame fenetrePrincipale, Parametres2 pars) {
    parametres = pars;
    // Attention à l'ordre des paramètres à apprendre
    vals = new double[]{
      pars.getSeuilDureeProms(),
      pars.getSeuilHauteurProms(),
      pars.getSeuilMonteeProms(),
      pars.getPoidsPhonemes()
    };
    String titre = "Apprentissage des paramètres de détection de proéminences";
    String[][] labels = {
      {"Seuil de durée", "(relatif)"},
      {"Seuil de hauteur", "(demi-tons)"},
      {"Seuil de montée", "(demi-tons)"},
      {"Poids des phonèmes", "pour la durée"}
    };
    float[] formats = {100, 1200, 1200, 100};
    visu = new VisuApprentissage(fenetrePrincipale, this, titre, labels, formats);
  }
  private void initApprend(File[] fics) {
    if (!initSyllabes(fics)) return;
    visu.setNbExemples(nbExemples);
    perfs = evaluer(vals);
    visu.setValsInitiales(vals, perfs);
    visu.setValsCourantes(vals, perfs);
    visu.setVisible(true);
  }
// pour passer les valeurs courantes à CalculApprentissage
  @Override
  public double[] getVals() {
    return vals;
  }
// pour passer les performances courantes à CalculApprentissage
  @Override
  public double[] getPerfs() {
    return perfs;
  }
  // à utiliser après initSyllabes
  @Override
  public double[] evaluer(double[] vals) {
    int nbPromsCalc = 0, nbPromsRef = 0, nbPromsOk = 0;
    // Attention poidsphonemes doit être inférieur ou égal à 1
    // C'est ici que l'on impose cette contrainte
    vals[3] = Math.min(vals[3], 1);
    SyllabeCalculee.setVals(vals);
    for (int i = 0; i<syllabes.length; i++) {
      SyllabeCalculee syllPrecProm = null;
      for (int j = 0; j<syllabes[i].length; j++) {
        boolean isPromCalc = syllabes[i][j].isProm(syllPrecProm);
        syllPrecProm = isPromCalc ? syllabes[i][j] : null;
        boolean isPromRef = references[i][j];
        if (isPromCalc) nbPromsCalc++;
        if (isPromRef) nbPromsRef++;
        if (isPromCalc&&isPromRef) nbPromsOk++;
      }
    }
    double nbDoubleOk = (double) nbPromsOk;
    return new double[]{nbDoubleOk/nbPromsCalc, nbDoubleOk/nbPromsRef,
              2*nbDoubleOk/(nbPromsCalc+nbPromsRef)};
  }
  private boolean initSyllabes(File[] fics) {
    DetectionProeminences detect = new DetectionProeminences(parametres);
    references = new boolean[fics.length][];
    syllabes = new SyllabeCalculee[fics.length][];
    if (parametres.getEmpans()[0]==0&&parametres.getEmpans()[1]==0) {
      GMessages.impossibiliteAction("Paramètres d'empan incorrects");
      return false;
    }
    if (parametres.getPoidsForcePause()==0
            &&parametres.getPoidsForceMontee()==0
            &&parametres.getPoidsForceHauteur()==0
            &&parametres.getPoidsForceDuree()==0) {
      GMessages.impossibiliteAction(
              "Paramètres de pondération des critères incorrects");
      return false;
    }
    Donnees donnees = new Donnees();
    for (int i = 0; i<fics.length; i++) {
      String nomDonnees = Fichiers.extraireNom(fics[i]);
      FichiersJava.ouvrirDonnees(donnees, fics[i], false);
      Tire tireRef = donnees.trouverTireParNom(
              parametres.getNomTirePromsReference());
      if (tireRef==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les proéminences de référence\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          syllabes[i] = new SyllabeCalculee[0];
          continue;
        }
      }
      references[i] = CalculsAnalyses.reperer(tireRef.getMots(),
              parametres.getMarquesPromsReference());
      CourbeF0 f0 = donnees.getCourbeF0();
      if (f0==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé de fondamentale\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          syllabes[i] = new SyllabeCalculee[0];
          continue;
        }
      }
      double[] bornesTireSyll, bornesTireDysf, bornesTirePhon;
      boolean[] isPause, isDysf, isVoyelle;
      String[] textesSyll;
      Tire tireSyll = donnees.trouverTireParNom(
              parametres.getNomTireSyllabesProms());
      if (tireSyll==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les syllabes\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          syllabes[i] = new SyllabeCalculee[0];
          continue;
        }
      }
      if (tireSyll.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees+" : La tire syllabique\n"
                +"doit être une tire de type intervalle\n"
                +"Ce document ne sera pas pris en compte")) return false;
        syllabes[i] = new SyllabeCalculee[0];
        continue;
      }
      bornesTireSyll = tireSyll.getBornes();
      textesSyll = tireSyll.getMots();
      if (textesSyll.length!=references[i].length) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : La tire indiquant les proéminences de référence\n"
                +"n'a pas la même taille que la tire syllabique\n"
                +"Ce document ne sera pas pris en compte")) return false;
        syllabes[i] = new SyllabeCalculee[0];
        continue;
      }
      isPause = CalculsAnalyses.reperer(textesSyll,
              parametres.getMarquesPauseProms());
      Tire tireDysf = donnees.trouverTireParNom(
              parametres.getNomTireDysFluences());
      if (tireDysf==null) {
        if (!GMessages.confirmationAction(nomDonnees+" : Pas trouvé la tire "
                +"indiquant les dysfluences\n"
                +"Aucune dysfluence ne sera prise en compte"))
          return false;
        bornesTireDysf = new double[0];
        isDysf = null;
      } else if (tireDysf.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : La tire indiquant les dysfluences\n"
                +"doit être une tire de type intervalle\n"
                +"Ce document ne sera pas pris en compte")) return false;
        syllabes[i] = new SyllabeCalculee[0];
        continue;
      } else {
        bornesTireDysf = tireDysf.getBornes();
        isDysf = CalculsAnalyses.reperer(tireDysf.getMots(),
                parametres.getMarquesDysFluences());
      }
      if (!parametres.getRestrictionVocalique()
              &&parametres.getPoidsPhonemes()==0) {
        bornesTirePhon = new double[0];
        isVoyelle = null;
      } else {
        Tire tirePhon = donnees.trouverTireParNom(
                parametres.getNomTirePhonemes());
        if (tirePhon==null) {
          if (!GMessages.confirmationAction(nomDonnees
                  +" : Pas trouvé la tire phonémique. Le poids des phonèmes"
                  +" et la restriction vocalique seront ignorés"))
            return false;
          bornesTirePhon = new double[0];
          isVoyelle = null;
        } else if (tirePhon.getType()!=Tire.TypeTire.INTERVALLE) {
          if (!GMessages.confirmationAction(nomDonnees+" La tire phonémique\n"
                  +"doit être une tire de type intervalle\n"
                  +"Ce document ne sera pas pris en compte")) return false;
          syllabes[i] = new SyllabeCalculee[0];
          continue;
        } else {
          bornesTirePhon = tirePhon.getBornes();
          isVoyelle = CalculsAnalyses.reperer(tirePhon.getMots(),
                  parametres.getVoyelles());
        }
      }
      detect.determinerProeminences(f0, bornesTireSyll, textesSyll, isPause,
              bornesTireDysf, isDysf, bornesTirePhon, isVoyelle);
      Syllabe[] sylls = detect.getSyllabes();
      syllabes[i] = new SyllabeCalculee[sylls.length];
      for (int j = 0; j<sylls.length; j++) {
        syllabes[i][j] = new SyllabeCalculee();
        syllabes[i][j].init(sylls[j]);
      }
    }
    nbExemples =0;
    for (int i = 0; i<fics.length; i++) nbExemples += syllabes[i].length;
    if(nbExemples>0) return true;
    GMessages.impossibiliteAction(
            "Aucune donnée pertinente dans les documents choisis\n"
            +"Apprentissage impossible");
    return false;
  }
  @Override
  public void appliquer() {
    parametres.setSeuilDureeProms(vals[0]);
    parametres.setSeuilHauteurProms(vals[1]);
    parametres.setSeuilMonteeProms(vals[2]);
    parametres.setPoidsPhonemes(vals[3]);
    parametres.modifApprendProeminences();
  }
  @Override
  public void demarrerReprendre() {
    visu.setValsInitiales(vals, perfs);
    calcul = new CalculApprentissage(this, visu);
    calcul.start();
  }
  @Override
  public void stopper() {
    calcul.stopper();
  }
}
