/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import org.jfree.chart.annotations.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import visuAnalor.PanneauEditeur.*;
import visuAnalor.analyses.DetectionPeriodes;
import visuAnalor.analyses.DetectionProeminences;
import visuAnalor.analyses.DetectionProeminences.Syllabe;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
public class BandeauResultats extends XYPlot implements Bandeau {
  private final static Stroke LIGNE_SIMPLE = new BasicStroke(1f);
  private final static Stroke LIGNE_EPAISSE = new BasicStroke(2f);
  private final static Paint FOND_INTERVALLE = new Color(0.95f, 0.95f, 0.95f);
  private final static Paint FOND_SELECTION = new Color(1f, 0.7f, 0.7f, 0.5f);
  private final static Paint COULEUR_PERIODE = new Color(0f, 0.5f, 0f);
  private final static Font FONTE_PERIODE = Font.decode("Arial-bold-18");
  private final static Paint COULEUR_PROM = new Color(0f, 0f, 0.5f);
  private final static Font FONTE_PROM = Font.decode("Arial-bold");
  private PanneauEditeur editeur;
  private double[][] parcoursIntervalles = null;
  private int indSelect = -1;
  private ArrayList<XYLineAnnotation> parcours =
          new ArrayList<XYLineAnnotation>();
  private XYLineAnnotation parcoursSelect = null;
  private DetectionPeriodes detectPeriodes;
  private double[] periodesPositions = null;
  private ArrayList<XYTextAnnotation> periodes =
          new ArrayList<XYTextAnnotation>();
  private JPopupPeriode popupPeriode;
  private DetectionProeminences detectProms;
  private double[] promsBornesPositions = null;
  private ArrayList<XYLineAnnotation> promsBornes =
          new ArrayList<XYLineAnnotation>();
  private ArrayList<XYTextAnnotation> proms =
          new ArrayList<XYTextAnnotation>();
   private JPopupProm popupProm;
 public BandeauResultats(PanneauEditeur editeur) {
    super();
    this.editeur = editeur;
    NumberAxis axe = new NumberAxis();
    axe.setTickLabelsVisible(false);
    axe.setTickMarksVisible(false);
    axe.setLabelPaint(Color.RED);
    axe.setLabelAngle(Math.PI/2);
    axe.setLabelFont(BandeauTire.FONTE_TITRE_SELECT);
    axe.setVisible(false);
    setRangeAxis(axe);
    setDomainGridlinesVisible(false);
    setRangeGridlinesVisible(false);
    setBackgroundPaint(null);
    popupPeriode = new JPopupPeriode(editeur);
    popupProm = new JPopupProm(editeur);
  }
  void supParcoursF0() {
    parcoursIntervalles = null;
    for (XYLineAnnotation segment : parcours) removeAnnotation(segment);
    parcours.clear();
    getRangeAxis().setVisible(false);
    indSelect = -1;
    effacerSelection();
  }
  void setParcoursF0(double[][] intervalles, boolean initSelect) {
    ValueAxis axe = getRangeAxis();
    this.parcoursIntervalles = intervalles;
    if (intervalles.length==0) {
      supParcoursF0();
      return;
    }
    for (XYLineAnnotation segment : parcours) removeAnnotation(segment);
    parcours.clear();
    for (double[] intervalle : intervalles) {
      XYLineAnnotation segment = new XYLineAnnotation(intervalle[0], 1,
              intervalle[1], 1, LIGNE_SIMPLE, Color.BLACK);
      parcours.add(segment);
      addAnnotation(segment);
    }
    if(initSelect) {
      indSelect = 0;
       axe.setLabel("[\u2190 | \u2192]");
      axe.setVisible(true);
      afficherSelection();    
    } else {
    if(indSelect<0) indSelect = 0;
    else if(indSelect>= intervalles.length) indSelect = intervalles.length-1;
    afficherSelection();
    }
  }
  double[] getSelectionParcours() {
       if (parcoursIntervalles==null||indSelect<0) return null;
       return parcoursIntervalles[indSelect];
  }
  boolean deplacerSelection(int sens) {
    if (parcoursIntervalles==null||indSelect<0) return false;
    if (parcoursSelect!=null) incrementerIndSelect(sens);
    afficherSelection();
    return true;
  }
  boolean actionClicAnnot(double t, double y, boolean ajout, boolean doubleClic) {
    return actionClicPlot(t, y, ajout, doubleClic);
  }
  final static double SEUIL_PROCHE_INTERVALLE = 0.003;
  boolean actionClicPlot(double t, double y, boolean ajout, boolean doubleClic) {
    if (parcoursIntervalles==null) return false;
    ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
    double seuil = 2*(axe.getUpperBound()-axe.getLowerBound())
            *SEUIL_PROCHE_INTERVALLE;
    boolean modifSelection = false;
    for (int ind = 0; ind<parcoursIntervalles.length; ind++) {
      double[] intervalle = parcoursIntervalles[ind];
      if (t>intervalle[1]+seuil) continue;
      if (t<intervalle[0]-seuil) break;
      if (parcoursSelect!=null&&ind==indSelect) break;
      indSelect = ind;
      afficherSelection();
      modifSelection = true;
    }
    return modifSelection;
  }
  final static double SEUIL_PROCHE_ANNOTATION = 0.01;
  JPopupMenu getPopup(double x) {
    ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
    double fentemp = axe.getUpperBound()-axe.getLowerBound();
    if (detectProms!=null) {
      int nosyll = trouverNoSyll(x);
      Syllabe syll = detectProms.getSyllabes()[nosyll];
      if(syll.isADetailler()) {
        popupProm.init(nosyll, syll);
        return popupProm;
      }
    }
    if (detectPeriodes==null) return null;
    int noper = trouverNoPeriode(x, fentemp*SEUIL_PROCHE_ANNOTATION);
    if (noper<0) return null;
    popupPeriode.init(noper, detectPeriodes);
    return popupPeriode;
  }
  private int trouverNoSyll(double t) {
    if (t<promsBornesPositions[0]) return 0;
    for (int i = 1; i<promsBornesPositions.length; i++)
      if (t<promsBornesPositions[i])
        return i-1;
    return promsBornesPositions.length-2;
  }
  private int trouverNoPeriode(double t, double seuil) {
    if (t<periodesPositions[0]) {
      if (periodesPositions[0]-t<seuil) return 0;
      return -1;
    }
    for (int i = 1; i<periodesPositions.length; i++)
      if (t<periodesPositions[i]) {
        if (t-periodesPositions[i-1]<periodesPositions[i]-t) {
          if (t-periodesPositions[i-1]<seuil) return i-1;
          return -1;
        }
        if (periodesPositions[i]-t<seuil) return i;
        return -1;
      }
    if (t-periodesPositions[periodesPositions.length-1]<seuil)
      return periodesPositions.length-1;
    return -1;
  }
  private void incrementerIndSelect(int sens) {
    indSelect += sens;
// intervalles n'est pas vide
    if (indSelect<0) indSelect = parcoursIntervalles.length-1;
    if (indSelect>=parcoursIntervalles.length) indSelect = 0;
  }
  private void afficherSelection() {
    effacerSelection();
    double[] intervalle = parcoursIntervalles[indSelect];
    parcoursSelect = new XYLineAnnotation(intervalle[0], 1, intervalle[1], 1,
            LIGNE_EPAISSE, Color.RED);
    addAnnotation(parcoursSelect);
    editeur.rendreVisible(intervalle);
  }
  @Override
  public void effacerSelection() {
    if (parcoursSelect==null) return;
    removeAnnotation(parcoursSelect);
    parcoursSelect = null;
  }
  @Override
  public double[] getTSelects() {
    if (parcoursSelect==null) return null;
    return parcoursIntervalles[indSelect];
  }
  @Override
  public void setSelection(double[] intervalle) {
    // ne répond pas à cette injonction
  }
  void effacerInfosPeriodes() {
    for (int i = 0; i<periodes.size(); i++) removeAnnotation(periodes.get(i));
    periodes.clear();
    periodesPositions = null;
    detectPeriodes = null;
  }
  void afficherInfosPeriodes(double[] coupes, DetectionPeriodes detectPeriodes) {
    effacerInfosPeriodes();
    this.detectPeriodes = detectPeriodes;
    DetectionPeriodes.Candidat[] candidats = detectPeriodes.getCandidats();
    for (int i = 0; i<coupes.length; i++) {
      XYTextAnnotation periode = new XYTextAnnotation("\u2666", coupes[i], 0.7);
      periode.setPaint(COULEUR_PERIODE);
      periode.setFont(FONTE_PERIODE);
      periode.setToolTipText("<html>candidat n°"+(i+1)+"<p>"
              +candidats[i].infosDecision());
      addAnnotation(periode);
      periodes.add(periode);
    }
    periodesPositions = coupes;
  }
  void effacerInfosProeminences() {
    for (int i = 0; i<proms.size(); i++) removeAnnotation(proms.get(i));
    proms.clear();
    for (int i = 0; i<promsBornes.size(); i++)
      removeAnnotation(promsBornes.get(i));
    promsBornes.clear();
    promsBornesPositions = null;
    detectProms = null;
  }
  void afficherInfosProeminences(DetectionProeminences detectProms) {
    effacerInfosProeminences();
    this.detectProms = detectProms;
    DetectionProeminences.Syllabe[] syllabes = detectProms.getSyllabes();
    int nb = syllabes.length;
    promsBornesPositions = new double[nb+1];
    for (int i = 0; i<nb; i++) {
      Syllabe syll = syllabes[i];
      XYTextAnnotation prom = new XYTextAnnotation(syll.infos(),
              0.5*(syll.getBornes()[0]+syll.getBornes()[1]), 0.5);
      prom.setPaint(COULEUR_PROM);
      prom.setFont(FONTE_PROM);
      prom.setToolTipText("<html>syllabe n°"+(i+1)+" : "
              +syllabes[i].infosBulle());
      addAnnotation(prom);
      proms.add(prom);
      promsBornesPositions[i] = syll.getBornes()[0];
      XYLineAnnotation borne = new XYLineAnnotation(promsBornesPositions[i], 0,
              promsBornesPositions[i], 1, LIGNE_SIMPLE, Color.BLACK);
      addAnnotation(borne);
      promsBornes.add(borne);
    }
    promsBornesPositions[nb] = syllabes[nb-1].getBornes()[1];
    XYLineAnnotation borne = new XYLineAnnotation(promsBornesPositions[nb], 0,
            promsBornesPositions[nb], 1, LIGNE_SIMPLE, Color.BLACK);
    addAnnotation(borne);
    promsBornes.add(borne);

  }
  void effacerTout() {
    effacerInfosPeriodes();
    supParcoursF0();
    effacerInfosProeminences();
  }
  void setParcoursI(double[][] intervalles, int i) {
    throw new UnsupportedOperationException("Not yet implemented");
  }

  private static class JPopupPeriode extends JPopupMenu {
    int noCandidat;
    DetectionPeriodes detectPeriodes;
    DetectionPeriodes.Candidat candidat;
    Component contenant;
    private JPopupPeriode(Component contenant) {
      this.contenant = contenant;
    }
    private void init(int no, DetectionPeriodes detect) {
      this.noCandidat = no;
      this.detectPeriodes = detect;
      this.candidat = detect.getCandidats()[no];
      removeAll();
      add(new GAction("Détails...") {
        @Override
        public void executer() {
          JOptionPane.showMessageDialog(contenant, candidat.infosDetails(),
                  "pause n°"+(noCandidat+1), JOptionPane.PLAIN_MESSAGE);
        }
      });
      if (!candidat.decision.isModifiable()) return;
      if (candidat.decision.isPeriode())
        add(new GAction("Supprimer la coupure de période") {
          @Override
          public void executer() {
            detectPeriodes.invaliderCoupure(noCandidat);
          }
        });
      else
        add(new GAction("Introduire une coupure de période") {
          @Override
          public void executer() {
            detectPeriodes.validerCoupure(noCandidat);
          }
        });
      if (candidat.decision.isDoute())
        add(new GAction("Valider la coupure de période") {
          @Override
          public void executer() {
            detectPeriodes.validerCoupure(noCandidat);
          }
        });
    }
  }
 private static class JPopupProm extends JPopupMenu {
    int noSyll;
    DetectionProeminences detectProm;
    Syllabe syll;
    Component contenant;
    private JPopupProm(Component contenant) {
      this.contenant = contenant;
    }
   private void init(int no, Syllabe syl) {
      this.noSyll = no;
      this.syll = syl;
      removeAll();
      add(new GAction("Détails...") {
        @Override
        public void executer() {
          JOptionPane.showMessageDialog(contenant, syll.infosDetails(),
                  "syllabe n°"+(noSyll+1)+": \""+ syll.getMot()+" \"", 
                  JOptionPane.PLAIN_MESSAGE);
        }
      });
    }
  }}
