/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor;

import java.awt.*;
import java.awt.event.*;
import org.jfree.chart.annotations.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import visuAnalor.PanneauEditeur.*;
import visuAnalor.donnees.*;

/**
 *
 * @author Bernard
 */
public class BandeauSon extends XYPlot implements Bandeau {
  private BalayageSon balayage;
  private Son son;
  private XYBoxAnnotation zoneSelect;
  private double t1Select = -1, t2Select = -1;
  private boolean isMovingt1Select = false, isMovingt2Select = false;
  final static Stroke CONTOUR_SELECTION = new BasicStroke(1f);
  final static Paint COULEUR_CONTOUR_SELECTION = Color.BLACK;
  final static Paint FOND_SELECTION = new Color(1f, 0.7f, 0.7f, 0.5f);
  final static Paint FOND = new Color(0.98f, 0.98f, 0.98f);
  final static Font FONTE_STANDARD = Font.decode("Arial-14");
  final static Font FONTE_SELECT = Font.decode("Arial-bold-14");

  public BandeauSon(Crosshair balai) {
    super();
    balayage = new BalayageSon(this, balai);
    son = null;
    NumberAxis axeSon = new NumberAxis();
    axeSon.setTickLabelsVisible(false);
    axeSon.setTickMarksVisible(false);
    axeSon.setLabelAngle(Math.PI/2);
    axeSon.setLabelFont(FONTE_STANDARD);
    axeSon.setLabelPaint(Color.GRAY);
    axeSon.setLabel("Son");
    setRangeAxis(axeSon);
    setDomainGridlinesVisible(false);
    setRangeGridlinesVisible(false);
  }

  public void newSon(Son son) {
    this.son = son;
    if (son!=null) {
      getRangeAxis().setLabelFont(FONTE_STANDARD);
      getRangeAxis().setLabelPaint(Color.BLACK);
      setBackgroundPaint(FOND);
    } else {
      effacerSelection();
      clearAnnotations();
      setBackgroundPaint(null);
      getRangeAxis().setLabelFont(FONTE_STANDARD);
      getRangeAxis().setLabelPaint(Color.GRAY);
    }
  }

  void initSelectionEnCours(double x1, double x2) {
    if (son==null) return;
    isMovingt1Select = isMovingt2Select = false;
    if (zoneSelect!=null) {
      ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
      double seuil = 2*(axe.getUpperBound()-axe.getLowerBound())
              *BandeauTire.SEUIL_PROCHE_POIGNEE;
      if (Math.abs(x1-t1Select)<seuil) isMovingt1Select = true;
      else if (Math.abs(x1-t2Select)<seuil) isMovingt2Select = true;
    }
    dessinerSelectionEnCours(x1, x2);
  }

  void dessinerSelectionEnCours(double x1, double x2) {
    if (son==null) return;
    if (zoneSelect!=null) removeAnnotation(zoneSelect);
    if(isMovingt1Select) x1 = t2Select;
    if(isMovingt2Select) x1 = t1Select;
    zoneSelect = new XYBoxAnnotation(Math.min(x1, x2),
            getRangeAxis().getLowerBound(), Math.max(x1, x2),
            getRangeAxis().getUpperBound());
    addAnnotation(zoneSelect);
  }

  void finSelectionEnCours(double x1, double x2) {
     if(isMovingt1Select) x1 = t2Select;
    if(isMovingt2Select) x1 = t1Select;
    t1Select = Math.min(x1, x2);
    t2Select = Math.max(x1, x2);
    afficherSelection();
  }

  @Override
  public void setSelection(double[] tSelects) {
    t1Select = tSelects[0];
    t2Select = tSelects[1];
    afficherSelection();
  }

  @Override
  public void effacerSelection() {
    if (zoneSelect==null) return;
    removeAnnotation(zoneSelect);
    zoneSelect = null;
    t1Select = t2Select = -1;
  }

  @Override
  public double[] getTSelects() {
    if (son==null||zoneSelect==null) return null;
    return new double[]{t1Select, t2Select};
  }

  private void afficherSelection() {
    if (son==null) return;
    if (zoneSelect!=null) removeAnnotation(zoneSelect);
    zoneSelect = new XYBoxAnnotation(t1Select, getRangeAxis().getLowerBound(),
            t2Select, getRangeAxis().getUpperBound(),
            CONTOUR_SELECTION, COULEUR_CONTOUR_SELECTION, FOND_SELECTION);

    addAnnotation(zoneSelect);
  }

  public void selectionnerTitre(boolean oui) {
    if (oui) getRangeAxis().setLabelFont(FONTE_SELECT);
    else getRangeAxis().setLabelFont(FONTE_STANDARD);
  }

  void lancerSonSelection() {
    if (son.enTrain()) return;
    lancerSon(t1Select, t2Select);
  }

  void lancerSonFenetre() {
    ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
    if (son==null) return;
    if (son.enTrain()) return;
    lancerSon(axe.getLowerBound(), axe.getUpperBound());
  }

  private void lancerSon(double t1, double t2) {
    selectionnerTitre(true);
    balayage.initBalayage(t1, t2);
  }

  void lancerStopperSonComplet() {
    if (son==null) return;
    if (son.enTrain()) {
      balayage.stopBalayage();
      selectionnerTitre(false);
    } else lancerSon(son.getTmin(), son.getTmax());
  }

  private static class BalayageSon implements ActionListener {
    private final static int delai = 20;
    private static javax.swing.Timer timer;
    private BandeauSon bandeau;
    private Crosshair balai;
    private Son son;
    private long duree;
    private double t1;
    private long initTime;

    BalayageSon(BandeauSon bandeau, Crosshair balai) {
      super();
      this.bandeau = bandeau;
      this.balai = balai;
      timer = new javax.swing.Timer(delai, this);
    }

    void initBalayage(double t1, double t2) {
      this.son = bandeau.son;
      this.t1 = t1;
      duree = Math.round((t2-t1)*1000);
      balai.setValue(t1);
      balai.setVisible(true);
      initTime = System.currentTimeMillis();
      timer.start();
      son.demarrer(t1);
    }

    void stopBalayage() {
      timer.stop();
      son.stopper();
      balai.setVisible(false);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      long elapsedTime = System.currentTimeMillis()-initTime;
      if (elapsedTime<duree) {
        double t = t1+elapsedTime/1000.0;
        balai.setValue(t);
      } else {
        stopBalayage();
        bandeau.selectionnerTitre(false);
      }
    }
  }
}
