/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import org.jfree.chart.annotations.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import visuAnalor.PanneauEditeur.*;
import visuAnalor.donnees.*;
import visuAnalor.donnees.Tire.*;
import visuAnalor.util.GMessages;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
public class BandeauTire extends XYPlot implements Bandeau {
  private Tire tire;
  private PanneauEditeur editeur;
  private XYBoxAnnotation zoneSelect;
  private XYLineAnnotation[] poignees;
  private double t1Select = -1, t2Select = -1;
  private int noBorneSelect = -1;
  private double tBorneSelect;
  private BandeauFonctions bandeauFonctions;
  private JPopupAnnot popupAnnot;
  private JPopupTitre popupTitre;
  final static double HAUTEUR_POIGNEES = 0.8;
  private final static double HAUTEUR_TEXTE = HAUTEUR_POIGNEES/2;
  private final static Stroke LIGNE_SIMPLE = new BasicStroke(1f);
  private final static Stroke LIGNE_POIGNEES = new BasicStroke(2f);
  final static Paint FOND_TIRE = new Color(0.98f, 0.98f, 0.98f);
  private final static Stroke CONTOUR_SELECTION = new BasicStroke(1f);
  private final static Paint COULEUR_CONTOUR_SELECTION = Color.BLACK;
  private final static Paint FOND_SELECTION = new Color(1f, 0.7f, 0.7f, 0.3f);
  final static Font FONTE_TITRE_STANDARD = Font.decode("Arial-14");
  final static Font FONTE_TITRE_SELECT = Font.decode("Arial-bold-14");
  public BandeauTire(PanneauEditeur editeur, Tire tire) {
    super();
    this.tire = tire;
    this.editeur = editeur;
    bandeauFonctions = null;
    NumberAxis axeTire = new NumberAxis();
    axeTire.setTickLabelsVisible(false);
    axeTire.setTickMarksVisible(false);
    axeTire.setLabel(tire.getNom());
    axeTire.setLabelAngle(Math.PI/2);
    axeTire.setLabelFont(FONTE_TITRE_STANDARD);
    setRangeAxis(axeTire);
    setDomainGridlinesVisible(false);
    setRangeGridlinesVisible(false);
    setBackgroundPaint(FOND_TIRE);
    dessinerAnnotations();
    popupAnnot = new JPopupAnnot(tire);
    popupTitre = new JPopupTitre(editeur, this);
  }
  void reafficherAnnotations() {
    effacerAnnotations();
    dessinerAnnotations();
    if (bandeauFonctions!=null)
      bandeauFonctions.afficherBornesTire(tire.getBornes());
  }
  void reafficherTitre() {
    getRangeAxis().setLabel(tire.getNom());
  }
  private void effacerAnnotations() {
    clearAnnotations();
    poignees = new XYLineAnnotation[0];
  }
  private void dessinerAnnotations() {
    Paint couleurPoignees = tire.isLibre() ? Color.BLACK : Color.GRAY;
    addAnnotation(new XYBoxAnnotation(editeur.getTmin(), HAUTEUR_POIGNEES,
            editeur.getTmax(), 1, LIGNE_SIMPLE, Color.BLACK, Color.WHITE));
    poignees = new XYLineAnnotation[tire.getBornes().length];
    switch (tire.getType()) {
      case POINT:
        for (int i = 0; i<tire.getBornes().length; i++) {
          double t = tire.getBornes()[i];
          poignees[i] = new XYLineAnnotation(t, 1, t, HAUTEUR_POIGNEES,
                  LIGNE_POIGNEES, couleurPoignees);
          addAnnotation(poignees[i]);
          String mot = tire.getMots()[i];
          XYTextAnnotation m = new XYTextAnnotation(mot, t, HAUTEUR_TEXTE);
          if (!mot.isEmpty())
            m.setToolTipText(String.format("<html>n°%d sur %d items"
                    +"<p>position : %.3f s.", t));
          addAnnotation(m);
        }
        break;
      case INTERVALLE:
        for (int i = 0; i<tire.getBornes().length-1; i++) {
          double t1 = tire.getBornes()[i];
          double t2 = tire.getBornes()[i+1];
          if (i>0) {
            addAnnotation(new XYLineAnnotation(t1, HAUTEUR_POIGNEES, t1, 0,
                    LIGNE_SIMPLE, Color.BLACK));
            poignees[i] = new XYLineAnnotation(t1, 1, t1, HAUTEUR_POIGNEES,
                    LIGNE_POIGNEES, couleurPoignees);
            addAnnotation(poignees[i]);
          }
          String mot = tire.getMots()[i];
          XYTextAnnotation m = new XYTextAnnotation(mot, (t1+t2)/2, HAUTEUR_TEXTE);
          if (!mot.isEmpty())
            m.setToolTipText(String.format("<html>n°%d sur %d items"
                    +"<p>durée : %.0f ms.",
                    i+1, tire.getMots().length, (t2-t1)*1000));
          addAnnotation(m);
        }
        break;
    }
  }
  @Override
  public double[] getTSelects() {
    if (t1Select==t2Select) return null;
    return new double[]{t1Select, t2Select};
  }
  void promouvoir(BandeauFonctions bandeauFonctions) {
    this.bandeauFonctions = bandeauFonctions;
    bandeauFonctions.afficherBornesTire(tire.getBornes());
    getRangeAxis().setLabelFont(FONTE_TITRE_SELECT);

  }
  void destituer() {
    bandeauFonctions.effacerBornesTire();
    bandeauFonctions = null;
    getRangeAxis().setLabelFont(FONTE_TITRE_STANDARD);
  }
  final static double SEUIL_PROCHE_TEXTE = 0.2;
  boolean actionClicPlot(double t, double y, boolean ajout, boolean doubleClic) {
    boolean modifSelection = false;
// impossible (annotation) : if (y>HAUTEUR_POIGNEES&&!ajout) tire.creerBorne(t);
    if (y<HAUTEUR_POIGNEES&&tire.getType()==TypeTire.INTERVALLE) {
      double[] intervalle = tire.localiserIntervalle(t);
      if (intervalle!=null) {
        if (ajout) augmenterSelection(intervalle);
        else setSelection(intervalle);
        modifSelection = true;
      }
    }
    if (y<HAUTEUR_POIGNEES&&doubleClic&&!ajout) {
      ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
      double fentemp = axe.getUpperBound()-axe.getLowerBound();
      int no = tire.trouverNoAnnotation(t, fentemp*SEUIL_PROCHE_TEXTE);
      if (no>=0) tire.editerAnnotation(no);
    }
    return modifSelection;
  }
  final static double SEUIL_PROCHE_POIGNEE = 0.003;
  private double seuilPoignee;
  boolean actionClicAnnot(double t, double y, boolean ajout, boolean doubleClic) {
    boolean modifSelection = false;
    ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
    double fentemp = axe.getUpperBound()-axe.getLowerBound();
    if (y>HAUTEUR_POIGNEES&&!ajout&&doubleClic) {
      seuilPoignee = fentemp*SEUIL_PROCHE_POIGNEE;
      int noborne = tire.trouverNoBorne(t, seuilPoignee);
      if (noborne!=Integer.MIN_VALUE) {
        if (noborne<0) tire.creerBorne(t, -noborne-1);
        else tire.supprimerBorne(noborne);
        if (getTSelects()!=null) {
          effacerSelection();
          modifSelection = true;
        }
      }
    }
    if (y<HAUTEUR_POIGNEES&&tire.getType()==TypeTire.INTERVALLE) {
      double[] intervalle = tire.localiserIntervalle(t);
      if (intervalle!=null) {
        if (t>t1Select&&t<t2Select) {
          if (ajout) reduireSelection(intervalle);
          else effacerSelection();
        } else {
          if (ajout&&zoneSelect!=null) augmenterSelection(intervalle);
          else setSelection(intervalle);
        }
        modifSelection = true;
      }
    }
    if (y<HAUTEUR_POIGNEES&&doubleClic&&!ajout) {
      int no = tire.trouverNoAnnotation(t, fentemp*SEUIL_PROCHE_TEXTE);
      if (no>=0) tire.editerAnnotation(no);
    }

    return modifSelection;
  }
  boolean selectionnerPoignee(double t) {
    boolean modifSelection = false;
    ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
    seuilPoignee = 2*(axe.getUpperBound()-axe.getLowerBound())*SEUIL_PROCHE_POIGNEE;
    noBorneSelect = tire.trouverNoBorne(t, seuilPoignee);
    if (noBorneSelect>=0) {
      tBorneSelect = tire.getBornes()[noBorneSelect];
      if (getTSelects()!=null) {
        effacerSelection();
        modifSelection = true;
      }
    }
    return modifSelection;
  }
  void deplacerPoigneeSelectionnee(double t) {
    if (noBorneSelect<0) return;  //pas de poignée sélectionnée
    tire.deplacerBorne(noBorneSelect, t, seuilPoignee);
  }
  void ajusterPositionPoigneeSelectionnee(double t) {
    if (noBorneSelect<0) return;  //pas de poignée sélectionnée
    if (!tire.isLibre()) t = tire.alignerBorne(noBorneSelect, t);
    tire.prevenirDeplacementBorne(tBorneSelect, t);

  }
  @Override
  public void setSelection(double[] intervalle) {
    if(intervalle==null) {
      effacerSelection();
      return;
    }
    t1Select = intervalle[0];
    t2Select = intervalle[1];
    afficherSelection();
  }
  private void augmenterSelection(double[] intervalle) {
    t1Select = Math.min(t1Select, intervalle[0]);
    t2Select = Math.max(t2Select, intervalle[1]);
    afficherSelection();
  }
  private void reduireSelection(double[] intervalle) {
    t2Select = intervalle[1];
    afficherSelection();
  }
  private void afficherSelection() {
    if (zoneSelect!=null) removeAnnotation(zoneSelect);
    zoneSelect = new XYBoxAnnotation(t1Select, 0, t2Select, HAUTEUR_POIGNEES,
            CONTOUR_SELECTION, COULEUR_CONTOUR_SELECTION, FOND_SELECTION);
    addAnnotation(zoneSelect);
  }
  @Override
  public void effacerSelection() {
    if (zoneSelect==null) return;
    removeAnnotation(zoneSelect);
    zoneSelect = null;
    t1Select = t2Select = -1;
  }
 JPopupMenu getPopupAnnot(double x) {
    ValueAxis axe = ((XYPlot) getParent()).getDomainAxis();
    double fentemp = axe.getUpperBound()-axe.getLowerBound();
    int no = tire.trouverNoAnnotation(x, fentemp*SEUIL_PROCHE_TEXTE);
    if (no<0) return null;
    popupAnnot.putNoAnnot(no);
    return popupAnnot;
  }
  JPopupMenu getPopupTitre() {
    popupTitre.putTitreConversion();
    return popupTitre;
  }

  private static class JPopupAnnot extends JPopupMenu {
    int noAnnot;
    Tire tire;
    JMenuItem itemNo;
    private void putNoAnnot(int no) {
      noAnnot = no;
      itemNo.setText("n°"+(no+1)+" sur "+tire.getMots().length+" items");

    }
    private JPopupAnnot(Tire t) {
      super();
      tire = t;
      add(itemNo = new JMenuItem());
      add(new JSeparator());
      add(new GAction("Editer l'annotation") {
        @Override
        public void executer() {
          tire.editerAnnotation(noAnnot);
        }
      });

    }
  }

  private static class JPopupTitre extends JPopupMenu {
    Tire tire;
    BandeauTire bandeauTire;
    PanneauEditeur editeur;
    JMenuItem conversion;
    private void putTitreConversion() {
      conversion.setText("Convertir (\u2192"+(tire.getType() // \u00BB \uFFEB
              ==TypeTire.INTERVALLE ? "points)" : "intervalles)"));
    }
    private JPopupTitre(PanneauEditeur ed, BandeauTire bt) {
      super();
      bandeauTire = bt;
      tire = bt.tire;
      editeur = ed;
      add(new JMenuItem(tire.getMots().length+" items"));
      add(new JSeparator());
      add(new GAction("Rechercher...") {
        @Override
        public void executer() {
          String mot = JOptionPane.showInputDialog(null,
                  "Chaîne de caractères à rechercher ", 
                  "Recherche d'items dans une tire",
                  JOptionPane.PLAIN_MESSAGE);
          if (mot==null) return;
          if ((mot = mot.trim()).isEmpty()) return;
          double[][] intervalles = tire.rechercher(mot);
          if(intervalles.length==0) GMessages.impossibiliteAction("Pas trouvé "
                  + "d'item contenant \""+mot+ "\"");
          editeur.newParcoursItems(bandeauTire, intervalles);
        }
      });
      add(new GAction("Masquer") {
        @Override
        public void executer() {
          editeur.masquerTire(tire);
        }
      });
      add(new GAction("(Dé)Solidariser...") {
        @Override
        public void executer() {
          editeur.solidariserTire(tire);
        }
      });
      add(new GAction("Renommer...") {
        @Override
        public void executer() {
          tire.renommer();
        }
      });
      add(new GAction("Exporter (tableur)...") {
        @Override
        public void executer() {
          tire.exporterTableur();
        }
      });
      add(new GAction("Exporter (Praat)...") {
        @Override
        public void executer() {
          tire.exporterPraat();
        }
      });
      add(new GAction("Remonter") {
        @Override
        public void executer() {
          editeur.remonterTire(tire);
        }
      });
      add(new GAction("Redescendre") {
        @Override
        public void executer() {
          editeur.redescendreTire(tire);
        }
      });
      add(new GAction("Dupliquer") {
        @Override
        public void executer() {
          editeur.dupliquerTire(tire);
        }
      });
      add(conversion = new JMenuItem(new GAction("Convertir") {
        @Override
        public void executer() {
          tire.convertir();
        }
      }));
      add(new GAction("Supprimer") {
        @Override
        public void executer() {
          tire.supprimer();
        }
      });

    }
  }
}
