/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor;

import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.JPopupMenu;
import org.jfree.chart.*;
import org.jfree.chart.entity.*;
import org.jfree.chart.plot.*;
import org.jfree.ui.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class ChartPanneauEditeur extends ChartPanel implements MouseWheelListener,
        ChartMouseListener {
  private PanneauEditeur editeur;
  private CombinedDomainXYPlot plot;
  private Rectangle2D dataAreaSelect;
  private double x1Pressed, y1Pressed;
  private boolean isJustPressed;
  private int indexBandeauPressed;
  private boolean inDataAreaPressed;
  private boolean isBouton1Pressed;

  public ChartPanneauEditeur(JFreeChart chart, PanneauEditeur edit) {
    super(chart);
    this.editeur = edit;
    this.plot = (CombinedDomainXYPlot) chart.getPlot();
    setMouseZoomable(false);
    setPopupMenu(null);
    addChartMouseListener(this);
  }

  @Override
  public void chartMouseClicked(ChartMouseEvent e) {
//   System.out.println("clicked");
    try {
      if (e.getTrigger().getButton()!=MouseEvent.BUTTON1) return;
      if (e.getEntity().getClass()==PlotEntity.class)
        editeur.mouseClickedPlot(indexBandeauPressed, x1Pressed, y1Pressed,
                e.getTrigger().isShiftDown(), e.getTrigger().getClickCount()==2);
      else if (e.getEntity().getClass()==XYAnnotationEntity.class)
        editeur.mouseClickedAnnotation(indexBandeauPressed, x1Pressed, y1Pressed,
                e.getTrigger().isShiftDown(),  e.getTrigger().getClickCount()==2);
      else if (e.getEntity().getClass()==AxisEntity.class) {
        editeur.mouseClickedAxis(indexBandeauPressed, e.getTrigger().isShiftDown());
       // System.out.println(e.getTrigger().getX());
      }
      else if (e.getEntity().getClass()==XYItemEntity.class)
        editeur.mouseClickedPoint(indexBandeauPressed, x1Pressed);
    } catch (Throwable ex) {
      GMessages.erreurFatale(ex);
    }
// System.out.println(e.getEntity().getClass());
  }

  @Override
  public void chartMouseMoved(ChartMouseEvent e) {
  }

  @Override
  public void mousePressed(MouseEvent e) {
    try {
      Point2D pt = translateScreenToJava2D(e.getPoint());
      localiserMousePressed(pt);
      isBouton1Pressed = e.getButton()==MouseEvent.BUTTON1;
      if (indexBandeauPressed<0) return;
      if (e.isPopupTrigger()) {
        showPopup(e);
        return;
      }
      if (!isBouton1Pressed) return;
      if (!inDataAreaPressed) return;
      isJustPressed = true;
    } catch (Throwable ex) {
      GMessages.erreurFatale(ex);
    }
//    System.out.println(x1Pressed);
  }

  @Override
  public void mouseReleased(MouseEvent e) {
    try {
      if (e.isPopupTrigger()) {
        showPopup(e);
        return;
      }
      if (isJustPressed) return;  // traitement par chartMouseClicked
      if (!isBouton1Pressed) return;
      if (!editeur.isDraggable(indexBandeauPressed, inDataAreaPressed,
              x1Pressed, y1Pressed)) return;
      Point2D pt = translateScreenToJava2D(e.getPoint());
      double x2 = localiserX(pt.getX());
      editeur.finMouseDragged(indexBandeauPressed, x1Pressed, x2);
    } catch (Throwable ex) {
      GMessages.erreurFatale(ex);
    }
  }

  @Override
  public void mouseDragged(MouseEvent e) {
    try {
      if (!isBouton1Pressed) return;
       if (!editeur.isDraggable(indexBandeauPressed, inDataAreaPressed,
              x1Pressed, y1Pressed)) return;
     Point2D pt = translateScreenToJava2D(e.getPoint());
      double x2 = localiserX(pt.getX());
      if (isJustPressed) {
        editeur.initMouseDragged(indexBandeauPressed, x1Pressed, x2);
        isJustPressed = false;
      } else editeur.mouseDragged(indexBandeauPressed, x1Pressed, x2);
    } catch (Throwable ex) {
      GMessages.erreurFatale(ex);
    }
  }

  @Override
  public void mouseMoved(MouseEvent e) {
  }

  @Override
  public void mouseWheelMoved(MouseWheelEvent e) {
  }

  private void localiserMousePressed(Point2D pt) {
    PlotRenderingInfo info = getChartRenderingInfo().getPlotInfo();

    int subplotCount = info.getSubplotCount();
    double y = pt.getY();
    for (int i = 0; i<subplotCount; i++) {
      PlotRenderingInfo infosub = info.getSubplotInfo(i);
      Rectangle2D area = infosub.getDataArea();
      if (y>=area.getY()&&y<area.getY()+area.getHeight()) {
        indexBandeauPressed = i;
        inDataAreaPressed = pt.getX()>=area.getX()
                &&pt.getX()<area.getX()+area.getWidth();
        if (inDataAreaPressed) {
          dataAreaSelect = area;
          x1Pressed = localiserX(pt.getX());
          y1Pressed = localiserY(pt.getY());
        }
        return;
      }
    }
    indexBandeauPressed = -1;
  }

  private double localiserX(double x2D) {
    return plot.getDomainAxis().java2DToValue(x2D, dataAreaSelect,
            RectangleEdge.TOP);
  }

  private double localiserY(double y2D) {
    return ((XYPlot) plot.getSubplots().get(indexBandeauPressed)).getRangeAxis().
            java2DToValue(y2D, dataAreaSelect, RectangleEdge.LEFT);
  }

  private void showPopup(MouseEvent e) {
    JPopupMenu popup;
    if (inDataAreaPressed)
      popup = editeur.getPopupMenuData(indexBandeauPressed, x1Pressed);
    else
      popup = editeur.getPopupMenuAxis(indexBandeauPressed);
    if (popup==null) return;
    popup.show(e.getComponent(), e.getX(), e.getY());
  }
}
