/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.jfree.chart.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.panel.*;
import org.jfree.chart.plot.*;
import org.jfree.ui.RectangleEdge;
import visuAnalor.analyses.DetectionPeriodes;
import visuAnalor.analyses.DetectionProeminences;
import visuAnalor.donnees.MessageDonnes.*;
import visuAnalor.donnees.*;
import visuAnalor.donnees.Donnees.*;
import visuAnalor.util.GMessages;
import visuAnalor.util.GMessages.*;
import visuAnalor.util.GVisu;

/**
 *
 * @author Bernard
 */
public class PanneauEditeur extends JPanel
        implements DonneesListener, ChangeListener {
  public static interface Bandeau {
    void setSelection(double[] intervalle);
    void effacerSelection();
    double[] getTSelects();
  }
  private BandeauSon bandeauSon;
  private BandeauFonctions bandeauFonctions;
  private BandeauResultats bandeauResultats;
  private JPopupMenu popupFlechesResultats;
  private BandeauTire[] bandeauTires;
  private HashMap<Tire, Integer> nosBandeauTires;
  private int noBandeauTirePrincipal;
  private BandeauTire bandeauTireParcours;
  private ChartPanneauEditeur graphique;
  private JScrollBar glissiere;
  private CombinedDomainXYPlot plot;
  private Donnees donnees;
  private double tmin, tmax;
  static final double TAILLE_FENETRE_TEMPORELLE_INITIALE = 5;
  private double tailleFenetre = TAILLE_FENETRE_TEMPORELLE_INITIALE;
  public PanneauEditeur(Donnees donnees) {
    super(new BorderLayout());
    this.donnees = donnees;
    plot = new CombinedDomainXYPlot(new NumberAxis());
    plot.setDomainAxisLocation(AxisLocation.TOP_OR_LEFT);
    AxisSpace EspaceBoutons = new AxisSpace();
    EspaceBoutons.ensureAtLeast(200, RectangleEdge.LEFT);
    plot.setFixedRangeAxisSpace(EspaceBoutons);
    tmin = Double.POSITIVE_INFINITY;
    tmax = Double.NEGATIVE_INFINITY;
    recalculerTMinMax();
    Crosshair balai = new Crosshair();
    bandeauSon = new BandeauSon(balai);
    plot.add(bandeauSon);
    bandeauFonctions = new BandeauFonctions(this);
    plot.add(bandeauFonctions);
    bandeauResultats = new BandeauResultats(this);
    plot.add(bandeauResultats);
    popupFlechesResultats = new JPopupMenu();
    popupFlechesResultats.add(new GAction("Précédent") {
      @Override
      public void executer() {
        resultatPrecedent();
      }
    });
    popupFlechesResultats.add(new GAction("Suivant") {
      @Override
      public void executer() {
        resultatSuivant();
      }
    });
    bandeauTires = new BandeauTire[0];
    nosBandeauTires = new HashMap<Tire, Integer>();
    noBandeauTirePrincipal = -1;
    ajusterTailleBandeaux();
    plot.setGap(0);
    graphique = new ChartPanneauEditeur(new JFreeChart(plot), this);
    CrosshairOverlay c = new CrosshairOverlay();
    c.addDomainCrosshair(balai);
    graphique.setDisplayToolTips(true);
    graphique.addOverlay(c);
    add(graphique);
    glissiere = new JScrollBar(JScrollBar.HORIZONTAL);
    Box fenetreBar = Box.createHorizontalBox();
    JButton zoomplus = new JButton(new GAction("+") {
      @Override
      public void executer() {
        zoomer(1);
      }
    });
    zoomplus.setRequestFocusEnabled(false);
    fenetreBar.add(zoomplus);
    JButton zoommoins = new JButton(new GAction("-") {
      @Override
      public void executer() {
        zoomer(-1);
      }
    });
    zoommoins.setRequestFocusEnabled(false);
    fenetreBar.add(zoommoins);
    fenetreBar.add(glissiere);
    add(fenetreBar, BorderLayout.SOUTH);
    glissiere.getModel().addChangeListener(this);
    donnees.addEventListener(this);
    setFocusable(true);
    addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        try {
          int key = e.getKeyCode();
          //System.out.println(key);
          if (key==KeyEvent.VK_KP_LEFT||key==KeyEvent.VK_LEFT
                  ||(key==KeyEvent.VK_SPACE&&e.isShiftDown())) {
            resultatPrecedent();
          } else if (key==KeyEvent.VK_KP_RIGHT||key==KeyEvent.VK_RIGHT
                  ||key==KeyEvent.VK_SPACE) {
            resultatSuivant();
          }
        } catch (Throwable ex) {
          GMessages.erreurFatale(ex);
        }
      }
    });
  }
  public double getTmin() {
    return tmin;
  }
  public double getTmax() {
    return tmax;
  }
  void saisirTiresAVoir() {
    Tire[] tires = donnees.getTires();
    if (tires.length==0) return;
    VisuTiresAVoir fenetreSaisie =
            new VisuTiresAVoir((JFrame) getTopLevelAncestor(), tires, this);
    fenetreSaisie.setVisible(true);
  }
  private boolean isTireAffichee(Tire tire) {
    return nosBandeauTires.get(tire)!=null;
  }
  private void recalculerTMinMax() {
    double[] tminmax = donnees.calculerTMinMax();
    if (tminmax==null) {
      plot.getDomainAxis().setTickLabelsVisible(false);
      plot.getDomainAxis().setTickMarksVisible(false);
      plot.getDomainAxis().setLowerBound(0);
      plot.getDomainAxis().setUpperBound(1);

    } else {
      tmin = tminmax[0];
      tmax = tminmax[1];
      plot.getDomainAxis().setTickLabelsVisible(true);
      plot.getDomainAxis().setTickMarksVisible(true);
      plot.getDomainAxis().setLowerBound(tmin);
      plot.getDomainAxis().setUpperBound(tmax);
    }
  }
  private void ajusterFenetreTemporelle(double t01, double t02) {
    double t1 = Math.max(tmin, t01);
    double t2 = Math.min(tmax, t02);
    tailleFenetre = t2-t1;
    plot.getDomainAxis().setLowerBound(t1);
    plot.getDomainAxis().setUpperBound(t2);
    glissiere.setValues(Math.round((float) t1*1000),
            Math.round((float) (tailleFenetre)*1000),
            Math.round((float) tmin*1000), Math.round((float) tmax*1000));
    glissiere.setBlockIncrement(Math.round((float) (tailleFenetre)*100));
    glissiere.setUnitIncrement(Math.round((float) (tailleFenetre)*10));
  }
  private void ajusterTailleBandeaux() {
    int pdsSon = bandeauTires.length<=6 ? 1 : 2;
    bandeauSon.setWeight(pdsSon);
    bandeauResultats.setWeight(2);
    for (BandeauTire bandeau : bandeauTires) bandeau.setWeight(2);
    bandeauFonctions.setWeight(Math.max(10, 2*bandeauTires.length-(2+pdsSon)));
    plot.setGap(0);
  }
  public void rendreVisible(double[] bornes) {
    ValueAxis axe = plot.getDomainAxis();
    double t1 = axe.getLowerBound();
    double t2 = axe.getUpperBound();
    double b1 = bornes[0];
    double b2 = bornes[1];
    double newt1;
    if (t1>b2||t2<b1||(b2-b1<t2-t1&&(b1<t1||b2>t2))) {
      newt1 = Math.min(Math.max(tmin, (b1+b2+t1-t2)/2), tmax+t1-t2);
      axe.setLowerBound(newt1);
      axe.setUpperBound(newt1+t2-t1);
      glissiere.setValue(Math.round((float) newt1*1000));
    }
  }
  public void zoomer(int sens) {
    double t1 = plot.getDomainAxis().getLowerBound();
    double t2 = plot.getDomainAxis().getUpperBound();
    if (sens>0) ajusterFenetreTemporelle(0.9*t1+0.1*t2, 0.1*t1+0.9*t2);
    else ajusterFenetreTemporelle(1.1*t1-0.1*t2, 1.1*t2-0.1*t1);
  }
  void saisirFenetreTemporelle() {
    double[] tminmax = donnees.calculerTMinMax();
    if (tminmax==null) return;
    VisuFenetreTemporelle fenetreSaisie =
            new VisuFenetreTemporelle((JFrame) getTopLevelAncestor(), tminmax, this);
    fenetreSaisie.setVisible(true);

  }
  @Override
  public void stateChanged(ChangeEvent e) {  // glissière
    if (e.getSource()==glissiere.getModel()) {
      double t1 = glissiere.getValue()/1000.0;
      double t2 = t1+glissiere.getVisibleAmount()/1000.0;
      plot.getDomainAxis().setLowerBound(t1);
      plot.getDomainAxis().setUpperBound(t2);
    }
  }
  @Override
  public void traiterEventDonnees(MessageDonnes evt) {
    Integer noTire;
    switch (evt.getType()) {
      case CLEAR_DONNEES:
        initPasDeDonnees();
        return;
      case NEW_DONNEES:
        initNewDonnees();
        tailleFenetre = TAILLE_FENETRE_TEMPORELLE_INITIALE;
        ajusterFenetreTemporelle(tmin, tmin+tailleFenetre);
        return;
      case MODIF_F0:
        bandeauFonctions.afficherF0();
        return;
      case MODIF_TIRE:
        noTire = nosBandeauTires.get((Tire) evt.getSource());
        if (noTire!=null) bandeauTires[noTire].reafficherAnnotations();
        return;
      case MODIF_NOM_TIRE:
        noTire = nosBandeauTires.get((Tire) evt.getSource());
        if (noTire!=null) bandeauTires[noTire].reafficherTitre();
        return;
      case NEW_SON:
        initNewSon();
        ajusterFenetreTemporelle(tmin, tmin+tailleFenetre);
        return;
      case NEW_F0:
        initNewF0();
        tailleFenetre = TAILLE_FENETRE_TEMPORELLE_INITIALE;
        ajusterFenetreTemporelle(tmin, tmin+tailleFenetre);
        return;
      case NEW_INTENSITE:
        initNewIntensite();
        ajusterFenetreTemporelle(tmin, tmin+tailleFenetre);
        return;
      case ADD_TIRES:
        addNewTires();
        ajusterFenetreTemporelle(tmin, tmin+tailleFenetre);
        return;
      case SUP_TIRE:
        supprimerTire((Tire) evt.getSource());
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  final static int INDEX_SON = 0;
  final static int INDEX_FONCTIONS = 1;
  final static int INDEX_RESULTATS = 2;
  final static int INDEX_TIRE_0 = 3;
  public void newParcoursItems(BandeauTire bandeau, double[][] intervalles) {
    bandeauResultats.setParcoursF0(intervalles, true);
    repercuterSelection(bandeauResultats);
    bandeau.setSelection(intervalles[0]);
    bandeauTireParcours = bandeau;
    ((VisuMain) getTopLevelAncestor()).premierPlan();
  }
  public void newParcoursF0(double[][] intervalles) {
    bandeauResultats.setParcoursF0(intervalles, true);
    repercuterSelection(bandeauResultats);
    ((VisuMain) getTopLevelAncestor()).premierPlan();
  }
  public void newParcoursI(double[][] intervalles) {
    bandeauResultats.setParcoursF0(intervalles, true);
    repercuterSelection(bandeauResultats);
    ((VisuMain) getTopLevelAncestor()).premierPlan();
  }
  private void resultatPrecedent() {
    if (bandeauResultats.deplacerSelection(-1)) {
      repercuterSelection(bandeauResultats);
      if (bandeauTireParcours!=null)
        bandeauTireParcours.setSelection(bandeauResultats.getSelectionParcours());
    }
  }
  private void resultatSuivant() {
    if (bandeauResultats.deplacerSelection(1)) {
      repercuterSelection(bandeauResultats);
      if (bandeauTireParcours!=null)
        bandeauTireParcours.setSelection(bandeauResultats.getSelectionParcours());
    }
  }
  private void repercuterSelection(Bandeau bandeau) {
    double[] tselects = bandeau.getTSelects();
    if (tselects==null) {
      effacerSelectionTous();
      return;
    }
    if (bandeau!=bandeauFonctions) bandeauFonctions.setSelection(tselects);
    if (bandeau!=bandeauResultats) bandeauResultats.effacerSelection();
    for (BandeauTire b : bandeauTires) if (bandeau!=b) b.effacerSelection();
    if (bandeau!=bandeauSon) bandeauSon.setSelection(tselects);

  }
  boolean isDraggable(int index, boolean inDataArea, double x, double y) {
    return inDataArea&&(index==INDEX_SON||index==INDEX_FONCTIONS
            ||(index>=INDEX_TIRE_0&&y>BandeauTire.HAUTEUR_POIGNEES));
  }
  void initMouseDragged(int indexPressed, double x1, double x2) {
    switch (indexPressed) {
      case INDEX_SON:
        bandeauSon.initSelectionEnCours(x1, x2);
        return;
      case INDEX_FONCTIONS:
        bandeauFonctions.dessinerSelectionEnCours(x1, x2);
        return;
      case INDEX_RESULTATS:
        return;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return;
        BandeauTire bandeau = bandeauTires[indBandeau];
        boolean modifSelection = bandeau.selectionnerPoignee(x1);
        if (modifSelection) repercuterSelection(bandeau);
        bandeau.deplacerPoigneeSelectionnee(x2);
    }
  }
  void finMouseDragged(int indexPressed, double x1, double x2) {
    double[] tSelects = new double[]{Math.min(x1, x2), Math.max(x1, x2)};

    switch (indexPressed) {
      case INDEX_SON:
        bandeauSon.finSelectionEnCours(x1, x2);
        repercuterSelection(bandeauSon);
        return;
      case INDEX_FONCTIONS:
        bandeauFonctions.setSelection(tSelects);
        repercuterSelection(bandeauFonctions);
        return;
      case INDEX_RESULTATS:
        return;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return;
        BandeauTire bandeau = bandeauTires[indBandeau];
        bandeau.ajusterPositionPoigneeSelectionnee(x2);
    }
  }
  void mouseDragged(int indexPressed, double x1, double x2) {
    switch (indexPressed) {
      case INDEX_SON:
        bandeauSon.dessinerSelectionEnCours(x1, x2);
        return;
      case INDEX_FONCTIONS:
        bandeauFonctions.dessinerSelectionEnCours(x1, x2);
        return;
      case INDEX_RESULTATS:
        return;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return;
        BandeauTire bandeau = bandeauTires[indBandeau];
        bandeau.deplacerPoigneeSelectionnee(x2);
    }
  }
  void mouseClickedPlot(int indexPressed, double x1Pressed, double y1Pressed,
          boolean toucheShift, boolean doubleClic) {
    switch (indexPressed) {
      case INDEX_SON:
        bandeauSon.lancerSonFenetre();
        return;
      case INDEX_FONCTIONS:
        bandeauFonctions.chercherSelectionPortion(x1Pressed, y1Pressed);
        repercuterSelection(bandeauFonctions);
        return;
      case INDEX_RESULTATS:
        if (bandeauResultats.actionClicPlot(x1Pressed, y1Pressed,
                toucheShift, doubleClic)) repercuterSelection(bandeauResultats);
        return;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return;
        BandeauTire bandeau = bandeauTires[indBandeau];
        boolean modifSelection = bandeau.actionClicPlot(x1Pressed, y1Pressed,
                toucheShift, doubleClic);
        if (modifSelection) repercuterSelection(bandeau);
    }
  }
  void mouseClickedAnnotation(int indexPressed, double x1Pressed, double y1Pressed,
          boolean toucheShift, boolean doubleClic) {
    switch (indexPressed) {
      case INDEX_SON:
        bandeauSon.lancerSonSelection();
        return;
      case INDEX_FONCTIONS:
        return;
      case INDEX_RESULTATS:
        if (bandeauResultats.actionClicAnnot(x1Pressed, y1Pressed,
                toucheShift, doubleClic)) repercuterSelection(bandeauResultats);
        return;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return;
        BandeauTire bandeau = bandeauTires[indBandeau];
        boolean modifSelection = bandeau.actionClicAnnot(x1Pressed, y1Pressed,
                toucheShift, doubleClic);
        if (modifSelection) repercuterSelection(bandeau);

    }
  }
  void mouseClickedAxis(int indexPressed, boolean isShift) {
    switch (indexPressed) {
      case INDEX_SON:
        bandeauSon.lancerStopperSonComplet();
        return;
      case INDEX_FONCTIONS:
        return;
      case INDEX_RESULTATS:
        if (isShift) resultatPrecedent();
        else resultatSuivant();
        return;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return;
        if (noBandeauTirePrincipal==indBandeau) {
          bandeauTires[indBandeau].destituer();
          noBandeauTirePrincipal = -1;
        } else {
          if (noBandeauTirePrincipal>=0)
            bandeauTires[noBandeauTirePrincipal].destituer();
          bandeauTires[indBandeau].promouvoir(bandeauFonctions);
          noBandeauTirePrincipal = indBandeau;
        }
    }
  }
  void mouseClickedPoint(int indexPressed, double x1Pressed) {
    switch (indexPressed) {
      case INDEX_SON:
        return;
      case INDEX_FONCTIONS:
        bandeauFonctions.afficherSelectionPortion(x1Pressed);
        repercuterSelection(bandeauFonctions);
        return;
      case INDEX_RESULTATS:
        return;
      default:  // INDEX TIRE
        return;
    }
  }
  JPopupMenu getPopupMenuData(int indexPressed, double x1Pressed) {
    switch (indexPressed) {
      case INDEX_SON:
        return null;
      case INDEX_FONCTIONS:
        return bandeauFonctions.getPopup();
      case INDEX_RESULTATS:
        return bandeauResultats.getPopup(x1Pressed);
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return null;
        BandeauTire bandeau = bandeauTires[indBandeau];
        return bandeau.getPopupAnnot(x1Pressed);
    }
  }
  JPopupMenu getPopupMenuAxis(int indexPressed) {
    switch (indexPressed) {
      case INDEX_SON:
        return null;
      case INDEX_FONCTIONS:
        return null;
      case INDEX_RESULTATS:
        return popupFlechesResultats;
      default:  // INDEX TIRE
        int indBandeau = indexPressed-INDEX_TIRE_0;
        if (indBandeau<0||indBandeau>=bandeauTires.length) return null;
        BandeauTire bandeau = bandeauTires[indBandeau];
        return bandeau.getPopupTitre();
    }
  }
  private void effacerSelectionTous() {
    bandeauSon.effacerSelection();
    bandeauFonctions.effacerSelection();
    bandeauResultats.effacerSelection();
    for (BandeauTire bandeauTire : bandeauTires) bandeauTire.effacerSelection();
  }
  private void initPasDeDonnees() {
    recalculerTMinMax();
    bandeauSon.newSon(null);
    bandeauFonctions.newF0(null);
    bandeauFonctions.newIntensite(null);
    bandeauResultats.effacerTout();
    for (int i = 0; i<bandeauTires.length; i++) plot.remove(bandeauTires[i]);
    bandeauTires = new BandeauTire[0];
    nosBandeauTires.clear();
    noBandeauTirePrincipal = -1;
    ajusterTailleBandeaux();
  }
  private void initNewDonnees() {
    if (donnees.getSon()!=null) initNewSon();
    if (donnees.getCourbeF0()!=null) initNewF0();
    if (donnees.getCourbeIntensite()!=null) initNewIntensite();
    if (donnees.getTires().length>0) addNewTires();

  }
  private void initNewSon() {
    Son son = donnees.getSon();
    if (son==null) {
      bandeauSon.newSon(null);
      return;
    }
    recalculerTMinMax();
    bandeauSon.newSon(son);
    double[] tSelects = chercherSelection();
    if (tSelects!=null) bandeauSon.setSelection(tSelects);
  }
  private double[] chercherSelection() {
    double[] tSelects = bandeauFonctions.getTSelects();
    if (tSelects==null) tSelects = bandeauResultats.getTSelects();
    if (tSelects==null) for (BandeauTire bandeau : bandeauTires)
        if ((tSelects = bandeau.getTSelects())!=null) break;
    return tSelects;
  }
  private void initNewF0() {
    CourbeF0 f0 = donnees.getCourbeF0();
    recalculerTMinMax();
    bandeauFonctions.newF0(f0);
    bandeauResultats.effacerTout();
  }
  void effAffF0() {
    bandeauFonctions.effAffF0();
  }
  private void initNewIntensite() {
    CourbeIntensite I = donnees.getCourbeIntensite();
    recalculerTMinMax();
    bandeauFonctions.newIntensite(I);
  }
  void effAffI() {
    bandeauFonctions.effAffI();
  }
//  private XYPlot bandeauSelect;
//  private XYBoxAnnotation zoneSelect, zoneSonSelect;
  private void addNewTires() {
    Tire[] tires = donnees.getTires();
    recalculerTMinMax();
    BandeauTire[] newbandeautires = new BandeauTire[tires.length];
    System.arraycopy(bandeauTires, 0, newbandeautires, 0, bandeauTires.length);
    for (int i = bandeauTires.length; i<tires.length; i++) {
      newbandeautires[i] = new BandeauTire(this, tires[i]);
      nosBandeauTires.put(tires[i], i);
      plot.add(newbandeautires[i]);
    }
    bandeauTires = newbandeautires;
    ajusterTailleBandeaux();
  }
  private void setTiresAVoir(boolean[] isAVoir) {
    Tire[] tires = donnees.getTires();
    boolean[] isVu = new boolean[tires.length];
    Tire tirePrincipale = null;
    for (int i = 0; i<tires.length; i++) {
      Integer notire = nosBandeauTires.get(tires[i]);
      isVu[i] = notire!=null;
      if (notire!=null&&notire==noBandeauTirePrincipal)
        tirePrincipale = tires[i];
    }
    int no = 0;
    for (boolean b : isAVoir) if (b) no++;
    for (BandeauTire bandeau : bandeauTires) plot.remove(bandeau);
    BandeauTire[] newbandeauTires = new BandeauTire[no];
    HashMap<Tire, Integer> newnosBandeau = new HashMap<Tire, Integer>();
    no = 0;
    for (int i = 0; i<tires.length; i++) if (isAVoir[i]) {
        if (!isVu[i]) newbandeauTires[no] = new BandeauTire(this, tires[i]);
        else newbandeauTires[no] = bandeauTires[nosBandeauTires.get(tires[i])];
        plot.add(newbandeauTires[no]);
        newnosBandeau.put(tires[i], no++);
      }
    nosBandeauTires = newnosBandeau;
    bandeauTires = newbandeauTires;
    if (tirePrincipale==null||!nosBandeauTires.containsKey(tirePrincipale))
      noBandeauTirePrincipal = -1;
    else noBandeauTirePrincipal = nosBandeauTires.get(tirePrincipale);
    ajusterTailleBandeaux();
  }
  void masquerTire(Tire tire) {
    Integer notire = nosBandeauTires.get(tire);
    if (notire==null) return;
    if (notire==noBandeauTirePrincipal) noBandeauTirePrincipal = -1;
    if (bandeauTireParcours==(bandeauTires[notire])) bandeauTireParcours = null;
    BandeauTire[] newbandeauTires = new BandeauTire[bandeauTires.length-1];
    System.arraycopy(bandeauTires, 0, newbandeauTires, 0, notire);
    System.arraycopy(bandeauTires, notire+1, newbandeauTires, notire,
            newbandeauTires.length-notire);
    nosBandeauTires.remove(tire);
    for (Tire tir : nosBandeauTires.keySet()) {
      Integer no = nosBandeauTires.get(tir);
      if (no>notire) nosBandeauTires.put(tir, no-1);
    }
    plot.remove(bandeauTires[notire]);
    bandeauTires = newbandeauTires;
    ajusterTailleBandeaux();
  }
  void solidariserTire(Tire tire) {
    Tire[] tires = donnees.getTires();
    Integer notire = nosBandeauTires.get(tire);
    String[] nomTires = new String[tires.length];
    nomTires[0] = GVisu.styleDef("aucune");
    String nom0 = tire.isLibre() ? nomTires[0] : tire.getTireGuide().getNom();
    int i = 1;
    for (Tire t : tires) if (t!=tire) nomTires[i++] = t.getNom();
    String nom1 = (String) JOptionPane.showInputDialog(this,
            "Tire avec laquelle aligner la tire \""+tire.getNom()+"\" : ",
            "Solidariser une tire", JOptionPane.PLAIN_MESSAGE, null, nomTires,
            nom0);
    if (nom1==null) return;
    Tire tire1 = donnees.trouverTireParNom(nom1);
    if (tire1==null) tire.liberer();
    else tire.solidariser(tire1);
  }
  void remonterTire(Tire tire) {
    Integer notire = nosBandeauTires.get(tire);
    if (notire==null||notire==0) return;
    for (Tire t : nosBandeauTires.keySet())
      if (nosBandeauTires.get(t)==notire-1) {
        donnees.placerTireAvant(tire, t);
        nosBandeauTires.put(tire, notire-1);
        nosBandeauTires.put(t, notire);
        break;
      }
    for (int no = notire-1; no<bandeauTires.length; no++)
      plot.remove(bandeauTires[no]);
    BandeauTire b = bandeauTires[notire];
    bandeauTires[notire] = bandeauTires[notire-1];
    bandeauTires[notire-1] = b;
    for (int no = notire-1; no<bandeauTires.length; no++)
      plot.add(bandeauTires[no]);
    ajusterTailleBandeaux();
  }
  void redescendreTire(Tire tire) {
    Integer notire = nosBandeauTires.get(tire);
    if (notire==null||notire==bandeauTires.length-1) return;
    for (Tire t : nosBandeauTires.keySet())
      if (nosBandeauTires.get(t)==notire+1) {
        donnees.placerTireAvant(t, tire);
        nosBandeauTires.put(tire, notire+1);
        nosBandeauTires.put(t, notire);
        break;
      }
    for (int no = notire; no<bandeauTires.length; no++)
      plot.remove(bandeauTires[no]);
    BandeauTire b = bandeauTires[notire];
    bandeauTires[notire] = bandeauTires[notire+1];
    bandeauTires[notire+1] = b;
    for (int no = notire; no<bandeauTires.length; no++)
      plot.add(bandeauTires[no]);
    ajusterTailleBandeaux();
  }
  private void supprimerTire(Tire tire) {
    Integer notire = nosBandeauTires.get(tire);
    if (notire==null) return;
    if (!GMessages.confirmationAction(
            "Etes-vous sûr de vouloir supprimer définitivement la tire \""
            +tire.getNom()+"\" ?")) return;
    nosBandeauTires.remove(tire);
    for (Tire tir : nosBandeauTires.keySet()) {
      Integer no = nosBandeauTires.get(tir);
      if (no>notire) nosBandeauTires.put(tir, no-1);
    }
    if (noBandeauTirePrincipal==notire) noBandeauTirePrincipal = -1;
    if (bandeauTireParcours==(bandeauTires[notire])) bandeauTireParcours = null;
    plot.remove(bandeauTires[notire]);
    BandeauTire[] newbandeauTires = new BandeauTire[bandeauTires.length-1];
    System.arraycopy(bandeauTires, 0, newbandeauTires, 0, notire);
    System.arraycopy(bandeauTires, notire+1, newbandeauTires, notire,
            newbandeauTires.length-notire);
    bandeauTires = newbandeauTires;
    ajusterTailleBandeaux();
  }
  void dupliquerTire(Tire tire) {
    donnees.dupliquerTire(tire);
  }
  public void newF0Lissee(CourbeF0 f0lissee) {
    if (f0lissee==null) bandeauFonctions.effacerLissage();
    else bandeauFonctions.afficherLissage(f0lissee);
  }
  public void newProsogramme(Prosogramme prosogramme) {
    if (prosogramme==null) bandeauFonctions.effacerProsogramme();
    else bandeauFonctions.afficherProsogramme(prosogramme);
  }
  double[][] getProsogramme() {
    return bandeauFonctions.getProsogramme();
  }
  public void effacerPeriodes() {
    bandeauFonctions.effacerPeriodes();
    bandeauResultats.effacerInfosPeriodes();
    bandeauResultats.supParcoursF0();
  }
  public void newPeriodes(DetectionPeriodes detectPeriodes, boolean init) {
    DetectionPeriodes.Candidat[] candidats = detectPeriodes.getCandidats();
    if (candidats.length==0) {
      effacerPeriodes();
      return;
    }
    double[] coupes = new double[candidats.length-1];
    double[][] intervalles = new double[candidats.length-1][2];
    int[] decisions = new int[candidats.length-1];
    for (int i = 0; i<coupes.length; i++) {
      intervalles[i][0] = candidats[i].bornes[1];
      intervalles[i][1] = candidats[i+1].bornes[0];
      coupes[i] = (intervalles[i][0]+intervalles[i][1])/2;
      decisions[i] = candidats[i].decision.isDoute() ? 1
              : candidats[i].decision.isPeriode() ? 2 : 0;
    }
    bandeauFonctions.afficherPeriodes(coupes, decisions);
    bandeauResultats.afficherInfosPeriodes(coupes, detectPeriodes);
    bandeauResultats.setParcoursF0(intervalles, init);

  }
  public void effacerProeminences() {
    bandeauResultats.effacerInfosProeminences();
  }
  public void newProeminences(DetectionProeminences detectProms, boolean init) {
    DetectionProeminences.Syllabe[] syllabes = detectProms.getSyllabes();
    if (syllabes.length==0) {
      effacerProeminences();
      return;
    }
    bandeauResultats.afficherInfosProeminences(detectProms);
  }
  public void exporterPNG(File f) throws IOException {
    //   org.jfree.chart.encoders.SunPNGEncoderAdapter aa;
    ChartUtilities.saveChartAsPNG(f, graphique.getChart(), graphique.getWidth(),
            graphique.getHeight());
  }

  static private class VisuTiresAVoir extends JDialog {
    private JCheckBox[] saisieTires;
    private PanneauEditeur editeur;
    private VisuTiresAVoir(JFrame fenetre, Tire[] tires, PanneauEditeur ed) {
      super(fenetre, "Choix des tires à afficher", true);
      this.editeur = ed;
      getContentPane().setLayout(new BorderLayout());
      JPanel panneauSaisie = new JPanel(new GridLayout(0, 1));
      panneauSaisie.add(new JLabel("Tires à afficher :"));
      saisieTires = new JCheckBox[tires.length];
      for (int i = 0; i<tires.length; i++) {
        saisieTires[i] =
                new JCheckBox(tires[i].getNom(), editeur.isTireAffichee(tires[i]));
        panneauSaisie.add(saisieTires[i]);
      }
      getContentPane().add(panneauSaisie, BorderLayout.CENTER);
      Box commandes = Box.createHorizontalBox();
      commandes.add(new JButton(new GAction("Annuler") {
        @Override
        public void executer() {
          dispose();
        }
      }));
      commandes.add(new JButton(new GAction("Valider") {
        @Override
        public void executer() {
          boolean[] isAVoir = new boolean[saisieTires.length];
          for (int i = 0; i<saisieTires.length; i++)
            isAVoir[i] = saisieTires[i].isSelected();
          dispose();
          editeur.setTiresAVoir(isAVoir);
        }
      }));
      getContentPane().add(commandes, BorderLayout.SOUTH);
      setLocationRelativeTo(fenetre);
      pack();
    }
  }

  static private class VisuFenetreTemporelle extends JDialog {
    private JTextField saisieBorneInf, saisieBorneSup;
    private double tmin, tmax;
    private PanneauEditeur editeur;
    private VisuFenetreTemporelle(JFrame fenetre, double[] tminmax,
            PanneauEditeur ed) {
      super(fenetre, "Ajustement de la fenêtre temporelle", true);
      this.editeur = ed;
      this.tmin = tminmax[0];
      this.tmax = tminmax[1];
      getContentPane().setLayout(new BorderLayout());
      JPanel panneauSaisie = new JPanel(new GridLayout(4, 1));
      panneauSaisie.add(new JLabel("Borne inférieure (s.)"));
      saisieBorneInf = new JTextField(""+tmin, 7);
      panneauSaisie.add(saisieBorneInf);
      panneauSaisie.add(new JLabel("Borne supérieure(s.)"));
      saisieBorneSup = new JTextField(""+tmax, 7);
      panneauSaisie.add(saisieBorneSup);
      getContentPane().add(panneauSaisie, BorderLayout.CENTER);
      Box commandes = Box.createHorizontalBox();
      commandes.add(new JButton(new GAction("Annuler") {
        @Override
        public void executer() {
          dispose();
        }
      }));
      commandes.add(new JButton(new GAction("Valider") {
        @Override
        public void executer() {
          try {
            double min = Double.valueOf(saisieBorneInf.getText());
            if (min<tmin) min = tmin;
            double max = Double.valueOf(saisieBorneSup.getText());
            if (max>tmax) max = tmax;
            if (min>=max) throw new NumberFormatException(
                      "Borne supérieure inférieure à la borne inférieure");
            editeur.ajusterFenetreTemporelle(min, max);
            dispose();
          } catch (NumberFormatException ex) {
            GMessages.impossibiliteAction("Erreur de saisie :\n"
                    +ex.getMessage());
            return;
          }

        }
      }));
      getContentPane().add(commandes, BorderLayout.SOUTH);
      setLocationRelativeTo(fenetre);
      pack();
    }
  }
}
