/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import visuAnalor.Apprentissage.*;
import visuAnalor.analyses.*;
import visuAnalor.donnees.*;
import visuAnalor.donnees.Donnees.*;
import visuAnalor.fichiers.*;
import visuAnalor.parametres.*;
import visuAnalor.statistiques.*;
import visuAnalor.util.*;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
@SuppressWarnings("serial")
public class VisuMain extends JFrame implements DonneesListener,
        Parametres2.ParametresListener {
  private Donnees donnees;
  private PanneauEditeur panneauEditeur;
  private Parametres2 parametres;
  private Lissage lissage;
  private DetectionPeriodes detectPeriodes;
  private DetectionProeminences detectProms;
  private String nomDonnees = "";
  private String nomParams = "";
  public static void main(String[] args) {
    try {
      // Fenêtre
      lancerVisuAnalor(args);
    } catch (Throwable ex) {
      GMessages.erreurFatale(ex);
    }
  }
  public static void lancerVisuAnalor(String[] args)
          throws IOException, ClassNotFoundException {
    // Fenêtre
    VisuMain fenetre = new VisuMain();
    fenetre.setTitre();
    Rectangle rmax = GraphicsEnvironment.getLocalGraphicsEnvironment().
            getMaximumWindowBounds();
    fenetre.setLocation(50, 50);
    fenetre.setSize(rmax.width-100, rmax.height-100);
    fenetre.setVisible(true);
    fenetre.traiterArguments(args);
//    System.out.println(fenetre.donnees.getStructure().toString());
  }
  VisuMain() {
    super();
    donnees = new Donnees();
    parametres = new Parametres2();
    parametres.initDefauts();
    parametres.addEventListener(this);
    panneauEditeur = new PanneauEditeur(donnees);
    lissage = new Lissage(donnees, parametres, panneauEditeur);
    detectPeriodes = new DetectionPeriodes(donnees, parametres, panneauEditeur);
    detectProms = new DetectionProeminences(donnees, parametres, panneauEditeur);
    donnees.addEventListener(this);
    // Cadre global de la fenêtre
    setContentPane(panneauEditeur);
    // Menus
    JMenuBar menuBar = new JMenuBar();
    setJMenuBar(menuBar);

    // Menu donnees
    JMenu donneesMenu = new JMenu("Documents");
    menuBar.add(donneesMenu);
    donneesMenu.add(new GAction("Ouvrir un document (.or)...") {
      @Override
      public void executer() {
        if (!sauvegarde()) return;
        donnees.fermer();
        if(!FichiersJava.ouvrirDonnees(donnees, "")) return;
        nomDonnees = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.DONNEES);
        setTitre();
      }
    });
    donneesMenu.add(new GAction("Ouvrir un fichier son (.wav)...") {
      @Override
      public void executer() {
        if (!sauvegarde()) return;
        FichiersSon.importerWav(donnees);
      }
    });
    JMenuItem item;
    donneesMenu.add(item = new JMenuItem(new GAction("Enregistrer") {
      @Override
      public void executer() {
        File fichier = Fichiers.getFichierCourant(Fichiers.TypeFichier.DONNEES);
        if (FichiersJava.enregistrerDonnees(donnees, fichier)) {
          if (fichier==null)
            nomDonnees = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.DONNEES);
          setTitre();
        }
      }
    }));
    item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    donneesMenu.add(new GAction("Enregistrer sous...") {
      @Override
      public void executer() {
        if (FichiersJava.enregistrerDonneesSous(donnees)) {
          nomDonnees = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.DONNEES);
          setTitre();
        }
      }
    });
    donneesMenu.add(new GAction("Fermer") {
      @Override
      public void executer() {
        if (!sauvegarde()) return;
        donnees.fermer();
      }
    });
    donneesMenu.insertSeparator(donneesMenu.getMenuComponentCount());
    donneesMenu.add(new GAction("Exporter le graphique (.png)...") {
      @Override
      public void executer() {
        File f = Fichiers.choisirFichier(Fichiers.Commande.ENREGISTRER,
                "Nom du fichier graphique", Fichiers.TypeFichier.GRAPHIQUE_PNG);
        try {
          panneauEditeur.exporterPNG(f);
        } catch (IOException ex) {
          GMessages.impossibiliteAction("Erreur d'écriture de fichier :\n"+ex);
        }

      }
    });
    donneesMenu.insertSeparator(donneesMenu.getMenuComponentCount());
    donneesMenu.add(new GAction("Extraire une partie du document") {
      @Override
      public void executer() {
        GMessages.pasImplemente();
      }
    });
    donneesMenu.add(new GAction("Concaténer deux documents") {
      @Override
      public void executer() {
        GMessages.pasImplemente();
      }
    });
    donneesMenu.insertSeparator(donneesMenu.getMenuComponentCount());

    donneesMenu.add(new GAction("Importer un document Praat...") {
      @Override
      public void executer() {
        if (!fermerDonnees()) return;
        FichiersPraat.importerPraat(donnees);
      }
    });
    donneesMenu.add(new GAction("Exporter un document Praat...") {
      @Override
      public void executer() {
        FichiersPraat.exporterPraat(VisuMain.this, donnees);
      }
    });
    donneesMenu.insertSeparator(donneesMenu.getMenuComponentCount());
    donneesMenu.add(new GAction("Quitter") {
      @Override
      public void executer() {
        fermerFenetre();
      }
    });

    // Menu Affichage
    JMenu affMenu = new JMenu("Affichage");
    menuBar.add(affMenu);
    affMenu.add(new GAction("Ajuster les bornes temporelles") {
      @Override
      public void executer() {
        panneauEditeur.saisirFenetreTemporelle();
      }
    });
    affMenu.insertSeparator(affMenu.getMenuComponentCount());
    affMenu.add(new GAction("Masquer/afficher la fondamentale") {
      public void executer() {
        panneauEditeur.effAffF0();
      }
    });
    affMenu.add(new GAction("Masquer/afficher l'intensité") {
      public void executer() {
        panneauEditeur.effAffI();
      }
    });
    affMenu.insertSeparator(affMenu.getMenuComponentCount());
    affMenu.add(new GAction("Tires à afficher...") {
      public void executer() {
        panneauEditeur.saisirTiresAVoir();
      }
    });
    affMenu.add(new GAction("Nouvelle tire (intervalles)") {
      public void executer() {
        donnees.creerTireVide(Tire.TypeTire.INTERVALLE);
      }
    });
    affMenu.add(new GAction("Nouvelle tire (points)") {
      public void executer() {
        donnees.creerTireVide(Tire.TypeTire.POINT);
      }
    });
    // Menu Stats
    JMenu statsMenu = new JMenu("Statistiques");
    menuBar.add(statsMenu);
    statsMenu.add(new GAction("Fondamentale") {
      @Override
      public void executer() {
        new VisuStatsF0(getContentPane(), donnees, panneauEditeur);
      }
    });
    statsMenu.add(new GAction("Intensité") {
      @Override
      public void executer() {
        new VisuStatsIntensite(getContentPane(), donnees, panneauEditeur);
      }
    });
    statsMenu.add(new GAction("Durées...") {
      @Override
      public void executer() {
        GMessages.pasImplemente();
      }
    });
    // Menu Paramètres
    JMenu paramMenu = new JMenu("Paramétrage");
    menuBar.add(paramMenu);
    paramMenu.add(new GAction("Gestion des paramètres...") {
      @Override
      public void executer() {
        VisuParametres.newVisuParametres(VisuMain.this, parametres);
      }
    });
    paramMenu.add(new GAction("Ouvrir un fichier paramètres (.orp)...") {
      @Override
      public void executer() {
        FichiersJava.ouvrirParametres(parametres);
        nomParams = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.PARAMETRES);
        setTitre();
      }
    });
    paramMenu.add(new GAction("Enregistrer un fichier paramètres (.orp)...") {
      @Override
      public void executer() {
        if (FichiersJava.enregistrerParametres(parametres)) {
          nomParams = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.PARAMETRES);
          setTitre();
        }
      }
    });
    paramMenu.add(new GAction("Exporter un fichier paramètres (.txt)...") {
      @Override
      public void executer() {
        FichiersTableur.exporterTable(parametres.getModeleTableur(),
                Fichiers.TypeFichier.TABLEUR_PARAMS);
      }
    });
    paramMenu.add(new GAction("Rétablir les paramètres par défaut") {
      @Override
      public void executer() {
        parametres.initDefauts();
        nomParams = "";
        setTitre();
      }
    });
    // Menu Lissage
    JMenu stylMenu = new JMenu("Stylisations");
    menuBar.add(stylMenu);
    stylMenu.add(new GAction("Lisser la fondamentale") {
      @Override
      public void executer() {
        lissage.lisserF0();
      }
    });
    stylMenu.add(new GAction("Supprimer la courbe lissée") {
      @Override
      public void executer() {
        lissage.supprimerLissageF0();
      }
    });
    stylMenu.add(new GAction("Exporter la courbe lissée...") {
      @Override
      public void executer() {
        FichiersPraat.exporterPraatF0Lissee(VisuMain.this, lissage.getF0Lissee());
      }
    });
    stylMenu.add(new GAction("Importer une courbe lissée...") {
      @Override
      public void executer() {
        if (donnees.getCourbeF0()==null) {
          GMessages.impossibiliteAction("Avant d'importer une courbe lissée, "
                  +"il faut avoir affiché une fondamentale");
          return;
        }
        FichiersPraat.importerPraatCourbeLissee(panneauEditeur);
      }
    });
   stylMenu.add(new GAction("Importer un prosogramme...") {
      @Override
      public void executer() {
        if (donnees.getCourbeF0()==null) {
          GMessages.impossibiliteAction("Avant d'importer un prosogramme, "
                  +"il faut avoir affiché une fondamentale");
          return;

        }
        Tire syll = donnees.trouverTireParNom(
                parametres.getNomTireSyllabesProsogrammes());
        if (syll==null) {
          GMessages.impossibiliteAction("La tire syllabique "
                  +"pour les prosogrammes, \""
                  +parametres.getNomTireSyllabesProsogrammes()+"\","
                  +" est absente");
          return;
        }
        FichiersPraat.importerPraatProsogramme(panneauEditeur, syll);
      }
    });
    stylMenu.add(new GAction("Exporter le prosogramme...") {
      @Override
      public void executer() {
        double[][] proso = panneauEditeur.getProsogramme();
        if (proso!=null)
          FichiersTableur.exporterTable(Prosogramme.calculerTable(proso),
                  Fichiers.TypeFichier.PROSOGRAMME);
      }
    });
    stylMenu.add(new GAction("Supprimer le prosogramme") {
      @Override
      public void executer() {
        panneauEditeur.newProsogramme(null);
      }
    });
    // Menu Périodes
    JMenu perMenu = new JMenu("Périodes");
    menuBar.add(perMenu);
    perMenu.add(new GAction("Détecter les périodes") {
      @Override
      public void executer() {
        detectPeriodes.detecterPeriodes();
      }
    });
    perMenu.add(new GAction("Exporter les résultats...") {
      @Override
      public void executer() {
        detectPeriodes.exporterResultats();
      }
    });
   perMenu.add(new GAction("Apprentissage...") {
      @Override
      public void executer() {
//                GMessages.pasImplemente();
        ApprendPeriodes.newApprentissage(VisuMain.this, parametres);
      }
    });
   perMenu.add(new GAction("Performances...") {
      @Override
      public void executer() {
//                GMessages.pasImplemente();
        new PerfPeriodes(VisuMain.this, parametres);
      }
    });
    // Menu Proms
    JMenu proMenu = new JMenu("Proéminences");
    menuBar.add(proMenu);
    proMenu.add(new GAction("Détecter les proéminences") {
      @Override
      public void executer() {
        detectProms.detecterProeminences();
      }
    });
    proMenu.add(new GAction("Exporter les résultats...") {
      @Override
      public void executer() {
        detectProms.exporterResultats();
      }
    });
    proMenu.add(new GAction("Apprentissage...") {
      @Override
      public void executer() {
//                GMessages.pasImplemente();
        ApprendProeminences.newApprentissage(VisuMain.this, parametres);
      }
    });
  proMenu.add(new GAction("Performances...") {
      @Override
      public void executer() {
//                GMessages.pasImplemente();
        new PerfProms(VisuMain.this, parametres);
      }
    });
 
    // Fermeture de la fenêtre
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent evt) {
        fermerFenetre();
      }
    });

  }
  public void premierPlan() {
    if ((getExtendedState()&Frame.ICONIFIED)>0) {
      setExtendedState(getExtendedState()&~Frame.ICONIFIED);
    }
    toFront();

  }
  private void fermerFenetre() {
    if (sauvegarde()) System.exit(0);
  }
  private boolean sauvegarde() {
    if (!donnees.isModifie()) return true;
    int rep = JOptionPane.showOptionDialog(this, "Sauvegarder le document en cours ?",
            "Document modifié", JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,
            null, new Object[]{"Oui", "Non", "Annuler"}, "Non");
    if (rep==2) {
      return false;
    }
    if (rep==0) {
      if (FichiersJava.enregistrerDonneesSous(donnees)) {
        nomDonnees = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.DONNEES);
        setTitre();
      } else return false;
    }
    return true;
  }
  private boolean fermerDonnees() {
    if (donnees.isVide()) return true;
    int rep = JOptionPane.showOptionDialog(this, "Fermer d'abord le document en cours ?",
            "Ajout ou remplacement de données", JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,
            null, new Object[]{"Oui", "Non", "Annuler"}, "Non");
    if (rep==2) return false;
    if (rep==0) {
      if (!sauvegarde()) return false;
      donnees.fermer();
      setTitre();
    }
    return true;
  }
  public void traiterEventDonnees(MessageDonnes evt) {
    switch (evt.getType()) {
      case CLEAR_DONNEES:
        nomDonnees = "";
        Fichiers.removeFichierCourant(Fichiers.TypeFichier.DONNEES);
        setTitre();
        return;
      case NEW_DONNEES:
        nomDonnees = Fichiers.getNomFichierCourant(Fichiers.TypeFichier.DONNEES);
        setTitre();
        return;
      case MODIF_F0:
      case MODIF_TIRE:
      case MODIF_NOM_TIRE:
      case NEW_F0:
      case NEW_INTENSITE:
      case ADD_TIRES:
      case SUP_TIRE:
        setTitre();
        return;
      case NEW_SON:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  void setTitre() {
    String titre = "Analor 0.3.1";
    if (!nomDonnees.isEmpty()) {
      titre += " : "+nomDonnees;
      if (donnees.isModifie()) titre += "*";
    }
    if (nomParams.isEmpty()) {
      if (parametres.isModifie()) titre += " - paramètres modifiés*";
      else titre += " - paramètres par défaut";
    } else {
      titre += " - paramètres : "+nomParams;
      if (parametres.isModifie()) titre += "*";
    }
    setTitle(titre);
  }
  private void traiterArguments(String[] args) throws IOException, ClassNotFoundException {
    if (args.length==0) return;
//    FichiersJava.ouvrirCorpusVue(corpus, vue, args[0]);
  }
  @Override
  public void traiterEventParametres(MessageParametres e) {
    setTitre();
  }
}
