/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.analyses;

import java.util.*;
import javax.swing.table.*;
import visuAnalor.*;
import visuAnalor.donnees.*;
import visuAnalor.fichiers.FichiersTableur;
import visuAnalor.parametres.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class DetectionPeriodes implements Donnees.DonneesListener,
        Parametres2.ParametresListener {
  private Donnees donnees;
  private Parametres2 params;
  private PanneauEditeur editeur;
  private Candidat[] candidats = new Candidat[0];
  private Tire tirePauses, tireEuh, tireLoc;
  private double moyenneFq;
  private TablePeriodes tableur;

  public enum ValeurEuh {
    ABSENT, AVANT, APRES_STANDARD, APRES_DESCENDANT;
    boolean nonCoupure() {
      return this==AVANT||this==APRES_DESCENDANT;
    }
  }

  public enum DecisionPeriode {
    NON_EUH, NON_CRITERES, NON_USER, DOUTE_CRITERES,
    OUI_CRITERES, OUI_CHGT_LOC, OUI_FIN, OUI_USER;
    public boolean isPeriode() {
      return this!=NON_CRITERES&&this!=NON_EUH&&this!=NON_USER;
    }
    public boolean isDoute() {
      return this==DOUTE_CRITERES;
    }
    public boolean isModifiable() {
      return this!=OUI_CHGT_LOC&&this!=OUI_FIN;
    }
    public boolean areCriteresPertinents() {
      return this==OUI_CRITERES||this==NON_CRITERES||this==DOUTE_CRITERES;
    }
  }

  public static class Candidat {
    public double[] bornes;  // bornes du tronçon
    private double newBorne0; // borne0 de la période
    private int nbpoints;  // nb points du tronçon
    private int newnbpoints; // nb points de la période
    private double fqMoyenne; // moyenne du tronçon 
    private double newfqMoyenne; // moyenne de la période
    private boolean chtLocuteur;
    private double dureePause;
    private double fqApres;
    private double fqFinale;
    private double fqMaxFinale;
    private int registre;
    private double geste;
    private double saut;
    private ValeurEuh euh;
    private int criterePause;
    private int critereGeste;
    private int critereSaut;
    public DecisionPeriode decision;
    public String infosDetails() {
      String infos = "<html>"
              +String.format("Bornes : %.3fs. - %.3fs.", newBorne0, bornes[1])
              +"<p>"+String.format("Durée de la pause : %.3fs.", dureePause)
              +"<p>"+String.format("Hauteur moyenne : %.2fst.", newfqMoyenne*12)
              +"<p>"+String.format("Hauteur finale : %.2fst.", fqFinale*12)
              +"<p>"+String.format("Hauteur max finale : %.2fst.", fqMaxFinale*12)
              +"<p>"+String.format("Hauteur initiale suivante : %.2fst.", fqApres*12)
              +"<p> -----------<p>";
      switch (decision) {
        case OUI_CHGT_LOC:
        case OUI_FIN:
        case OUI_CRITERES:
          return infos+infosDecision()+"<p>\u21d2 COUPURE DE PERIODE";
        case OUI_USER:
          return infos+infosCriteres()+"<p> -----------<p>"
                  +infosDecision()+"<p>\u21d2 COUPURE DE PERIODE";
        case NON_EUH:
        case NON_CRITERES:
          return infos+infosDecision()+"<p>\u21d2 PAS DE COUPURE DE PERIODE";
        case NON_USER:
          return infos+infosCriteres()+"<p> -----------<p>"
                  +infosDecision()+"<p>\u21d2 PAS DE COUPURE DE PERIODE";
        case DOUTE_CRITERES:
          return infos+infosDecision()+"<p>\u21d2 COUPURE DE PERIODE PROBABLE ?";
        default:
          throw new IllegalArgumentException("Valeur de isperiode"
                  +" non prévue dans un switch");
      }
    }
    private String motifsDecisionOutreCriteres() {
      switch (decision) {
        case OUI_CHGT_LOC:
          return "Changement de locuteur";
        case OUI_FIN:
          return "Dernière période";
        case OUI_USER:
          return "Décision de l'utilisateur";
        case NON_USER:
          return "Décision de l'utilisateur";
        case NON_EUH:
          switch (euh) {
            case AVANT:
              return "Pause précédée par un \"euh\"";
            case APRES_DESCENDANT:
              return "Pause suivie par un \"euh\" descendant";
            default:
              throw new IllegalArgumentException("Cas de ValeurEuh"
                      +" non prévu dans un switch");
          }
        default:  // Utilisation des critères
          throw new IllegalArgumentException("Cas de décision"
                  +" réclamant les critères");
      }
    }
    public String infosDecision() {
      return decision.areCriteresPertinents()
              ? "<html>"+infosCriteres() : motifsDecisionOutreCriteres();
    }
    private String infosCriteres() {
      return valeurCritere(criterePause)+"pause<p>"
              +valeurCritere(critereGeste)+"geste<p>"
              +valeurCritere(critereSaut)+"saut"
              +(euh==ValeurEuh.ABSENT ? ""
              : "<p>(pause suivie par un \"euh\"  non descendant)");
    }
    private static String valeurCritere(int critere) {
      switch (critere) {
        case -1:
          return " - ";
        case 0:
          return " = ";
        default:
          String val = " ";
          for (int n = 0; n<critere; n++) val += "+";
          return val+" ";
      }
    }
    private String infosPauseTire() {
      switch (decision) {
        case OUI_CRITERES:
          return "-";
        case OUI_CHGT_LOC:
          return "$";
        case OUI_USER:
          return "!";
        case DOUTE_CRITERES:
          return "?";
        default:
          throw new IllegalArgumentException("Cas de DecisionPeriode"
                  +" non prévu dans un switch");
      }
    }
    private String infosPeriodeTire(int no) {
      return "période n°"+(no+1)+" registre "+(registre>0 ? "haut" : registre<0
              ? "bas" : "normal")+" geste "+(critereGeste<0 ? "plat"
              : geste>0 ? "montant" : "descendant");
    }
    private final static String[] nomChampTableur = {
      "N°pause", "Coupure?", "Motif décision",
      "Critère pause", "Critère geste", "Critère saut",
      "Début période (s.)", "Début tronçon (s.)",
      "Début pause (s.)", "Durée pause(s.)",
      "Hauteur moyenne (st.)", "Hauteur finale (st.)",
      "Hauteur max finale (st.)", "Hauteur initiale suiv. (st.)"};
    private String valeurChampTableur(int col) {
      switch (col) {
        case 1:
          return decision.isDoute() ? "oui?"
                  : decision.isPeriode() ? "oui" : "non";
        case 2:
          return decision.areCriteresPertinents()
                  ? "Valeur des critères" : motifsDecisionOutreCriteres();
        case 3:
          return decision.areCriteresPertinents()
                  ? valeurCritere(criterePause) : "";
        case 4:
          return decision.areCriteresPertinents()
                  ? valeurCritere(critereGeste) : "";
        case 5:
          return decision.areCriteresPertinents()
                  ? valeurCritere(critereSaut) : "";
        case 6:
          return String.valueOf(newBorne0);
        case 7:
          return String.valueOf(bornes[0]);
        case 8:
          return String.valueOf(bornes[1]);
        case 9:
          return String.valueOf(dureePause);
        case 10:
          return String.valueOf(newfqMoyenne*12);
        case 11:
          return String.valueOf(fqFinale*12);
        case 12:
          return String.valueOf(fqMaxFinale*12);
        case 13:
          return String.valueOf(fqApres*12);
        default:
          throw new IllegalArgumentException("Champ de tableur"
                  +" non prévu dans un switch");
      }
    }
  }
  public DetectionPeriodes(Donnees donnees, Parametres2 params,
          PanneauEditeur editeur) {
    this.donnees = donnees;
    donnees.addEventListener(this);
    this.params = params;
    params.addEventListener(this);
    this.editeur = editeur;
  }
  public Candidat[] getCandidats() {
    return candidats;
  }
  public void supprimerDetectionPeriodes() {
    candidats = new Candidat[0];
    editeur.effacerPeriodes();
  }
  public void detecterPeriodes() {
    if (!calculerPeriodes()) {
      supprimerDetectionPeriodes();
      return;
    }
    String nomTire = params.getNomTirePeriodesCalculees();
    Tire tire = donnees.trouverTireParNom(nomTire);
    if (tire!=null&&!GMessages.confirmationAction("Remplacer la tire "+nomTire
            +" existante")) {
      supprimerDetectionPeriodes();
      return;
    }
    editeur.newPeriodes(this, true);
    afficherTirePeriodes();
  }
  private void redetecterPeriodes() {
    if (candidats.length==0) return;
    if (!calculerPeriodes()) {
      supprimerDetectionPeriodes();
      return;
    }
    editeur.newPeriodes(this, false);
    afficherTirePeriodes();
  }
  private void afficherTirePeriodes() {
    String nomTire = params.getNomTirePeriodesCalculees();
    Tire tire = donnees.trouverTireParNom(nomTire);
    Tire newtire = creerTirePeriodes(nomTire);
    if (tire==null) donnees.ajouterTires(new Tire[]{newtire});
    else tire.remplacerTire(newtire);

  }
  private boolean calculerPeriodes() {
    CourbeF0 f0 = donnees.getCourbeF0();
    if (f0==null) return false;
    double[] bornesTirePauses, bornesLoc, bornesTireEuh;
    boolean[] isPause, isEuh;
    tirePauses = donnees.trouverTireParNom(params.getNomTirePausesPeriodes());
    if (tirePauses==null) {
      GMessages.impossibiliteAction(
              "Pas trouvé la tire indiquant les pauses");
      return false;
    } else if (tirePauses.getType()!=Tire.TypeTire.INTERVALLE) {
      GMessages.impossibiliteAction("La tire indiquant les pauses\n"
              +"doit être une tire de type intervalle");
      return false;
    } else {
      bornesTirePauses = tirePauses.getBornes();
      isPause = CalculsAnalyses.reperer(tirePauses.getMots(), params.getMarquesPausePeriodes());
    }
    tireEuh = donnees.trouverTireParNom(params.getNomTireEuh());
    if (tireEuh==null) {
      if (!GMessages.confirmationAction("Pas trouvé la tire "
              +"indiquant les \"euh\"\nAucun \"euh\" ne sera pris  en compte"))
        return false;
      bornesTireEuh = new double[0];
      isEuh = null;
    } else if (tireEuh.getType()!=Tire.TypeTire.INTERVALLE) {
      GMessages.impossibiliteAction("La tire indiquant les \"euh\"\n"
              +"doit être une tire de type intervalle");
      return false;
    } else {
      bornesTireEuh = tireEuh.getBornes();
      isEuh = CalculsAnalyses.reperer(tireEuh.getMots(), params.getMarquesEuh());
    }
    tireLoc = donnees.trouverTireParNom(params.getNomTireLocuteurs());
    if (tireLoc==null) {
      if (!GMessages.confirmationAction("Pas trouvé la tire "
              +"indiquant les changements de locuteur\nL'enregistrement sera "
              +"traité comme monolocuteur"))
        return false;
      bornesLoc = new double[0];
    } else bornesLoc = tireLoc.getBornes();
    CourbeF0 f0joint = new CourbeF0(f0);
    CalculsAnalyses.joindre(f0joint, bornesTirePauses, isPause, bornesLoc);
    moyenneFq = f0joint.calculerMoyenne();
    determinerCandidats(f0joint, moyenneFq, bornesTirePauses, isPause,
            bornesLoc, bornesTireEuh, isEuh);
    return true;
  }
  static final double PROXIMITE_BORNE = 0.02;  // sur la tire des "euh" et des périodes
  public void determinerCandidats(CourbeF0 f, double fqmoyenne,
          double[] bornesTirePauses, boolean[] isPause, double[] bornesLoc,
          double[] bornesTireEuh, boolean[] isEuh) {
    // il y a autant de candidats que de tronçons (après jointure)
    double[][] bornesPauses = CalculsAnalyses.extraireIntervallesPertinents(
            bornesTirePauses, isPause);
    double[] bdebs = CalculsAnalyses.fusionTri(bornesPauses[1], bornesLoc,
            new double[]{bornesTirePauses[0]});
    double[] bfins = CalculsAnalyses.fusionTri(bornesPauses[0], bornesLoc,
            new double[]{bornesTirePauses[bornesTirePauses.length-1]});
    double[][] bornesEuh = CalculsAnalyses.extraireIntervallesPertinents(
            bornesTireEuh, isEuh);
    candidats = new Candidat[f.getCoupes().size()-1];
    // Première boucle : calcul des bornes
    for (int i = 0; i<candidats.length; i++) {
      candidats[i] = new Candidat();
      candidats[i].bornes = new double[2];
      double tdeb = f.getPoints()[0][f.getCoupes().get(i)];
      double tfin = f.getPoints()[0][f.getCoupes().get(i+1)-1];
      int n = CalculsAnalyses.trouverIndiceIntervalle(tdeb, bornesPauses);
      candidats[i].bornes[0] = n>=0 ? bornesPauses[1][n]
              : CalculsAnalyses.trouverMaxAvant(tdeb, bdebs);
      n = CalculsAnalyses.trouverIndiceIntervalle(tfin, bornesPauses);
      candidats[i].bornes[1] = n>=0 ? bornesPauses[0][n]
              : CalculsAnalyses.trouverMinApres(tfin, bfins);
    }
    // Deuxième boucle : suite et fin des calculs
    for (int i = 0; i<candidats.length; i++) {
      candidats[i].newBorne0 = (i>0&&!candidats[i-1].decision.isPeriode())
              ? candidats[i-1].newBorne0 : candidats[i].bornes[0];
      candidats[i].chtLocuteur = i==candidats.length-1 ? false
              : CalculsAnalyses.trouverBorneDansIntervalle(bornesLoc,
              candidats[i].bornes[1], candidats[i+1].bornes[0])>=0;
      candidats[i].dureePause = i==candidats.length-1 ? Double.NaN
              : candidats[i+1].bornes[0]-candidats[i].bornes[1];
      candidats[i].fqApres = i==candidats.length-1 ? Double.NaN
              : f.getPoints()[1][f.getCoupes().get(i+1)];
      candidats[i].fqFinale = f.getPoints()[1][f.getCoupes().get(i+1)-1];
      candidats[i].fqMaxFinale = CalculsAnalyses.calculerFqMaxFinal(f.getPoints(),
              f.getCoupes().get(i), f.getCoupes().get(i+1)-1,
              params.getDistanceMaximumPeriodes());
      // la moyenne
      double moyenne = 0;
      int nbpts = f.getCoupes().get(i+1)-f.getCoupes().get(i);
      for (int k = f.getCoupes().get(i); k<f.getCoupes().get(i+1); k++)
        moyenne += f.getPoints()[1][k];
      moyenne /= nbpts;
      candidats[i].nbpoints = nbpts;
      candidats[i].fqMoyenne = moyenne;
      if (i==0||candidats[i-1].decision.isPeriode()) {
        candidats[i].newfqMoyenne = moyenne;
        candidats[i].newnbpoints = nbpts;
      } else {
        candidats[i].newnbpoints = candidats[i-1].newnbpoints+nbpts;
        candidats[i].newfqMoyenne =
                (candidats[i-1].newnbpoints*candidats[i-1].newfqMoyenne
                +nbpts*moyenne)/candidats[i].newnbpoints;
      }
      // le registre, le geste et le saut
      candidats[i].registre =
              candidats[i].newfqMoyenne>fqmoyenne+params.getRegistrePeriodes() ? 1
              : candidats[i].newfqMoyenne<fqmoyenne-params.getRegistrePeriodes()
              ? -1 : 0;
      candidats[i].geste =
              Math.abs(candidats[i].newfqMoyenne-candidats[i].fqFinale)
              >Math.abs(candidats[i].newfqMoyenne-candidats[i].fqMaxFinale)
              ? candidats[i].fqFinale-candidats[i].newfqMoyenne
              : candidats[i].fqMaxFinale-candidats[i].newfqMoyenne;
      candidats[i].saut = candidats[i].fqApres-candidats[i].fqFinale;
      // la valeur de euh
      int n = CalculsAnalyses.trouverBorneDansIntervalle(bornesEuh[1],
              candidats[i].bornes[1]-PROXIMITE_BORNE,
              candidats[i].bornes[1]+PROXIMITE_BORNE);
      if (n>=0) candidats[i].euh = ValeurEuh.AVANT;
      else if (i==candidats.length-1) candidats[i].euh = ValeurEuh.ABSENT;
      else {
        n = CalculsAnalyses.trouverBorneDansIntervalle(bornesEuh[0],
                candidats[i+1].bornes[0]-PROXIMITE_BORNE,
                candidats[i+1].bornes[0]+PROXIMITE_BORNE);
        if (n>=0) {
          double hausse = CalculsAnalyses.calculerHausse(f.getPoints(),
                  f.getCoupes().get(i+1), bornesEuh[1][n]);
          candidats[i].euh = hausse<-params.getEuhDescendantPeriodes()
                  ? ValeurEuh.APRES_DESCENDANT : ValeurEuh.APRES_STANDARD;
        } else candidats[i].euh = ValeurEuh.ABSENT;
      }
      // la décision finale
      if (i==candidats.length-1)
        candidats[i].decision = DecisionPeriode.OUI_FIN;
      else if (candidats[i].chtLocuteur)
        candidats[i].decision = DecisionPeriode.OUI_CHGT_LOC;
      else if (candidats[i].euh.nonCoupure())
        candidats[i].decision = DecisionPeriode.NON_EUH;
      else {
        double valPause = candidats[i].dureePause/params.getSeuilPausePeriodes()-1;
        double valGeste = Math.abs(candidats[i].geste)/params.getSeuilGestePeriodes()-1;
        double valSaut = Math.abs(candidats[i].saut)/params.getSeuilSautPeriodes()-1;
        candidats[i].criterePause = calculerCritere(valPause);
        candidats[i].critereGeste = calculerCritere(valGeste);
        candidats[i].critereSaut = calculerCritere(valSaut);
        candidats[i].decision = calculerDecision(valPause, valGeste, valSaut,
                params.getPoidsPausePeriodes(), params.getPoidsGestePeriodes(),
                params.getPoidsSautPeriodes());
      }
    }
  }
  private static int calculerCritere(double valeur) {
    // le critère a pour valeur un entier entre -1 et 2 
    if (valeur<-0.25) return -1;
    if (valeur<0.75) return 0;
    if (valeur<2) return 1;
    return 2;
  }
  private static DecisionPeriode calculerDecision(double val1, double val2,
          double val3, double pds1, double pds2, double pds3) {
    double somme = (val1*pds1+val2*pds2+val3*pds3)/(pds1+pds2+pds3);
    if (somme<0.75) return DecisionPeriode.NON_CRITERES;
    if (somme>1.25) return DecisionPeriode.OUI_CRITERES;
    return DecisionPeriode.DOUTE_CRITERES;
  }
  public void invaliderCoupure(int no) {
    candidats[no].decision = DecisionPeriode.NON_USER;
    repercuterUserDecision(no, false);
    editeur.newPeriodes(this, false);
    afficherTirePeriodes();
  }
  public void validerCoupure(int no) {
    boolean wasdoutecoupure =
            candidats[no].decision==DecisionPeriode.DOUTE_CRITERES;
    candidats[no].decision = DecisionPeriode.OUI_USER;
    if (!wasdoutecoupure) repercuterUserDecision(no, true);
    editeur.newPeriodes(this, false);
    afficherTirePeriodes();
  }
  private void repercuterUserDecision(int no, boolean userdecision) {
    boolean iscoupure = userdecision;
    boolean wascoupure = !userdecision;
    for (int i = no+1; i<candidats.length; i++) {
      if (iscoupure) {
        candidats[i].newBorne0 = candidats[i].bornes[0];
        candidats[i].newfqMoyenne = candidats[i].fqMoyenne;
        candidats[i].newnbpoints = candidats[i].nbpoints;
      } else {
        candidats[i].newBorne0 = candidats[i-1].newBorne0;
        candidats[i].newnbpoints = candidats[i-1].newnbpoints
                +candidats[i].nbpoints;
        candidats[i].newfqMoyenne =
                (candidats[i-1].newnbpoints*candidats[i-1].newfqMoyenne
                +candidats[i].nbpoints*candidats[i].fqMoyenne)
                /candidats[i].newnbpoints;
      }
      candidats[i].registre =
              candidats[i].newfqMoyenne>moyenneFq+params.getRegistrePeriodes() ? 1
              : candidats[i].newfqMoyenne<moyenneFq-params.getRegistrePeriodes()
              ? -1 : 0;
      candidats[i].geste =
              Math.abs(candidats[i].newfqMoyenne-candidats[i].fqFinale)
              >Math.abs(candidats[i].newfqMoyenne-candidats[i].fqMaxFinale)
              ? candidats[i].fqFinale-candidats[i].newfqMoyenne
              : candidats[i].fqMaxFinale-candidats[i].newfqMoyenne;
      wascoupure = candidats[i].decision.isPeriode();
      if (candidats[i].decision.isModifiable()
              &&candidats[i].decision!=DecisionPeriode.NON_EUH) {
        double valPause = candidats[i].dureePause/params.getSeuilPausePeriodes()-1;
        double valGeste = Math.abs(candidats[i].geste)/params.getSeuilGestePeriodes()-1;
        double valSaut = Math.abs(candidats[i].saut)/params.getSeuilSautPeriodes()-1;
        candidats[i].critereGeste = calculerCritere(valGeste);
        candidats[i].decision = calculerDecision(valPause, valGeste, valSaut,
                params.getPoidsPausePeriodes(), params.getPoidsGestePeriodes(),
                params.getPoidsSautPeriodes());
      }
      iscoupure = candidats[i].decision.isPeriode();
      if (wascoupure&&iscoupure) return;
    }
  }
  private Tire creerTirePeriodes(String nom) {
    double[] tlims = donnees.calculerTMinMax();
    ArrayList<String> lmots = new ArrayList<String>();
    ArrayList<Double> lbornes = new ArrayList<Double>();
    lbornes.add(tlims[0]);
    if (candidats[0].bornes[0]-tlims[0]>PROXIMITE_BORNE) {
      lmots.add("");
      lbornes.add(candidats[0].bornes[0]);
    }
    int no = 0;
    for (int i = 0; i<candidats.length-1; i++) {
      if (!candidats[i].decision.isPeriode()) continue;
      lmots.add(candidats[i].infosPeriodeTire(no));
      lbornes.add(candidats[i].bornes[1]);
      if (candidats[i+1].newBorne0-candidats[i].bornes[1]>PROXIMITE_BORNE) {
        lmots.add(candidats[i].infosPauseTire());
        lbornes.add(candidats[i+1].newBorne0);
      }
      no++;
    }
    lmots.add(candidats[candidats.length-1].infosPeriodeTire(no));
    if (tlims[1]-candidats[candidats.length-1].bornes[1]>PROXIMITE_BORNE) {
      lbornes.add(candidats[candidats.length-1].bornes[1]);
      lmots.add("");
    }
    lbornes.add(tlims[1]);
    String[] mots = lmots.toArray(new String[0]);
    double[] bornes = new double[lbornes.size()];
    for (int i = 0; i<bornes.length; i++) bornes[i] = lbornes.get(i);
    return new Tire(donnees, Tire.TypeTire.INTERVALLE, nom, tlims,
            mots, bornes, "");
  }
  @Override
  public void traiterEventDonnees(MessageDonnes evt) {
    switch (evt.getType()) {
      case CLEAR_DONNEES:
      case NEW_DONNEES:
        supprimerDetectionPeriodes();
        return;
      case MODIF_F0:
        if (candidats.length==0) return;
        redetecterPeriodes();
        return;
      case MODIF_TIRE:
      case SUP_TIRE:
        if (candidats.length==0) return;
        if (evt.getSource()!=tirePauses&&evt.getSource()!=tireEuh
                &&evt.getSource()!=tireLoc) return;
        redetecterPeriodes();
        return;
      case MODIF_NOM_TIRE:
        Tire tire = (Tire) evt.getSource();
        if (params.getNomTireEuh().equals(tire.getNom())
                ||params.getNomTireLocuteurs().equals(tire.getNom()))
          redetecterPeriodes();
        return;
      case NEW_SON:
        return;
      case NEW_F0:
        supprimerDetectionPeriodes();
        return;
      case ADD_TIRES:
      case NEW_INTENSITE:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  @Override
  public void traiterEventParametres(MessageParametres evt) {
    switch (evt.getType()) {
      case LISSAGE:
      case AFFICHAGE:
      case PROEMINENCES:
      case APPRENTISSAGE:
        return;
      case NEW_PARAMS:
      case PERIODES:
        redetecterPeriodes();
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  public void exporterResultats() {
    if (candidats.length==0) return;
    if (tableur==null) tableur = new TablePeriodes(candidats);
    FichiersTableur.exporterTable(tableur, Fichiers.TypeFichier.TABLEUR_STATS);
  }

  private static class TablePeriodes extends AbstractTableModel {
    Candidat[] candidats;
    TablePeriodes(Candidat[] candidats) {
      this.candidats = candidats;
    }
    @Override
    public int getRowCount() {
      return candidats.length;
    }
    @Override
    public int getColumnCount() {
      return Candidat.nomChampTableur.length;
    }
    @Override
    public String getColumnName(int col) {
      return Candidat.nomChampTableur[col];
    }
    @Override
    public String getValueAt(int row, int col) {
      return col==0 ? String.valueOf(row+1)
              : candidats[row].valeurChampTableur(col);
    }
  }
  public DetectionPeriodes(Parametres2 params) {
    this.params = params;
  }

  public static class CandidatCalcule {
    static private double seuilPause;
    static private double seuilGeste;
    static private double seuilSaut;
    static private double pdsPause;
    static private double pdsGeste;
    static private double pdsSaut;
    private int nbpoints;  // nb points du tronçon
    private int newnbpoints; // nb points de la période
    private double fqMoyenne; // moyenne du tronçon 
    private double newfqMoyenne; // moyenne de la période
    private double dureePause;
    private double fqFinale;
    private double fqMaxFinale;
    private double saut;
    private boolean isPeriode;
    private boolean areCriteresPertinents;
    static public void setVals(double[] pars) {
// Attention à l'ordre des paramètres
      seuilPause = pars[0];
      seuilGeste = pars[1];
      seuilSaut = pars[2];
      pdsPause = pars[3];
      pdsGeste = pars[4];
      pdsSaut = pars[5];
    }
    public void init(Candidat candidat) {
      nbpoints = candidat.nbpoints;
      fqMoyenne = candidat.fqMoyenne;
      dureePause = candidat.dureePause;
      fqFinale = candidat.fqFinale;
      fqMaxFinale = candidat.fqMaxFinale;
      saut = candidat.saut;
      areCriteresPertinents = candidat.decision.areCriteresPertinents();
      isPeriode = candidat.decision.isPeriode();
    }
    public boolean calculerSiPeriode(int nbptscumule, double fqmoycumulee) {
      newnbpoints = nbptscumule+nbpoints;
      newfqMoyenne = (nbptscumule*fqmoycumulee+nbpoints*fqMoyenne)/newnbpoints;
      if (!areCriteresPertinents) return isPeriode;
      double geste =
              Math.abs(newfqMoyenne-fqFinale)>Math.abs(newfqMoyenne-fqMaxFinale)
              ? fqFinale-newfqMoyenne : fqMaxFinale-newfqMoyenne;
      double valPause = dureePause/seuilPause-1;
      double valGeste = Math.abs(geste)/seuilGeste-1;
      double valSaut = Math.abs(saut)/seuilSaut-1;
      return calculerDecision(valPause, valGeste, valSaut, pdsPause, pdsGeste,
              pdsSaut)!=DecisionPeriode.NON_CRITERES;
    }
    public int getNewnbpoints() {
      return newnbpoints;
    }
    public double getNewfqMoyenne() {
      return newfqMoyenne;
    }
  }
}
