/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.analyses;

import java.util.Arrays;
import javax.swing.table.*;
import visuAnalor.*;
import visuAnalor.donnees.*;
import visuAnalor.fichiers.*;
import visuAnalor.parametres.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class DetectionProeminences implements Donnees.DonneesListener,
        Parametres2.ParametresListener {
  private Donnees donnees;
  private Parametres2 params;
  private PanneauEditeur editeur;
  private Tire tireSyll, tireDysf, tirePhon;
  private Syllabe[] syllabes = new Syllabe[0];
  private TableProms tableur;

  public static class Syllabe {
    // Les Boolean peuvent être nuls : critères non pertinents ou non calculables
    private double[] bornes;
    private String mot;
    private Boolean isPause;
    private Boolean isDysf;
    private Boolean isIsole;
    private Boolean isPromPause;
    private Boolean isPromDuree;
    private Boolean isPromHauteur;
    private Boolean isNonPromDescente;
    private Boolean isPromMontee;
    private double dureeSyll;
    private double dureeVoc;
    private double duree;
    private double moyDureeSyll;
    private double moyDureeVoc;
    private double moyDuree;
    private double hauteur;
    private Boolean isDescente;
    private double moyHauteur;
    private double montee;
    private int nbFacteursProm;
    private double noteDuree;
    private double noteHauteur;
    private double noteMontee;
    private double notePause;
    private double force;
    public Parametres2 params;
    private Syllabe(Parametres2 params) {
      this.params = params;
    }
    public double[] getBornes() {
      return bornes;
    }
    public String getMot() {
      return mot;
    }
    public int getNbFacteursProm() {
      return nbFacteursProm;
    }
    private String infosTire() {
      switch (nbFacteursProm) {
        case 0:
          return "";
        case 1:
          return params.getMarquePromsFaiblesCalculees();
        default:
          return params.getMarquePromsFortesCalculees();
      }
    }
    public boolean isProm() {
      return nbFacteursProm>0;
    }
    public String infos() {
      if (isPause) return "_";
      if (isDysf) return "*";
      String res = String.format("%.0f", force*10);
      for (int i = 0; i<nbFacteursProm; i++) res += "+";
      if (isPromMontee==null||isPromDuree==null||isPromHauteur==null)
        res += "?";
      return res;
    }
    public String infosBulle() {
      String res = "\""+mot+"\"<p>";
      if (isPause) return res+"pause";
      if (isDysf) return res+"dysfluence";
      if (isPromPause) res += "+ PAUSE<p>";
      if (isPromMontee==null) res += "? MONTEE<p>";
      else if (isPromMontee) res += "+ MONTEE<p>";;
      if (isIsole!=null&&isIsole) return res+"? DUREE<p>? HAUTEUR<p>";
      if (isPromDuree==null) res += "? DUREE<p>";
      else if (isPromDuree) res += "+ DUREE<p>";
      if (isPromHauteur==null) res += "? HAUTEUR<p>";
      else if (isPromHauteur) res += "+ HAUTEUR<p>";
      return res;
    }
    public boolean isADetailler() {
      return !isPause&&!isDysf;
    }
    public String infosDetails() {
      if (isPause) return "pause";
      if (isDysf) return "dysf";
      String infos = "<html>";
      if (isPromPause)
        infos += "+ PAUSE : <p>Syllabe précédant une pause<p>";
      else
        infos += "- PAUSE : <p>Syllabe ne précédant pas une pause<p>";
      infos += "\u2192note : "+String.format("%.0f/10<p>", notePause*10);
      if (isPromMontee==null) infos += "? MONTEE : <p>Impossible de calculer"
                +" la montée intrasyllabique<p>(absence de fondamentale "
                +"ou absence de noyau vocalique ?)<p>";
      else {
        infos += (isPromMontee ? "+" : "-")+" MONTEE : <p>Montée intrasyllabique : "
                +String.format("%.2f st. <p>(seuil : %.2f st.)<p>",
                montee*12, params.getSeuilMonteeProms()*12);
        infos += "\u2192note : "+String.format("%.2f/10<p>", noteMontee*10);
      }
      if (isIsole!=null&&isIsole) {
        infos += "? HAUTEUR :<p>? DUREE :<p>Syllabe isolée<p>";
        infos += "FORCE : "+String.format("%.2f/10", force*10);
        return infos;
      }
      if (isPromHauteur==null) {
        if (!Double.isNaN(hauteur))
          infos += "? HAUTEUR : <p>Impossible de calculer"
                  +" la moyenne des hauteurs sur l'empan<p>"
                  +"(aucune syllabe pertinente dans l'empan ?)<p>"
                  +"hauteur de la syllabe : "+String.format("%.2f st.", hauteur)+"<p>";
        else
          infos += "? HAUTEUR : <p>Impossible de calculer"
                  +" la hauteur sur la syllabe <p>(absence de fondamentale ou "
                  +"absence de noyau vocalique ?)<p>";
      } else {
        infos += (isPromHauteur ? "+" : "-")+" HAUTEUR : <p>";
        if (isNonPromDescente!=null&&isNonPromDescente)
          infos += "Syllabe descendante précédée d'une syllabe de hauteur proéminente<p>";
        infos += "Hauteur de la syllabe : "
                +String.format("%.2f st. <p>(moyenne sur l'empan : %.2f st., seuil : %.2f st.)<p>",
                hauteur*12, moyHauteur*12, (moyHauteur+params.getSeuilHauteurProms())*12);
        infos += "\u2192note : "+String.format("%.2f/10<p>", noteHauteur*10);
      }
      if (isPromDuree==null) {
        if (!Double.isNaN(duree))
          infos += "? DUREE : <p>Impossible de calculer"
                  +" la moyenne des durées pondérées sur l'empan<p>"
                  +"(aucune syllabe pertinente dans l'empan ?)<p>"
                  +"duree de la syllabe : "+String.format("%.0f ms.", duree*1000)+"<p>";
        else
          infos += "? DUREE : <p>Impossible de calculer"
                  +" la durée pondérée de la syllabe<p> (absence de noyau vocalique ?)<p>";
      } else {
        infos += (isPromDuree ? "+" : "-")+" DUREE : <p>Durée pondérée de la syllabe : "
                +String.format("%.0f ms. <p>(moyenne sur l'empan : %.0f ms., seuil : %.0f ms.)<p>",
                duree*1000, moyDuree*1000, moyDuree*params.getSeuilDureeProms()*1000);
        infos += "\u2192note : "+String.format("%.2f/10<p>", noteDuree*10);
      }
      infos += "FORCE : "+String.format("%.2f/10", force*10);
      return infos;
    }
    private final static String[] nomChampTableur = {
      "N°syllabe", "Syllabe", "Statut", "Force  (sur 10)",
      "Prom avant pause", "Prom durée", "Prom hauteur", "Prom montée",
      "Durée pondérée (ms.)", "Moyenne durée (ms.)", "Note durée",
      "Hauteur (st.)", "Moyenne hauteur (st.)", "Note hauteur",
      "Montée (st.)", "Note montée"};
    private String valeurChampTableur(int col) {
      switch (col) {
        case 1:
          return mot;
        case 2:
          if (isPause) return "pause";
          if (isDysf) return "dysfluence";
          return infos();
        case 3:
          return isPause||isDysf ? "" : String.valueOf(force*10);
        case 4:
          return isPromPause==null ? "" : isPromPause ? "oui" : "non";
        case 5:
          return isPromDuree==null ? "" : isPromDuree ? "oui" : "non";
        case 6:
          return isPromHauteur==null ? "" : isPromHauteur ? "oui" : "non";
        case 7:
          return isPromMontee==null ? "" : isPromMontee ? "oui" : "non";
        case 8:
          return Double.isNaN(duree) ? "" : String.valueOf(duree*1000);
        case 9:
          return isPromDuree==null ? "" : String.valueOf(moyDuree*1000);
        case 10:
          return isPromDuree==null ? "" : String.valueOf(noteDuree*10);
        case 11:
          return Double.isNaN(hauteur) ? "" : String.valueOf(hauteur*12);
        case 12:
          return isPromHauteur==null ? "" : String.valueOf(moyHauteur*12);
        case 13:
          return isPromHauteur==null ? "" : String.valueOf(noteHauteur*10);
        case 14:
          return isPromMontee==null ? "" : String.valueOf(montee*12);
        case 15:
          return isPromMontee==null ? "" : String.valueOf(noteMontee*10);
        default:
          throw new IllegalArgumentException("Champ de tableur"
                  +" non prévu dans un switch");
      }
    }
  }
  public DetectionProeminences(Donnees donnees, Parametres2 params,
          PanneauEditeur editeur) {
    this.donnees = donnees;
    donnees.addEventListener(this);
    this.params = params;
    params.addEventListener(this);
    this.editeur = editeur;
  }
  public Syllabe[] getSyllabes() {
    return syllabes;
  }
  public void detecterProeminences() {
    if (!calculerProms("")) {
      supprimerDetectionProms();
      return;
    }
    String nomTire = params.getNomTirePromsCalculees();
    Tire tire = donnees.trouverTireParNom(nomTire);
    if (tire!=null&&!GMessages.confirmationAction("Remplacer la tire "+nomTire
            +" existante")) {
      supprimerDetectionProms();
      return;
    }
//    nomTire = params.getNomTirePromsForce();
//    tire = donnees.trouverTireParNom(nomTire);
//    if (tire!=null&&!GMessages.confirmationAction("Remplacer la tire "+nomTire
//            +" existante")) {
//      supprimerDetectionProms();
//      return;
//    }
    editeur.newProeminences(this, true);
    afficherTiresProms();
  }
  private void redetecterProeminences() {
    if (!calculerProms("")) {
      supprimerDetectionProms();
      return;
    }
    editeur.newProeminences(this, false);
    afficherTiresProms();
  }
  public boolean calculerProms(String nomDonnees) {
    CourbeF0 f0 = donnees.getCourbeF0();
    if (f0==null) return false;
    double[] bornesTireSyll, bornesTireDysf, bornesTirePhon;
    boolean[] isPause, isDysf, isVoyelle;
    String[] textesSyll;
    tireSyll = donnees.trouverTireParNom(params.getNomTireSyllabesProms());
    if (tireSyll==null) {
      GMessages.impossibiliteAction(nomDonnees
              +" Pas trouvé la tire indiquant les syllabes");
      return false;
    } else if (tireSyll.getType()!=Tire.TypeTire.INTERVALLE) {
      GMessages.impossibiliteAction(nomDonnees+" La tire syllabique\n"
              +"doit être une tire de type intervalle");
      return false;
    } else if (params.getEmpans()[0]==0&&params.getEmpans()[1]==0) {
      GMessages.impossibiliteAction(nomDonnees+" Paramètres d'empan incorrects");
      return false;
    } else if (params.getPoidsForcePause()==0&&params.getPoidsForceMontee()==0
            &&params.getPoidsForceHauteur()==0&&params.getPoidsForceDuree()==0) {
      GMessages.impossibiliteAction(nomDonnees+" Paramètres de pondération des critères incorrects");
      return false;
    } else {
      bornesTireSyll = tireSyll.getBornes();
      textesSyll = tireSyll.getMots();
      isPause = CalculsAnalyses.reperer(tireSyll.getMots(), params.getMarquesPauseProms());
    }
    tireDysf = donnees.trouverTireParNom(params.getNomTireDysFluences());
    if (tireDysf==null) {
      if (!GMessages.confirmationAction(nomDonnees+" Pas trouvé la tire "
              +"indiquant les dysfluences\nAucune dysfluence ne sera prise en compte"))
        return false;
      bornesTireDysf = new double[0];
      isDysf = null;
    } else if (tireDysf.getType()!=Tire.TypeTire.INTERVALLE) {
      GMessages.impossibiliteAction(nomDonnees+" La tire indiquant les dysfluences\n"
              +"doit être une tire de type intervalle");
      return false;
    } else {
      bornesTireDysf = tireDysf.getBornes();
      isDysf = CalculsAnalyses.reperer(tireDysf.getMots(), params.getMarquesDysFluences());
    }
    if (!params.getRestrictionVocalique()&&params.getPoidsPhonemes()==0) {
      bornesTirePhon = new double[0];
      isVoyelle = null;
    } else {
      tirePhon = donnees.trouverTireParNom(params.getNomTirePhonemes());
      if (tirePhon==null) {
        if (!GMessages.confirmationAction(nomDonnees+" Pas trouvé la tire phonémique."
                +"Le poids des phonèmes et la restriction vocalique seront ignorés"))
          return false;
        bornesTirePhon = new double[0];
        isVoyelle = null;
      } else if (tirePhon.getType()!=Tire.TypeTire.INTERVALLE) {
        GMessages.impossibiliteAction(nomDonnees+" La tire phonémique\n"
                +"doit être une tire de type intervalle");
        return false;
      } else {
        bornesTirePhon = tirePhon.getBornes();
        isVoyelle = CalculsAnalyses.reperer(tirePhon.getMots(), params.getVoyelles());
      }
    }
    determinerProeminences(f0, bornesTireSyll, textesSyll, isPause,
            bornesTireDysf, isDysf, bornesTirePhon, isVoyelle);
    return true;
  }
  public void determinerProeminences(CourbeF0 f0, double[] bornesTireSyll,
          String[] textesSyll, boolean[] arePauses, double[] bornesTireDysf,
          boolean[] areDysfs, double[] bornesTirePhon, boolean[] areVoyelles) {
    syllabes = new Syllabe[bornesTireSyll.length-1];
// Première boucle : tout ce qu'on peut faire par syllabe isolé (sans l'empan)
    double pts[][] = f0.getPoints();
    double poidsPhonemes = params.getPoidsPhonemes();
    double seuilMontee = params.getSeuilMonteeProms();
    int indTireDysf = 0;
    int indTirePhon = 0;
    int indF = 0;
    for (int i = 0; i<syllabes.length; i++) {
      syllabes[i] = new Syllabe(params);
      Syllabe syll = syllabes[i];
      syll.mot = textesSyll[i];
      syll.bornes = new double[]{bornesTireSyll[i], bornesTireSyll[i+1]};
      // Est-ce une pause
      if (syll.isPause = arePauses[i]) {
        syll.duree = Double.NaN;
        syll.hauteur = Double.NaN;
        syll.montee = Double.NaN;
        continue;
      }
      // Est-ce une dysfluence
      syll.isDysf = false;
      double milieu = 0.5*(syll.bornes[0]+syll.bornes[1]);
      for (; indTireDysf<bornesTireDysf.length-1; indTireDysf++) {
        if (bornesTireDysf[indTireDysf+1]<milieu) continue;
        syll.isDysf = areDysfs[indTireDysf];
        break;
      }
      if (syll.isDysf) {
        syll.duree = Double.NaN;
        syll.hauteur = Double.NaN;
        syll.montee = Double.NaN;
        continue;
      }
      // Précède une pause 
      // (la fin d'enregistrement est considérée comme une pause)
      syll.isPromPause = (i==syllabes.length-1||arePauses[i+1]);
      // noyau vocalique (dureeVoc = dureeSyll si pas de restriction vocalique)
      double[] bornesNoyau;
      if (bornesTirePhon.length==0) bornesNoyau = Arrays.copyOf(syll.bornes, 2);
      else {
        bornesNoyau = new double[]{Double.NaN, Double.NaN};
        boolean contientVoyelle = false;
        for (; indTirePhon<bornesTirePhon.length-1; indTirePhon++) {
          double b1 = bornesTirePhon[indTirePhon];
          double b2 = bornesTirePhon[indTirePhon+1];
          if (b2<=syll.bornes[0]) continue;
          if (b1<syll.bornes[0]&&b2-syll.bornes[0]<syll.bornes[0]-b1) continue;
          if (b2>syll.bornes[1]&&b2-syll.bornes[1]>syll.bornes[1]-b1) break;
          if (areVoyelles[indTirePhon]) {
            if (!contientVoyelle) {
              bornesNoyau[0] = b1;
              contientVoyelle = true;
            }
            bornesNoyau[1] = b2;
          } else {
            if (contientVoyelle) break;
            else continue;
          }
        }
      }
      // duree
      syll.dureeSyll = syll.bornes[1]-syll.bornes[0];
      syll.dureeVoc = bornesNoyau[1]-bornesNoyau[0];
      syll.duree = syll.dureeVoc*poidsPhonemes+syll.dureeSyll*(1-poidsPhonemes);
      // moyenne et rise de F0
      if (Double.isNaN(bornesNoyau[0])) {
        syll.hauteur = Double.NaN;
        syll.montee = Double.NaN;
      } else {
        int deb = -1, fin = -1;
        for (int k = indF; k<pts[0].length; k++) {
          if (pts[0][k]<bornesNoyau[0]) continue;
          if (deb<0&&pts[0][k]<=bornesNoyau[1]) deb = k;
          if (pts[0][k]>bornesNoyau[1]) {
            if (deb>=0) fin = k;
            indF = k==0 ? 0 : k-1;
            break;
          }
        }
        if (deb>=0&&fin<0) fin = pts[0].length;
        if (deb<0) {
          syll.hauteur = Double.NaN;
          syll.montee = Double.NaN;
        } else {
          syll.hauteur = CalculsAnalyses.calculerMoyenne(pts, deb, fin);
          syll.isDescente = CalculsAnalyses.calculerDescente(pts, deb, fin);
          syll.montee = CalculsAnalyses.calculerMontee(pts, deb, fin);
          syll.isPromMontee = syll.montee>=seuilMontee;
        }
      }
    }
// Deuxième boucle : calcul de l'empan et de ce qui en dépend
    int gauche = params.getEmpans()[0];
    int droite = params.getEmpans()[1];
    boolean centre = params.getCompteSyllabeCentrale();
    double seuilDuree = params.getSeuilDureeProms();
    double seuilHauteur = params.getSeuilHauteurProms();
    for (int i = 0; i<syllabes.length; i++) {
      Syllabe syll = syllabes[i];
      if (syll.isPause||syll.isDysf) continue;
      // Il y a 4 cas suivant que duree et hauteur sont définies ou non
      int n = 0;
      if (Double.isNaN(syll.duree)) {
        if (Double.isNaN(syll.hauteur)) continue; // 1er cas : on fait rien
        else { // 2è cas : seule la hauteur est traitée
          for (int j = i-1; j>=0&&j>=i-gauche; j--) {
            Syllabe syll1 = syllabes[j];
            if (syll1.isPause||syll1.isDysf) break;
            if (!Double.isNaN(syll1.hauteur)) {
              syll.moyHauteur += syll1.hauteur;
              n++;
            }
          }
          for (int j = i+1; j<syllabes.length&&j<=i+droite; j++) {
            Syllabe syll1 = syllabes[j];
            if (syll1.isPause||syll1.isDysf) break;
            if (!Double.isNaN(syll1.hauteur)) {
              syll.moyHauteur += syll1.hauteur;
              n++;
            }
          }
          if (syll.isIsole = n==0) continue;
          if (centre) {
            syll.moyHauteur += syll.hauteur;
            n++;
          }
          syll.moyHauteur /= n;
          syll.isPromHauteur = syll.hauteur>syll.moyHauteur+seuilHauteur;
          // Prise en compte de la règle de descente
          syll.isNonPromDescente = false;
          if (syll.isPromHauteur&&i>0) {
            Boolean isP = syllabes[i-1].isPromHauteur;
            if (isP!=null&&isP&&syll.isDescente) {
              syll.isPromHauteur = false;
              syll.isNonPromDescente = true;
            }
          }
        }
      } else if (Double.isNaN(syll.hauteur)) { //3è cas : on traite que la duree
        for (int j = i-1; j>=0&&j>=i-gauche; j--) {
          Syllabe syll1 = syllabes[j];
          if (syll1.isPause||syll1.isDysf) break;
          if (!Double.isNaN(syll1.duree)) {
            syll.moyDuree += syll1.duree;
            syll.moyDureeVoc += syll1.dureeVoc;
            syll.moyDureeSyll += syll1.dureeSyll;
            n++;
          }
        }
        for (int j = i+1; j<syllabes.length&&j<=i+droite; j++) {
          Syllabe syll1 = syllabes[j];
          if (syll1.isPause||syll1.isDysf) break;
          if (!Double.isNaN(syll1.duree)) {
            syll.moyDuree += syll1.duree;
            syll.moyDureeVoc += syll1.dureeVoc;
            syll.moyDureeSyll += syll1.dureeSyll;
            n++;
          }
        }
        if (syll.isIsole = n==0) continue;
        if (centre) {
          syll.moyDuree += syll.duree;
          syll.moyDureeVoc += syll.dureeVoc;
          syll.moyDureeSyll += syll.dureeSyll;
          n++;
        }
        syll.moyDuree /= n;
        syll.moyDureeVoc /= n;
        syll.moyDureeSyll /= n;
//       double moyDur = syll.moyDureeVoc*poidsPhonemes+syll.moyDureeSyll*(1-poidsPhonemes);
//        System.out.println(moyDur-syll.moyDuree);
        syll.isPromDuree = syll.duree>syll.moyDuree+seuilDuree;
      } else {  // 4è cas on traite hauteur et duree
        int n1 = 0;
        for (int j = i-1; j>=0&&j>=i-gauche; j--) {
          Syllabe syll1 = syllabes[j];
          if (syll1.isPause||syll1.isDysf) break;
          if (!Double.isNaN(syll1.duree)) {
            syll.moyDureeVoc += syll1.dureeVoc;
            syll.moyDureeSyll += syll1.dureeSyll;
            syll.moyDuree += syll1.duree;
            n1++;
          }
          if (!Double.isNaN(syll1.hauteur)) {
            syll.moyHauteur += syll1.hauteur;
            n++;
          }
        }
        for (int j = i+1; j<syllabes.length&&j<=i+droite; j++) {
          Syllabe syll1 = syllabes[j];
          if (syll1.isPause||syll1.isDysf) break;
          if (!Double.isNaN(syll1.duree)) {
            syll.moyDureeVoc += syll1.dureeVoc;
            syll.moyDureeSyll += syll1.dureeSyll;
            syll.moyDuree += syll1.duree;
            n1++;
          }
          if (!Double.isNaN(syll1.hauteur)) {
            syll.moyHauteur += syll1.hauteur;
            n++;
          }
        }
        if (syll.isIsole = n==0&&n1==0) continue;
        if (centre) {
          syll.moyDureeVoc += syll.dureeVoc;
          syll.moyDureeSyll += syll.dureeSyll;
          syll.moyDuree += syll.duree;
          n1++;
          syll.moyHauteur += syll.hauteur;
          n++;
        }
        if (n1>0) {
          syll.moyDuree /= n1;
          syll.moyDureeVoc /= n1;
          syll.moyDureeSyll /= n1;
//          double moyDur = syll.moyDureeVoc*poidsPhonemes+syll.moyDureeSyll*(1-poidsPhonemes);
//          System.out.println(moyDur-syll.moyDuree);
          syll.isPromDuree = syll.duree>syll.moyDuree*seuilDuree;
        }
        if (n>0) {
          syll.moyHauteur /= n;
          syll.isPromHauteur = syll.hauteur>syll.moyHauteur+seuilHauteur;
          // Prise en compte de la règle de descente
          syll.isNonPromDescente = false;
          if (syll.isPromHauteur&&i>0&&syll.isDescente) {
            Boolean isP = syllabes[i-1].isPromHauteur;
            if (isP!=null&&isP) {
              syll.isPromHauteur = false;
              syll.isNonPromDescente = true;
            }
          }
        }
      }
    }
    // Troisième boucle : calcul de la force et du nb de facteurs proéminents
    double penteForceDuree = params.getPenteForceDuree();
    double penteForceHauteur = params.getPenteForceHauteur();
    double penteForceMontee = params.getPenteForceMontee();
    double poidsForceDuree = params.getPoidsForceDuree();
    double poidsForceHauteur = params.getPoidsForceHauteur();
    double poidsForceMontee = params.getPoidsForceMontee();
    double poidsForcePause = params.getPoidsForcePause();
    for (int i = 0; i<syllabes.length; i++) {
      Syllabe syll = syllabes[i];
      syll.nbFacteursProm = 0;
      syll.noteDuree = Double.NaN;
      syll.noteHauteur = Double.NaN;
      syll.noteMontee = Double.NaN;
      syll.notePause = 0;
      double pds = 0;
      double notes = 0;
      if (syll.isPause||syll.isDysf) continue;
      if (syll.isPromPause) {
        syll.nbFacteursProm++;
        syll.notePause = 1;
      }
      notes += syll.notePause;
      pds += poidsForcePause;
      if (syll.isPromMontee!=null) {
        if (syll.isPromMontee) syll.nbFacteursProm++;
        syll.noteMontee = calculerNote(syll.montee, seuilMontee, penteForceMontee);
        notes += syll.noteMontee;
        pds += poidsForceMontee;
      }
      if (syll.isPromDuree!=null) {
        if (syll.isPromDuree) syll.nbFacteursProm++;
        syll.noteDuree = calculerNote(syll.duree, syll.moyDuree*seuilDuree,
                penteForceDuree);
        notes += syll.noteDuree;
        pds += poidsForceDuree;
      }
      if (syll.isPromHauteur!=null) {
        if (syll.isPromHauteur) syll.nbFacteursProm++;
        syll.noteHauteur = calculerNote(syll.hauteur, syll.moyHauteur+seuilHauteur,
                penteForceHauteur);
        notes += syll.noteHauteur;
        pds += poidsForceHauteur;
      }
      syll.force = notes/pds;
    }
  }
  private double calculerNote(double valeur, double seuil, double pente) {
    return 0.5+0.5*Math.tanh(2*pente*(valeur-seuil)/seuil);
  }
  private void supprimerDetectionProms() {
    syllabes = new Syllabe[0];
    editeur.effacerProeminences();
  }
  private void afficherTiresProms() {
    String nomTire = params.getNomTirePromsCalculees();
    Tire tire = donnees.trouverTireParNom(nomTire);
    Tire newtire = creerTireProms(nomTire);
    if (tire==null) donnees.ajouterTires(new Tire[]{newtire});
    else tire.remplacerTire(newtire);
//    nomTire = params.getNomTirePromsForce();
//    tire = donnees.trouverTireParNom(nomTire);
//    newtire = creerTireForce(nomTire);
//    if (tire==null) donnees.ajouterTires(new Tire[]{newtire});
//    else tire.remplacerTire(newtire);
  }
  private Tire creerTireProms(String nom) {
    int nb = syllabes.length;
    double[] tlims = new double[]{syllabes[0].bornes[0], syllabes[nb-1].bornes[1]};
    String[] mots = new String[nb];
    double bornes[] = new double[nb+1];
    for (int i = 0; i<syllabes.length; i++) {
      Syllabe syll = syllabes[i];
      mots[i] = syll.infosTire();
      bornes[i] = syll.bornes[0];
    }
    bornes[nb] = tlims[1];
    return new Tire(donnees, Tire.TypeTire.INTERVALLE, nom, tlims,
            mots, bornes, "");
  }
//  private Tire creerTireForce(String nomTire) {
//    throw new UnsupportedOperationException("Not yet implemented");
//  }
  @Override
  public void traiterEventDonnees(MessageDonnes evt) {
    switch (evt.getType()) {
      case CLEAR_DONNEES:
      case NEW_DONNEES:
        supprimerDetectionProms();
        return;
      case MODIF_F0:
        if (syllabes.length==0) return;
        redetecterProeminences();
        return;
      case MODIF_TIRE:
      case SUP_TIRE:
        if (syllabes.length==0) return;
        if (evt.getSource()!=tireSyll&&evt.getSource()!=tireDysf
                &&evt.getSource()!=tirePhon) return;
        redetecterProeminences();
        return;
      case MODIF_NOM_TIRE:
        Tire tire = (Tire) evt.getSource();
        if (params.getNomTireDysFluences().equals(tire.getNom())
                ||params.getNomTirePhonemes().equals(tire.getNom()))
          redetecterProeminences();
        return;
      case NEW_SON:
        return;
      case NEW_F0:
        supprimerDetectionProms();
        return;
      case NEW_INTENSITE:
      case ADD_TIRES:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  @Override
  public void traiterEventParametres(MessageParametres evt) {
    switch (evt.getType()) {
      case LISSAGE:
      case AFFICHAGE:
      case PERIODES:
      case APPRENTISSAGE:
        return;
      case NEW_PARAMS:
      case PROEMINENCES:
        if (syllabes.length==0) return;
        redetecterProeminences();
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  public void exporterResultats() {
    if (syllabes.length==0) return;
    if (tableur==null) tableur = new TableProms(syllabes);
    FichiersTableur.exporterTable(tableur, Fichiers.TypeFichier.TABLEUR_STATS);
  }

  private static class TableProms extends AbstractTableModel {
    Syllabe[] syllabes;
    TableProms(Syllabe[] syllabes) {
      this.syllabes = syllabes;
    }
    @Override
    public int getRowCount() {
      return syllabes.length;
    }
    @Override
    public int getColumnCount() {
      return Syllabe.nomChampTableur.length;
    }
    @Override
    public String getColumnName(int col) {
      return Syllabe.nomChampTableur[col];
    }
    @Override
    public String getValueAt(int row, int col) {
      return col==0 ? String.valueOf(row+1)
              : syllabes[row].valeurChampTableur(col);
    }
  }
  /* 
   * Pour l'apprentissage
   * Attention à l'ordre des paramètres à apprendre : 
   * il doit être le même dans setParams et dans ApprendProéminences
   */
  public DetectionProeminences(Parametres2 params) {
    this.params = params;
  }

  public static class SyllabeCalculee {
    static private double seuilDuree;
    static private double seuilHauteur;
    static private double seuilMontee;
    static private double poidsPhonemes;
    private boolean isNeverProm;
    private boolean isPromPrecPause;
    private double dureeSyll;
    private double dureeVoc;
    private double moyDureeSyll;
    private double moyDureeVoc;
    private double hauteur;
    private boolean isDescente;
    private double moyHauteur;
    private double montee;
    private boolean isPromHauteur;
    static public void setVals(double[] pars) {
// Attention à l'ordre des paramètres
      seuilDuree = pars[0];
      seuilHauteur = pars[1];
      seuilMontee = pars[2];
      poidsPhonemes =pars[3];
    }
    public void init(Syllabe syll) {
      isNeverProm = syll.isPause||syll.isDysf;
      if (isNeverProm) return;
      isPromPrecPause = syll.isPromPause;
      if (isPromPrecPause) return;
      if (syll.isPromHauteur==null) {// pas de hauteur ou pas de moyHauteur
        isDescente = false;
        hauteur = Double.NaN;
      } else {
        hauteur = syll.hauteur;
        moyHauteur = syll.moyHauteur;
        isDescente = syll.isDescente;
      }
      if (syll.isPromDuree==null) // pas de duree ou pas de moyDuree
        dureeSyll = dureeVoc = Double.NaN;
      else {
        dureeSyll = syll.dureeSyll;
        dureeVoc = syll.dureeVoc;
        moyDureeSyll = syll.moyDureeSyll;
        moyDureeVoc = syll.moyDureeVoc;
      }
      if (syll.isPromMontee==null) // pas de montee
        montee = Double.NaN;
      else
        montee = syll.montee;
    }
      // à appeler après init et setVals
    // met à jour isPromHauteur
    // syllPrecProm est null si la syll précédente n'est pas prom
    public boolean isProm(SyllabeCalculee syllPrecProm) {
      if (isNeverProm) return false;
      if (isPromPrecPause) return true;
      if (!Double.isNaN(hauteur)&&hauteur>moyHauteur+seuilHauteur) 
        if(!isDescente||(syllPrecProm==null)||!syllPrecProm.isPromHauteur) {
        isPromHauteur = true;
        return true;
        }
        isPromHauteur = false;
     if (!Double.isNaN(montee)&&montee>seuilMontee) return true;
      if (!Double.isNaN(dureeSyll)) {
        double duree = dureeVoc*poidsPhonemes+dureeSyll*(1-poidsPhonemes);
        double moyDuree = moyDureeVoc*poidsPhonemes+moyDureeSyll*(1-poidsPhonemes);
        if (duree>moyDuree*seuilDuree) return true;
        }
      return false;
    }
  }
}
