/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.analyses;

import visuAnalor.*;
import visuAnalor.donnees.*;
import visuAnalor.parametres.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class Lissage implements
        Donnees.DonneesListener, Parametres2.ParametresListener {
  private Donnees donnees;
  private Parametres2 params;
  private PanneauEditeur editeur;
  private CourbeF0 f0lissee;
  private Tire tirePauses, tireLoc;
  public Lissage(Donnees donnees, Parametres2 params, PanneauEditeur editeur) {
    this.donnees = donnees;
    donnees.addEventListener(this);
    this.params = params;
    params.addEventListener(this);
    this.editeur = editeur;
    this.f0lissee = null;
  }
  public CourbeF0 getF0Lissee() {
    return f0lissee;
  }
  public void lisserF0() {
    CourbeF0 f0 = donnees.getCourbeF0();
    f0lissee = f0==null ? null : lisser(f0);
    editeur.newF0Lissee(f0lissee);
  }
  public void supprimerLissageF0() {
    f0lissee = null;
    editeur.newF0Lissee(f0lissee);
  }
  private CourbeF0 lisser(CourbeF0 f0) {
    double[] bornesTirePauses, bornesLoc;
    boolean[] isPause;
    tirePauses = donnees.trouverTireParNom(params.getNomTirePausesPeriodes());
    if (tirePauses==null) {
      if (!GMessages.confirmationAction("Pas trouvé la tire "
              +"indiquant les pauses\nAucune pause ne sera prise en compte"))
        return null;
      bornesTirePauses = new double[0];
      isPause = null;
    } else {
      bornesTirePauses = tirePauses.getBornes();
      isPause = CalculsAnalyses.reperer(tirePauses.getMots(), params.getMarquesPausePeriodes());
    }
    tireLoc = donnees.trouverTireParNom(params.getNomTireLocuteurs());
    if (tireLoc==null) {
      if (!GMessages.confirmationAction("Pas trouvé la tire "
              +"indiquant les changements de locuteur\nL'enregistrement sera "
              +"traité comme monolocuteur"))
        return null;
      bornesLoc = new double[0];
    } else bornesLoc = tireLoc.getBornes();
    CourbeF0 f0lisse = new CourbeF0(f0);
    CalculsAnalyses.joindre(f0lisse, bornesTirePauses, isPause, bornesLoc);
    CalculsAnalyses.convoluer(f0lisse, params.getFenetreLissage(),
            Math.max(params.getEcartmaxLissage(), 0.0001));
    f0lisse.initCourbeF0();
    return f0lisse;
  }
  @Override
  public void traiterEventDonnees(MessageDonnes evt) {
    switch (evt.getType()) {
      case CLEAR_DONNEES:
      case NEW_DONNEES:
        supprimerLissageF0();
        return;
      case MODIF_F0:
        if (f0lissee==null) return;
        lisserF0();
        return;
      case MODIF_TIRE:
      case MODIF_NOM_TIRE:
      case SUP_TIRE:
        if (f0lissee==null) return;
        if (evt.getSource()!=tirePauses&&evt.getSource()!=tireLoc) return;
        lisserF0();
        return;
      case NEW_SON:
        return;
      case NEW_F0:
        supprimerLissageF0();
        return;
      case NEW_INTENSITE:
      case ADD_TIRES:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
  @Override
  public void traiterEventParametres(MessageParametres evt) {
    switch (evt.getType()) {
      case NEW_PARAMS:
      case LISSAGE:
        if (f0lissee==null) return;
        lisserF0();
        return;
      case AFFICHAGE:
      case PERIODES:
      case PROEMINENCES:
      case APPRENTISSAGE:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }
}
