/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.analyses;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import visuAnalor.analyses.DetectionPeriodes.*;
import visuAnalor.donnees.*;
import visuAnalor.fichiers.*;
import visuAnalor.parametres.*;
import visuAnalor.util.*;
import visuAnalor.util.Fichiers.*;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
public class PerfPeriodes {
  private Parametres2 parametres;
  private boolean[][] calcules;
  private boolean[][] references;
  private File[] fics;
  private String texte;
  // Marge d'acceptation d'un écart entre la borne d'une période calculée 
  // et la borne d'une période de référence
  static final double MARGE = 0.01;
  public PerfPeriodes(Frame fenetre, Parametres2 pars) {
    fics = Fichiers.choisirCorpus("Choix du corpus",
            Fichiers.TypeFichier.CORPUS_ANALOR);
    if (fics.length==0) return;
    this.parametres = pars;
    if (!calculerPers()) return;
    afficherPerf(fenetre);
  }
  private boolean calculerPers() {
    DetectionPeriodes detect = new DetectionPeriodes(parametres);
    references = new boolean[fics.length][];
    calcules = new boolean[fics.length][];
    if (parametres.getSeuilPausePeriodes()==0
            ||parametres.getSeuilGestePeriodes()==0
            ||parametres.getSeuilSautPeriodes()==0) {
      GMessages.impossibiliteAction(
              "Paramètres de seuil des critères incorrects");
      return false;
    }
    if (parametres.getPoidsPausePeriodes()==0
            &&parametres.getPoidsGestePeriodes()==0
            &&parametres.getPoidsSautPeriodes()==0) {
      GMessages.impossibiliteAction(
              "Paramètres de pondération des critères incorrects");
      return false;
    }
    Donnees donnees = new Donnees();
    for (int i = 0; i<fics.length; i++) {
      String nomDonnees = Fichiers.extraireNom(fics[i]);
      FichiersJava.ouvrirDonnees(donnees, fics[i], false);
      Candidat[] candidats;
      Tire tireRef = donnees.trouverTireParNom(
              parametres.getNomTirePeriodesReference());
      if (tireRef==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les périodes de référence\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      }
      CourbeF0 f0 = donnees.getCourbeF0();
      if (f0==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé de fondamentale\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      }
      double[] bornesTirePauses, bornesLoc, bornesTireEuh, bornesTireRef;
      boolean[] isPause, isEuh;
      bornesTireRef = tireRef.getBornes();
      Tire tirePauses = donnees.trouverTireParNom(parametres.getNomTirePausesPeriodes());
      if (tirePauses==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les pauses\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      } else if (tirePauses.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : La tire indiquant les pauses\n"
                +"doit être une tire de type intervalle\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      } else {
        bornesTirePauses = tirePauses.getBornes();
        isPause = CalculsAnalyses.reperer(tirePauses.getMots(),
                parametres.getMarquesPausePeriodes());
      }
      Tire tireEuh = donnees.trouverTireParNom(parametres.getNomTireEuh());
      if (tireEuh==null) {
        if (!GMessages.confirmationAction(nomDonnees+"Pas trouvé la tire "
                +"indiquant les \"euh\"\nAucun \"euh\" ne sera pris  en compte"))
          return false;
        bornesTireEuh = new double[0];
        isEuh = null;
      } else if (tireEuh.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees+"La tire indiquant les \"euh\"\n"
                +"doit être une tire de type intervalle"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      } else {
        bornesTireEuh = tireEuh.getBornes();
        isEuh = CalculsAnalyses.reperer(tireEuh.getMots(), parametres.getMarquesEuh());
      }
      Tire tireLoc = donnees.trouverTireParNom(parametres.getNomTireLocuteurs());
      if (tireLoc==null) bornesLoc = new double[0];
      else bornesLoc = tireLoc.getBornes();
      CourbeF0 f0joint = new CourbeF0(f0);
      CalculsAnalyses.joindre(f0joint, bornesTirePauses, isPause, bornesLoc);
      double moyenneFq = f0joint.calculerMoyenne();
      detect.determinerCandidats(f0joint, moyenneFq, bornesTirePauses, isPause,
              bornesLoc, bornesTireEuh, isEuh);
      candidats = detect.getCandidats();
      calcules[i] = new boolean[candidats.length];
      references[i] = new boolean[candidats.length];
      for (int j = 0; j<candidats.length; j++) {
        calcules[i][j] = candidats[j].decision.isPeriode();
        double bsup = candidats[j].bornes[1];
        references[i][j] = CalculsAnalyses.trouverBorneDansIntervalle(
                bornesTireRef, bsup-MARGE, bsup+MARGE)>=0;
//               System.out.println(j+" : calc "+candidats[i][j].decision.isPeriode()+
//                       " ; ref "+references[i][j]);

      }
    }
    return true;
  }
  private void afficherPerf(Frame fenetre) {
    int nbExemples = 0;
    for (int i = 0; i<fics.length; i++) nbExemples += calcules[i].length;
    if (nbExemples==0) {
      GMessages.impossibiliteAction(
              "Aucune donnée pertinente dans les documents choisis\n"
              +"Calcul de performance impossible");
      return;
    }
    Point pos = new Point(fenetre.getX()+250, fenetre.getY());
    Dimension dim = new Dimension(Math.max(200, fenetre.getWidth()-500),
            fenetre.getHeight());
    final JFrame affichage = new JFrame("Performances de la détection de périodes");
    affichage.setLocation(pos);
    affichage.setSize(dim);
    JTextPane afftexte = new JTextPane();
    JScrollPane scroll = new JScrollPane(afftexte);
    JPanel cadre = new JPanel(new BorderLayout());
    cadre.add(scroll, BorderLayout.CENTER);
    Box commandes = Box.createHorizontalBox();
    JButton exporte = new JButton(
            new GAction("Exporter") {
              @Override
              public void executer() {
                exporter();
              }
            ;
    });
    JButton ferme = new JButton(
            new GAction("Fermer") {
              @Override
              public void executer() {
                affichage.dispose();
              }
            ;
    });
    commandes.add(Box.createGlue());
    commandes.add(exporte);
    commandes.add(Box.createGlue());
    commandes.add(ferme);
    commandes.add(Box.createGlue());
    cadre.add(commandes, BorderLayout.SOUTH);
    affichage.setContentPane(cadre);
    texte = "";
    ecrirePars();
    ecrirePerfs(nbExemples, calculerPerf());
    for (int i = 0; i<fics.length; i++) {
      ecrireResults(Fichiers.extraireNom(fics[i]), calcules[i].length, 
              calculerFaux(calcules[i], references[i]));
    }
    afftexte.setText(texte);
    affichage.setVisible(true);


  }
  private double[] calculerPerf() {
    int nbPersCalc = 0, nbPersRef = 0, nbPersOk = 0;
    for (int i = 0; i<fics.length; i++)
      for (int j = 0; j<calcules[i].length; j++) {
        boolean isPerCalc = calcules[i][j];
        boolean isPerRef = references[i][j];
        if (isPerCalc) nbPersCalc++;
        if (isPerRef) nbPersRef++;
        if (isPerCalc&&isPerRef) nbPersOk++;

      }
    double nbDoubleOk = (double) nbPersOk;
    return new double[]{nbDoubleOk/nbPersCalc, nbDoubleOk/nbPersRef,
              2*nbDoubleOk/(nbPersCalc+nbPersRef)};

  }
private Integer[][] calculerFaux(boolean[] calcs, boolean[] refs) {
  ArrayList<Integer> fauxPos = new ArrayList<Integer>();
  ArrayList<Integer> fauxNeg = new ArrayList<Integer>();
  for (int i = 0; i<calcs.length; i++) {
    if(calcs[i]&&!refs[i]) fauxPos.add(i);
    if(!calcs[i]&&refs[i]) fauxNeg.add(i);
    
  }
  return new Integer[][]{fauxPos.toArray(new Integer[0]), fauxNeg.toArray(new Integer[0])};
}
  private void exporter() {
    File fichier;
    if ((fichier = Fichiers.choisirFichier(Commande.ENREGISTRER,
            "Périodes", Fichiers.TypeFichier.TEXTE_PERF))==null)
      return;
    PrintWriter output = null;
    try {
      output = new PrintWriter(fichier, "UTF-8");
      output.print(texte);
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Erreur d'écriture de fichier",
              JOptionPane.ERROR_MESSAGE);
    } finally {
      if (output!=null) output.close();
    }
  }
  private void ecrirePars() {
    texte += "PARAMETRES\n";
    texte += "Seuil de durée des pauses : "+Float.toString(
            Math.round((float) parametres.getSeuilPausePeriodes()*100)/100f)+" s.\n";
    texte += "Seuil de hauteur des gestes : "+Float.toString(
            Math.round((float) parametres.getSeuilGestePeriodes()*1200)/100f)+" st.\n";
    texte += "Seuil de hauteur des sauts : "+Float.toString(
            Math.round((float) parametres.getSeuilSautPeriodes()*1200)/100f)+" st.\n";
    texte += "Poids des durées des pauses : "+Float.toString(
            Math.round((float) parametres.getPoidsPausePeriodes()*100)/100f)+"\n";
    texte += "Poids des hauteurs des gestes : "+Float.toString(
            Math.round((float) parametres.getPoidsGestePeriodes()*100)/100f)+"\n";
    texte += "Poids des hauteurs des sauts : "+Float.toString(
            Math.round((float) parametres.getPoidsSautPeriodes()*100)/100f)+"\n";
    texte += "----------------------\n";
  }
  private void ecrirePerfs(int nbExemples, double[] perfs) {
    texte += "PERFORMANCES GLOBALES\n";
    texte += "Nombre d'exemples : "+nbExemples+"\n";
    texte += "Précision : "+Float.toString(
            Math.round((float) perfs[0]*10000)/100f)+"%\n";
    texte += "Rappel : "+Float.toString(
            Math.round((float) perfs[1]*10000)/100f)+"%\n";
    texte += "F-mesure : "+Float.toString(
            Math.round((float) perfs[2]*10000)/100f)+"%\n";
    texte += "----------------------\n";
  }
  private void ecrireResults(String nomFic, int nbex, Integer[][] faux) {
    texte += "DOCUMENT "+nomFic+"\n";
    texte += nbex+" exemple"+(nbex>1?"s\n":"\n");
    int nbf = faux[0].length;
    texte += nbf  + " faux positif"+(nbf>1?"s":"");
    for (int i = 0; i<nbf; i++) texte += (i==0?" : n°":", ")+(faux[0][i]+1);
    texte += "\n";
        nbf = faux[1].length;
     texte += nbf  + " faux négatif"+(nbf>1?"s":"");
    for (int i = 0; i<nbf; i++) texte += (i==0?" : n°":", ")+(faux[1][i]+1);
    texte += "\n";
    texte += "----------------------\n";
  }
}
