
/* To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.analyses;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import visuAnalor.analyses.DetectionProeminences.*;
import visuAnalor.donnees.*;
import visuAnalor.fichiers.*;
import visuAnalor.parametres.*;
import visuAnalor.util.*;
import visuAnalor.util.Fichiers.*;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
public class PerfProms {
  private Parametres2 parametres;
  private boolean[][] calcules;
  private boolean[][] references;
  private File[] fics;
  private String texte;
  // Marge d'acceptation d'un écart entre la borne d'une période calculée 
  // et la borne d'une période de référence
  static final double MARGE = 0.01;
  public PerfProms(Frame fenetre, Parametres2 pars) {
    fics = Fichiers.choisirCorpus("Choix du corpus",
            Fichiers.TypeFichier.CORPUS_ANALOR);
    if (fics.length==0) return;
    this.parametres = pars;
    if (!calculerProms()) return;
    afficherPerf(fenetre);
  }
  private boolean calculerProms() {
    DetectionProeminences detect = new DetectionProeminences(parametres);
    references = new boolean[fics.length][];
    calcules = new boolean[fics.length][];
    if (parametres.getEmpans()[0]==0&&parametres.getEmpans()[1]==0) {
      GMessages.impossibiliteAction("Paramètres d'empan incorrects");
      return false;
    }
    if (parametres.getPoidsForcePause()==0
            &&parametres.getPoidsForceMontee()==0
            &&parametres.getPoidsForceHauteur()==0
            &&parametres.getPoidsForceDuree()==0) {
      GMessages.impossibiliteAction(
              "Paramètres de pondération des critères incorrects");
      return false;
    }
    Donnees donnees = new Donnees();
    for (int i = 0; i<fics.length; i++) {
      String nomDonnees = Fichiers.extraireNom(fics[i]);
      FichiersJava.ouvrirDonnees(donnees, fics[i], false);
      SyllabeCalculee[] syllabes;
      Tire tireRef = donnees.trouverTireParNom(
              parametres.getNomTirePromsReference());
      if (tireRef==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les proéminences de référence\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      }
      CourbeF0 f0 = donnees.getCourbeF0();
      if (f0==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé de fondamentale\n"
                +"Ce document ne sera pas pris en compte")) return false;
        else {
          calcules[i] = new boolean[0];
          continue;
        }
      }
      double[] bornesTireSyll, bornesTireDysf, bornesTirePhon;
      boolean[] isPause, isDysf, isVoyelle;
      String[] textesSyll;
      references[i] = CalculsAnalyses.reperer(tireRef.getMots(),
              parametres.getMarquesPromsReference());
      Tire tireSyll = donnees.trouverTireParNom(
              parametres.getNomTireSyllabesProms());
      if (tireSyll==null) {
        if (!GMessages.confirmationAction(nomDonnees
                +" : pas trouvé la tire indiquant les syllabes\n"
                +"Ce document ne sera pas pris en compte")) return false;
          calcules[i] = new boolean[0];
          continue;
      }
      if (tireSyll.getType()!=Tire.TypeTire.INTERVALLE) {
        if (!GMessages.confirmationAction(nomDonnees+" : La tire syllabique\n"
                +"doit être une tire de type intervalle\n"
                +"Ce document ne sera pas pris en compte")) return false;
          calcules[i] = new boolean[0];
          continue;
      } 
        bornesTireSyll = tireSyll.getBornes();
        textesSyll = tireSyll.getMots();
        if (textesSyll.length!=references[i].length) {
          if (!GMessages.confirmationAction(nomDonnees
                  +" : La tire indiquant les proéminences de référence\n"
                  +"n'a pas la même taille que la tire syllabique\n"
                  +"Ce document ne sera pas pris en compte")) return false;
          calcules[i] = new boolean[0];
          continue;
        }

        isPause = CalculsAnalyses.reperer(textesSyll,
                parametres.getMarquesPauseProms());
        Tire tireDysf = donnees.trouverTireParNom(
                parametres.getNomTireDysFluences());
        if (tireDysf==null) {
          if (!GMessages.confirmationAction(nomDonnees+" : Pas trouvé la tire "
                  +"indiquant les dysfluences\n"
                  +"Aucune dysfluence ne sera prise en compte"))
            return false;
          bornesTireDysf = new double[0];
          isDysf = null;
        } else if (tireDysf.getType()!=Tire.TypeTire.INTERVALLE) {
          if (!GMessages.confirmationAction(nomDonnees
                  +" : La tire indiquant les dysfluences\n"
                  +"doit être une tire de type intervalle\n"
                  +"Ce document ne sera pas pris en compte")) return false;
            calcules[i] = new boolean[0];
            continue;
        } else {
          bornesTireDysf = tireDysf.getBornes();
          isDysf = CalculsAnalyses.reperer(tireDysf.getMots(),
                  parametres.getMarquesDysFluences());
        }
        if (!parametres.getRestrictionVocalique()
                &&parametres.getPoidsPhonemes()==0) {
          bornesTirePhon = new double[0];
          isVoyelle = null;
        } else {
          Tire tirePhon = donnees.trouverTireParNom(
                  parametres.getNomTirePhonemes());
          if (tirePhon==null) {
            if (!GMessages.confirmationAction(nomDonnees
                    +" : Pas trouvé la tire phonémique. Le poids des phonèmes"
                    +" et la restriction vocalique seront ignorés"))
              return false;
            bornesTirePhon = new double[0];
            isVoyelle = null;
          } else if (tirePhon.getType()!=Tire.TypeTire.INTERVALLE) {
            if (!GMessages.confirmationAction(nomDonnees+" La tire phonémique\n"
                    +"doit être une tire de type intervalle\n"
                    +"Ce document ne sera pas pris en compte")) return false;
            calcules[i] = new boolean[0];
            continue;
          } else {
            bornesTirePhon = tirePhon.getBornes();
            isVoyelle = CalculsAnalyses.reperer(tirePhon.getMots(),
                    parametres.getVoyelles());
          }
        }
        detect.determinerProeminences(f0, bornesTireSyll, textesSyll, isPause,
                bornesTireDysf, isDysf, bornesTirePhon, isVoyelle);
        Syllabe[] sylls = detect.getSyllabes();
        calcules[i] = new boolean[sylls.length];
        for (int j = 0; j<sylls.length; j++) {
          calcules[i][j] = sylls[j].isProm();
        }
    }
    return true;
  }
  private void afficherPerf(Frame fenetre) {
    int nbExemples = 0;
    for (int i = 0; i<fics.length; i++) nbExemples += calcules[i].length;
    if (nbExemples==0) {
      GMessages.impossibiliteAction(
              "Aucune donnée pertinente dans les documents choisis\n"
              +"Calcul de performance impossible");
      return;
    }
    Point pos = new Point(fenetre.getX()+250, fenetre.getY());
    Dimension dim = new Dimension(Math.max(200, fenetre.getWidth()-500),
            fenetre.getHeight());
    final JFrame affichage = new JFrame("Performances de la détection de périodes");
    affichage.setLocation(pos);
    affichage.setSize(dim);
    JTextPane afftexte = new JTextPane();
    JScrollPane scroll = new JScrollPane(afftexte);
    JPanel cadre = new JPanel(new BorderLayout());
    cadre.add(scroll, BorderLayout.CENTER);
    Box commandes = Box.createHorizontalBox();
    JButton exporte = new JButton(
            new GAction("Exporter") {
              @Override
              public void executer() {
                exporter();
              }
            ;
    });
    JButton ferme = new JButton(
            new GAction("Fermer") {
              @Override
              public void executer() {
                affichage.dispose();
              }
            ;
    });
    commandes.add(Box.createGlue());
    commandes.add(exporte);
    commandes.add(Box.createGlue());
    commandes.add(ferme);
    commandes.add(Box.createGlue());
    cadre.add(commandes, BorderLayout.SOUTH);
    affichage.setContentPane(cadre);
    texte = "";
    ecrirePars();
    ecrirePerfs(nbExemples, calculerPerf());
    for (int i = 0; i<fics.length; i++) {
      ecrireResults(Fichiers.extraireNom(fics[i]), calcules[i].length,
              calculerFaux(calcules[i], references[i]));
    }
    afftexte.setText(texte);
    affichage.setVisible(true);


  }
  private double[] calculerPerf() {
    int nbPromCalc = 0, nbPromRef = 0, nbPromOk = 0;
    for (int i = 0; i<fics.length; i++)
      for (int j = 0; j<calcules[i].length; j++) {
        boolean isPromCalc = calcules[i][j];
        boolean isPromRef = references[i][j];
        if (isPromCalc) nbPromCalc++;
        if (isPromRef) nbPromRef++;
        if (isPromCalc&&isPromRef) nbPromOk++;

      }
    double nbDoubleOk = (double) nbPromOk;
    return new double[]{nbDoubleOk/nbPromCalc, nbDoubleOk/nbPromRef,
              2*nbDoubleOk/(nbPromCalc+nbPromRef)};

  }
  private Integer[][] calculerFaux(boolean[] calcs, boolean[] refs) {
    ArrayList<Integer> fauxPos = new ArrayList<Integer>();
    ArrayList<Integer> fauxNeg = new ArrayList<Integer>();
    for (int i = 0; i<calcs.length; i++) {
      if (calcs[i]&&!refs[i]) fauxPos.add(i);
      if (!calcs[i]&&refs[i]) fauxNeg.add(i);

    }
    return new Integer[][]{fauxPos.toArray(new Integer[0]), fauxNeg.toArray(new Integer[0])};
  }
  private void exporter() {
    File fichier;
    if ((fichier = Fichiers.choisirFichier(Commande.ENREGISTRER,
            "Proéminences", Fichiers.TypeFichier.TEXTE_PERF))==null)
      return;
    PrintWriter output = null;
    try {
      output = new PrintWriter(fichier, "UTF-8");
      output.print(texte);
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Erreur d'écriture de fichier",
              JOptionPane.ERROR_MESSAGE);
    } finally {
      if (output!=null) output.close();
    }
  }
  private void ecrirePars() {
    texte += "PARAMETRES\n";
    texte += "Seuil de durée (relatif) : "+Float.toString(
            Math.round((float) parametres.getSeuilDureeProms()*100)/100f)+"\n";
    texte += "Seuil de hauteur : "+Float.toString(
            Math.round((float) parametres.getSeuilHauteurProms()*1200)/100f)+" st.\n";
    texte += "Seuil de montée : "+Float.toString(
            Math.round((float) parametres.getSeuilSautPeriodes()*1200)/100f)+" st.\n";
    texte += "Poids des phonèmes pour la durée : "+Float.toString(
            Math.round((float) parametres.getPoidsPhonemes()*100)/100f)+"\n";
    texte += "----------------------\n";
  }
  private void ecrirePerfs(int nbExemples, double[] perfs) {
    texte += "PERFORMANCES GLOBALES\n";
    texte += "Nombre d'exemples : "+nbExemples+"\n";
    texte += "Précision : "+Float.toString(
            Math.round((float) perfs[0]*10000)/100f)+"%\n";
    texte += "Rappel : "+Float.toString(
            Math.round((float) perfs[1]*10000)/100f)+"%\n";
    texte += "F-mesure : "+Float.toString(
            Math.round((float) perfs[2]*10000)/100f)+"%\n";
    texte += "----------------------\n";
  }
  private void ecrireResults(String nomFic, int nbex, Integer[][] faux) {
    texte += "DOCUMENT "+nomFic+"\n";
    texte += nbex+" exemple"+(nbex>1 ? "s\n" : "\n");
    int nbf = faux[0].length;
    texte += nbf+" faux positif"+(nbf>1 ? "s" : "");
    for (int i = 0; i<nbf; i++) texte += (i==0 ? " : n°" : ", ")+(faux[0][i]+1);
    texte += "\n";
    nbf = faux[1].length;
    texte += nbf+" faux négatif"+(nbf>1 ? "s" : "");
    for (int i = 0; i<nbf; i++) texte += (i==0 ? " : n°" : ", ")+(faux[1][i]+1);
    texte += "\n";
    texte += "----------------------\n";
  }
}
