/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.donnees;

import java.io.*;
import java.util.*;
import org.jfree.data.xy.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class CourbeF0 {
  private Donnees donnees;

  enum Cle {
    F0, F0_SELECT
  };
  private int NO_COURBE, NO_SELECTION;
  // deltaT est le pas temporel minimum en secondes
  private double tmin, tmax, deltaT;
  // points[0][i] est le temps en secondes du i-ème point, 
  // points[1][i] est la fréquence en octaves de ce point
  private double[][] points;
  // coupes.size() est égal au nombre de portions + 1.
  // coupes[i] est égal à l'indice du premier point de la portion n°i 
  //  et coupes[end] est égal à points[0].length.
  private ArrayList<Integer> coupes;
  private DefaultXYDataset dataset;
  private int indPtSelect1 = -1, indPtSelect2 = -1;
  public CourbeF0(double[] tlims, double deltaT, double[][] points,
          ArrayList<Integer> coupes) {
    this.tmin = tlims[0];
    this.tmax = tlims[1];
    this.deltaT = deltaT;
    this.coupes = coupes;
    this.points = points;
    initCourbeF0();
  }
  public CourbeF0(CourbeF0 f0) {
    this.tmin = f0.tmin;
    this.tmax = f0.tmax;
    this.deltaT = f0.deltaT;
    this.points = new double[][]{
      Arrays.copyOf(f0.points[0], f0.points[0].length),
      Arrays.copyOf(f0.points[1], f0.points[1].length)};
    this.coupes = new ArrayList<Integer>(f0.coupes);
  }
  public final void initCourbeF0() {
    double[][] data = calculerData();
    dataset = new DefaultXYDataset();
    dataset.addSeries(Cle.F0, data);
    dataset.addSeries(Cle.F0_SELECT, new double[2][0]);
    NO_COURBE = dataset.indexOf(Cle.F0);
    NO_SELECTION = dataset.indexOf(Cle.F0_SELECT);

  }
  public void setDonnees(Donnees donnees) {
    this.donnees = donnees;
  }
  public double[][] getPoints() {
    return points;
  }
  public void setPoints(double[][] points) {
    this.points = points;
  }
  public ArrayList<Integer> getCoupes() {
    return coupes;
  }
  public void setCoupes(ArrayList<Integer> coupes) {
    this.coupes = coupes;
  }
  public int getNoDataF0() {
    return NO_COURBE;
  }
  public int getNoDataF0Selection() {
    return NO_SELECTION;
  }
  public double getTmin() {
    return tmin;
  }
  public double getTmax() {
    return tmax;
  }
  public double getDeltaT() {
    return deltaT;
  }
  public DefaultXYDataset getDataSet() {
    return dataset;
  }
  public double[] getTSelects() {
    if (dataset.getItemCount(NO_SELECTION)==0) return null;
    return new double[]{
              dataset.getXValue(NO_SELECTION, 0),
              dataset.getXValue(NO_SELECTION, dataset.getItemCount(
              NO_SELECTION)-1)};
  }
  public void selectionner(double x1, double x2) {
    double[][] dataSelect = extraireDataSelect(x1, x2);
    calculerIndSelect(x1, x2);
    dataset.addSeries(Cle.F0_SELECT, dataSelect);
  }
  public void selectionnerPortionContenantunPoint(double x) {
    double[][] dataSelect = trouverSelectSegment(x);
    if (dataSelect==null) {
      deselectionner();
      return;
    }
    dataset.addSeries(Cle.F0_SELECT, dataSelect);
  }
  public void trouverSelectionnerPortion(double x, double y) {
    double[][] dataSelect = trouverSelectSegment(x, y);
    if (dataSelect==null) deselectionner();
    else dataset.addSeries(Cle.F0_SELECT, dataSelect);
  }
  public void deselectionner() {
    dataset.addSeries(Cle.F0_SELECT, new double[2][0]);
    indPtSelect1 = indPtSelect2 = -1;
  }
  public void supprimerSelection() {
    if (indPtSelect2==indPtSelect1) return;
    if (!GMessages.confirmationAction("Voulez-vous supprimer définitivement"+""
            +" le tronçon de fondamentale sélectionné ?")) return;
    int nbpoints = points[0].length-(indPtSelect2-indPtSelect1);
    double[][] newpoints = new double[2][nbpoints];
    System.arraycopy(points[0], 0, newpoints[0], 0, indPtSelect1);
    System.arraycopy(points[1], 0, newpoints[1], 0, indPtSelect1);
    System.arraycopy(points[0], indPtSelect2, newpoints[0], indPtSelect1,
            nbpoints-indPtSelect1);
    System.arraycopy(points[1], indPtSelect2, newpoints[1], indPtSelect1,
            nbpoints-indPtSelect1);
    points = newpoints;
    ArrayList<Integer> newcoupes = new ArrayList<Integer>();
    for (int coupe : coupes) {
      if (coupe<indPtSelect1) newcoupes.add(coupe);
      else break;
    }
    newcoupes.add(indPtSelect1);
    for (int coupe : coupes) {
      if (coupe>indPtSelect2) newcoupes.add(coupe-(indPtSelect2-indPtSelect1));
    }
    coupes = newcoupes;
    double[][] data = calculerData();
    dataset.addSeries(Cle.F0, data);
    deselectionner();
    donnees.fireMessage(new MessageDonnes(this, MessageDonnes.TypeMessage.MODIF_F0));
  }
  public void deplacerSelection(int octave) {
    for (int i = indPtSelect1; i<indPtSelect2; i++) points[1][i] += octave;
    int indData1 = indPtSelect1;
    int indData2 = indPtSelect2;
    for (int i = 1; i<coupes.size()-1; i++) {
      int coupe = coupes.get(i);
      if (coupe>=indPtSelect2) break;
      indData2++;
      if (coupe<=indPtSelect1) indData1++;
    }
    double[][] data = new double[2][dataset.getItemCount(NO_COURBE)];
    double[][] dataSelect = new double[2][dataset.getItemCount(NO_SELECTION)];
    for (int i = 0; i<data[0].length; i++) {
      data[0][i] = dataset.getXValue(NO_COURBE, i);
      if (i<indData1) data[1][i] = dataset.getYValue(NO_COURBE, i);
      else if (i<indData2) {
        dataSelect[0][i-indData1] = dataset.getXValue(NO_COURBE, i);
        dataSelect[1][i-indData1] = data[1][i] =
                dataset.getYValue(NO_COURBE, i)+octave;
      } else data[1][i] = dataset.getYValue(NO_COURBE, i);
    }
    dataset.addSeries(Cle.F0, data);
    dataset.addSeries(Cle.F0_SELECT, dataSelect);
    donnees.fireMessage(new MessageDonnes(this, MessageDonnes.TypeMessage.MODIF_F0));
  }
  public void ecrireJava(ObjectOutputStream oos) throws IOException {
    oos.writeObject(new double[]{tmin, tmax});
    oos.writeDouble(deltaT);
    oos.writeObject(points);
    oos.writeObject(coupes);
  }
  public static CourbeF0 lireJava(ObjectInputStream ois)
          throws IOException, ClassNotFoundException {
    double[] lims = (double[]) ois.readObject();
    double deltaT = ois.readDouble();
    double[][] points = (double[][]) ois.readObject();
    @SuppressWarnings("unchecked")
    ArrayList<Integer> coupes = (ArrayList<Integer>) ois.readObject();
    return new CourbeF0(lims, deltaT, points, coupes);
  }
  private double[][] calculerData() {
    // ajoute un NaN dans chaque intervalle entre deux portions
    double[][] data = new double[2][points[0].length+coupes.size()-2];
    int decalage = 0;
    for (int c = 0; c<coupes.size()-1; c++) {
      for (int i = coupes.get(c); i<coupes.get(c+1); i++) {
        data[0][i+decalage] = points[0][i];
        data[1][i+decalage] = points[1][i];
      }
      if (c==coupes.size()-2) break;
      int i = coupes.get(c+1);
      data[0][i+decalage] = (points[0][i]+points[0][i-1])/2;
      data[1][i+decalage] = Double.NaN;
      decalage++;
    }
    return data;
  }
  public double calculerMoyenne() {
    double somme = 0;
    for (int i = 0; i<points[0].length; i++) somme += points[1][i];
    return somme/points[0].length;
  }
  public double[] calculerBornesEtMoyenne() {
    double somme = 0, nb = 0, min = Double.MAX_VALUE, max = -Double.MAX_VALUE;
    for (int i = 0; i<points[0].length; i++) {
      double f = points[1][i];
      min = Math.min(min, f);
      max = Math.max(max, f);
      somme += f;
      nb++;
    }
    if (nb==0) return null;
    return new double[]{min, max, somme/nb};
  }
  public double[][] getTIntervalles(double binf, double bsup) {
    ArrayList<double[]> intervalles = new ArrayList<double[]>();
    double[] intervalle = null;
    boolean dedans = false;
    for (int i = 0; i<dataset.getItemCount(NO_COURBE); i++) {
      double f = dataset.getYValue(NO_COURBE, i);
      if (dedans) {
        if (Double.isNaN(f)||f<binf||f>=bsup) {
          intervalle[1] = dataset.getXValue(NO_COURBE, i-1);
          intervalles.add(intervalle);
          dedans = false;
        }
      } else {
        if (!Double.isNaN(f)&&f>=binf&&f<bsup) {
          intervalle = new double[2];
          intervalle[0] = dataset.getXValue(NO_COURBE, i);
          dedans = true;
        }
      }
    }
    if (dedans) {
      intervalle[1] = dataset.getXValue(NO_COURBE,
              dataset.getItemCount(NO_COURBE)-1);
      intervalles.add(intervalle);
    }
    return intervalles.toArray(new double[0][2]);
  }
  private double[][] extraireDataSelect(double x1, double x2) {
    int nbdata = dataset.getItemCount(NO_COURBE);
    int ind1 = -1;
    int ind2 = nbdata;
    boolean isNaNprec = false;
    for (int i = 0; i<nbdata; i++) {
      if (Double.isNaN(dataset.getYValue(NO_COURBE, i))) {
        isNaNprec = true;
        continue;
      }
      double x = dataset.getXValue(NO_COURBE, i);
      if (x>=x1) {
        if (ind1<0) ind1 = i;
        if (x>x2) {
          ind2 = i;
          if (isNaNprec) ind2--;
          break;
        }
      }
      isNaNprec = false;
    }
    if (ind1<0||ind1>=ind2) return new double[2][0];
    double[][] dataSelect = new double[2][ind2-ind1];
    for (int i = ind1; i<ind2; i++) {
      dataSelect[0][i-ind1] = dataset.getXValue(NO_COURBE, i);
      dataSelect[1][i-ind1] = dataset.getYValue(NO_COURBE, i);
    }
    return dataSelect;
  }
  private void calculerIndSelect(double x1, double x2) {
    int nbdata = points[0].length;
    indPtSelect1 = -1;
    indPtSelect2 = nbdata;
    for (int i = 0; i<nbdata; i++) {
      double x = points[0][i];
      if (x<x1) continue;
      if (indPtSelect1<0) indPtSelect1 = i;
      if (x>x2) {
        indPtSelect2 = i;
        break;
      }
    }
    if (indPtSelect1<0||indPtSelect2==indPtSelect1)
      indPtSelect1 = indPtSelect2 = -1;

  }
  private int[] trouverIndSelectPortion(double x) {
    // cherche une portion contenant l'abscisse x
    // renvoie les indices de debut et de fin dans data
    // ou null si x n'est pas sur une portion
    // et met à jour les indices de début et de fin de la portion dans points     int indData1 = -1, indData2 = -1;
    indPtSelect1 = indPtSelect2 = -1;
    for (int c = 0; c<coupes.size()-1; c++) {
      int ideb = coupes.get(c);
      if (x<points[0][ideb]) break;
      int ifin = coupes.get(c+1)-1;
      if (x>points[0][ifin]) continue;
      indPtSelect1 = ideb;
      indPtSelect2 = ifin+1;
      return new int[]{indPtSelect1+c, indPtSelect2+c};
    }
    return null;
  }
  private double[][] trouverSelectSegment(double x) {
    int[] indsData = trouverIndSelectPortion(x);
    if (indsData==null) return null;
    int indData1 = indsData[0];
    int indData2 = indsData[1];
    double[][] dataSelect = new double[2][indData2-indData1];
    for (int i = indData1; i<indData2; i++) {
      dataSelect[0][i-indData1] = dataset.getXValue(NO_COURBE, i);
      dataSelect[1][i-indData1] = dataset.getYValue(NO_COURBE, i);
    }
    return dataSelect;
  }
  final static double SEUIL_PROCHE_F0 = 0.3;
  private double[][] trouverSelectSegment(double x, double y) {
    int[] indsData = trouverIndSelectPortion(x);
    if (indsData==null) return null;
    int indData1 = indsData[0];
    int indData2 = indsData[1];
    int indDataSelect = -1;
    for (int i = indData1+1; i<indData2; i++) {
      if (x>=dataset.getXValue(NO_COURBE, i)) continue;
      indDataSelect = i-1;
      break;
    }
    if (indDataSelect<0) return null;
    if (Math.abs(y-dataset.getYValue(NO_COURBE, indDataSelect))>SEUIL_PROCHE_F0)
      return null;
    double[][] dataSelect = new double[2][indData2-indData1];
    for (int i = indData1; i<indData2; i++) {
      dataSelect[0][i-indData1] = dataset.getXValue(NO_COURBE, i);
      dataSelect[1][i-indData1] = dataset.getYValue(NO_COURBE, i);
    }
    return dataSelect;
  }
}
