/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.donnees;

import java.io.*;
import java.util.*;
import org.jfree.data.xy.*;

/**
 *
 * @author Bernard
 */
public class CourbeIntensite {
  private Donnees donnees;
  private double tmin, tmax;
  private double deltaT; // vaudra -1 si les deltaT sont irréguliers
  // points[0][i] est le temps en secondes du i-ème point, 
  // points[1][i] est l'intensité en db de ce point
  private double[][] points;
  private DefaultXYDataset dataset;
  public CourbeIntensite(double[] tlims, double[][] points) {
    this.tmin = tlims[0];
    this.tmax = tlims[1];
    this.points = points;
    dataset = new DefaultXYDataset();
    dataset.addSeries(0, points);
    deltaT = 0; // signifie que deltaT n'a pas encore été calculé
  }
  public boolean isDeltaTRegulier() {
    return getDeltaT()>0;
  }
  public double getDeltaT() { // deltaT = -1 si les deltaT sont irreguliers
 return deltaT==0 ? calculerDeltaT() : deltaT;
  }
 private double calculerDeltaT() {
    double[] tps = points[0];
    double delt = tmax-tmin;
    for (int i = 1; i<tps.length; i++)
      delt = Math.min(delt, tps[i]-tps[i-1]);
    for (int i = 0; i<tps.length; i++)
      if (Math.abs(tps[i]-tps[0]-i*delt)>delt/10) return -1;
    return delt;
  }
  public void setDonnees(Donnees donnees) {
    this.donnees = donnees;
  }
  public XYDataset getDataSet() {
    return dataset;
  }
  public double getTmin() {
    return tmin;
  }
  public double getTmax() {
    return tmax;
  }
  public double[][] getPoints() {
    return points;
  }
  public double[] calculerBornes() {
    double min = Double.MAX_VALUE, max = -Double.MAX_VALUE;
    if (points[0].length==0) return null;
    for (int i = 0; i<points[0].length; i++) {
      double val = points[1][i];
      min = Math.min(min, val);
      max = Math.max(max, val);
    }
    return new double[]{min, max};
  }
  public double[][] getTIntervalles(double binf, double bsup) {
    ArrayList<double[]> intervalles = new ArrayList<double[]>();
    double[] intervalle = null;
    boolean dedans = false;
    for (int i = 0; i<dataset.getItemCount(0); i++) {
      double val = dataset.getYValue(0, i);
      if (dedans) {
        if (val<binf||val>=bsup) {
          intervalle[1] = dataset.getXValue(0, i-1);
          intervalles.add(intervalle);
          dedans = false;
        }
      } else {
        if (val>=binf&&val<bsup) {
          intervalle = new double[2];
          intervalle[0] = dataset.getXValue(0, i);
          dedans = true;
        }
      }
    }
    if (dedans) {
      intervalle[1] = dataset.getXValue(0, dataset.getItemCount(0)-1);
      intervalles.add(intervalle);
    }
    return intervalles.toArray(new double[0][2]);
  }
  public void ecrireJava(ObjectOutputStream oos) throws IOException {
    oos.writeObject(new double[]{tmin, tmax});
    oos.writeObject(points);
  }
  public static CourbeIntensite lireJava(ObjectInputStream ois)
          throws IOException, ClassNotFoundException {
    double[] lims = (double[]) ois.readObject();
    double[][] points = (double[][]) ois.readObject();
    return new CourbeIntensite(lims, points);
  }
}
