/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.donnees;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.ArrayList;
import javax.sound.sampled.*;
import javax.swing.*;
import visuAnalor.*;
import visuAnalor.donnees.MessageDonnes.*;
import visuAnalor.util.GMessages;

/**
 *
 * @author Bernard
 */
public class Donnees {
  public static interface DonneesListener extends EventListener {
    void traiterEventDonnees(MessageDonnes e);
  }
  private ArrayList<DonneesListener> listeCorrespondants =
          new ArrayList<DonneesListener>();
  private boolean modifie = false;
  private Son son;
  private CourbeF0 f0;
  private CourbeIntensite I;
  private Tire[] tires = new Tire[0];
  public void addEventListener(DonneesListener correspondant) {
    listeCorrespondants.add(correspondant);
  }
  public void removeEventListener(DonneesListener correspondant) {
    listeCorrespondants.remove(correspondant);
  }
  void fireMessage(MessageDonnes evt) {
    modifie = !EnumSet.of(TypeMessage.CLEAR_DONNEES, TypeMessage.NEW_DONNEES,
            TypeMessage.NEW_SON).contains(evt.getType());
    for (Object corresp : listeCorrespondants.toArray()) {
      ((DonneesListener) corresp).traiterEventDonnees(evt);
    }
  }
  public boolean isVide() {
    return son==null&&f0==null&&I==null&&tires.length==0;
  }
  public double[] calculerTMinMax() {
    if (isVide()) return null;
    double tmin = Double.POSITIVE_INFINITY;
    double tmax = Double.NEGATIVE_INFINITY;
    if (son!=null) {
      tmin = Math.min(tmin, son.getTmin());
      tmax = Math.max(tmax, son.getTmax());
    }
    if (f0!=null) {
      tmin = Math.min(tmin, f0.getTmin());
      tmax = Math.max(tmax, f0.getTmax());
    }
    if (I!=null) {
      tmin = Math.min(tmin, I.getTmin());
      tmax = Math.max(tmax, I.getTmax());
    }
    for (Tire tire : tires) {
      tmin = Math.min(tmin, tire.getTmin());
      tmax = Math.max(tmax, tire.getTmax());
    }
    return new double[]{tmin, tmax};
  }
  public Son getSon() {
    return son;
  }
  public void setSon(Clip clip) {
    if (clip==null) son = null;
    else son = new Son(clip);
    fireMessage(new MessageDonnes(this, TypeMessage.NEW_SON));
  }
  public CourbeF0 getCourbeF0() {
    return f0;
  }
  public void setCourbeF0(CourbeF0 f0) {
    this.f0 = f0;
    f0.setDonnees(this);
    fireMessage(new MessageDonnes(this, TypeMessage.NEW_F0));
  }
  public CourbeIntensite getCourbeIntensite() {
    return I;
  }
  public void setCourbeIntensite(CourbeIntensite I) {
    this.I = I;
    I.setDonnees(this);
    fireMessage(new MessageDonnes(this, TypeMessage.NEW_INTENSITE));
  }
  public Tire[] getTires() {
    return tires;
  }
  public Tire trouverTireParNom(String nom) {
    for (Tire tire : tires) if (nom.equals(tire.getNom())) return tire;
    return null;
  }
  boolean isTireGuide(Tire tire) {
    for (Tire t : tires) if (t.getTireGuide()==tire) return true;
    return false;
  }
  public void ajouterTires(Tire[] newtires) {
    if (tires.length==0) tires = newtires;
    else {
      Tire[] tiresnew = new Tire[tires.length+newtires.length];
      System.arraycopy(tires, 0, tiresnew, 0, tires.length);
      System.arraycopy(newtires, 0, tiresnew, tires.length, newtires.length);
      tires = tiresnew;
    }
    fireMessage(new MessageDonnes(this, TypeMessage.ADD_TIRES));
    for (Tire tire : newtires) tire.initTireGuide();

  }
  public void dupliquerTire(Tire tire) {
    Tire newtire = new Tire(tire);
    ajouterTires(new Tire[]{newtire});
  }
  public void creerTireVide(Tire.TypeTire type) {
    double[] tlims = calculerTMinMax();
    if (tlims==null) {
      GMessages.impossibiliteAction("Impossible de créer une nouvelle tire \n"
              +"en l'absence d'autres données");
      return;
    }
    Tire newtire = new Tire(this, type, tlims);
    ajouterTires(new Tire[]{newtire});

  }
  void supprimerTire(Tire tire) {
    int no = -1;
    for (int i = 0; i<tires.length; i++) if (tires[i]==tire) {
        no = i;
        break;
      }
    if (no<0)
      throw new IllegalArgumentException("suppression d'une tire inexistante");
    Tire[] tiresnew = new Tire[tires.length-1];
    System.arraycopy(tires, 0, tiresnew, 0, no);
    System.arraycopy(tires, no+1, tiresnew, no, tiresnew.length-no);
    tires = tiresnew;
    for (Tire t : tires) if (t.getTireGuide()==tire) t.liberer();
  }
  void deplacementBorneTire(Tire tire, double oldT, double newT) {
    for (Tire t : tires) if (t!=tire&&t.getTireGuide()==tire)
        t.realignerBorne(oldT, newT);
  }
  void suppressionBorneTire(Tire tire, double supT) {
    for (Tire t : tires) if (t!=tire&&t.getTireGuide()==tire)
        t.reagirSuppressionBorne(supT);
  }
  public void placerTireAvant(Tire t, Tire t0) {
    int pos0 = 0;
    while (tires[pos0]!=t0) pos0++;
    int pos = pos0;
    while (tires[pos]!=t) pos++;
    for (int i = pos; i>pos0; i--) tires[i] = tires[i-1];
    tires[pos0] = t;
  }
  public boolean isModifie() {
    return modifie;
  }
  public void fermer() {
    if (isVide()) return;
    son = null;
    f0 = null;
    I = null;
    tires = new Tire[0];
    fireMessage(new MessageDonnes(this, TypeMessage.CLEAR_DONNEES));
  }

  private enum TypeDonnees {
    F0, TIRES, INTENSITE, FIN;
  }
  public void ecrireJava(ObjectOutputStream oos) throws IOException {
    if (f0!=null) {
      oos.writeObject(TypeDonnees.F0.name());
      f0.ecrireJava(oos);
    }
    if (I!=null) {
      oos.writeObject(TypeDonnees.INTENSITE.name());
      I.ecrireJava(oos);

    }
    if (tires.length>0) {
      oos.writeObject(TypeDonnees.TIRES.name());
      oos.writeInt(tires.length);
      for (int i = 0; i<tires.length; i++) tires[i].ecrireJava(oos);
    }
    oos.writeObject(TypeDonnees.FIN.name());
    modifie = false;
  }
  public void lireJava(ObjectInputStream ois, boolean fire)
          throws IOException, ClassNotFoundException {
    try {
      while (true) {
        switch (TypeDonnees.valueOf((String) ois.readObject())) {
          case FIN:
            if (f0==null&&tires.length==0) return;
            if (fire) {
              fireMessage(new MessageDonnes(this, TypeMessage.NEW_DONNEES));
              for (Tire tire : tires) tire.initTireGuide();
            }
            return;
          case F0:
            f0 = CourbeF0.lireJava(ois);
            f0.setDonnees(this);
            break;
          case INTENSITE:
            I = CourbeIntensite.lireJava(ois);
            I.setDonnees(this);
            break;
          case TIRES:
            int nbtires = ois.readInt();
            tires = new Tire[nbtires];
            for (int i = 0; i<nbtires; i++) {
              tires[i] = Tire.lireJava(this, ois);
            }
        }
      }
    } catch (IllegalArgumentException ex) {
      throw new IOException(ex.getLocalizedMessage());
    }
  }
}