/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.donnees;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import visuAnalor.*;
import visuAnalor.donnees.MessageDonnes.*;
import visuAnalor.fichiers.*;
import visuAnalor.util.*;

/**
 *
 * @author Bernard
 */
public class Tire {
  private Donnees donnees;
  // dans le type INTERVALLE les intervalles forment une partition de l'intervalle [tmin, tmax]
  // bornes.length = mots.length+1, tmin = bornes[0] et tmax = bornes[mots.length]

  public enum TypeTire {
    POINT, INTERVALLE
  }
  private TypeTire type;
  private double tmin, tmax;
  private String nom;
  private String[] mots;
  private double[] bornes;
  private Tire tireGuide;
  private String nomGuide;
  public Tire(Donnees donnees, TypeTire type, String nom, double[] tlims,
          String[] mots, double[] bornes, String nomGuide) {
    // doit ensuite passer par initTireGuide pour être cohérent
    this.donnees = donnees;
    this.type = type;
    this.tmin = tlims[0];
    this.tmax = tlims[1];
    if (nom.isEmpty()) {
      int n = 0;
      do nom = "Tire sans nom "+(++n);
      while (donnees.trouverTireParNom(nom)!=null);
    }
    this.nom = nom;
    this.mots = mots;
    this.bornes = bornes;
    if (type==TypeTire.INTERVALLE&&(bornes.length!=mots.length+1
            ||tmin!=bornes[0]||tmax!=bornes[mots.length]))
      throw new IllegalArgumentException("Tire de type intervalle non conforme");
    this.nomGuide = nomGuide;
  }
  Tire(Donnees donnees, TypeTire type, double[] tlims) { // pas de tireGuide
    this.donnees = donnees;
    this.type = type;
    tmin = tlims[0];
    tmax = tlims[1];
    int n = 0;
    do nom = "Nouvelle tire "+(++n);
    while (donnees.trouverTireParNom(nom)!=null);
    nomGuide = "";
    switch (type) {
      case INTERVALLE:
        mots = new String[]{""};
        bornes = new double[]{tmin, tmax};
        return;
      case POINT:
        mots = new String[0];
        bornes = new double[0];
        return;
    }
  }
  Tire(Tire tire) {  // duplication d'une tire
    donnees = tire.donnees;
    type = tire.type;
    tmin = tire.tmin;
    tmax = tire.tmax;
    int n = 0;
    do nom = "Copie "+(++n)+" de "+tire.getNom();
    while (tire.donnees.trouverTireParNom(nom)!=null);
    mots = Arrays.copyOf(tire.mots, tire.mots.length);
    bornes = Arrays.copyOf(tire.bornes, tire.bornes.length);
    nomGuide = tire.nomGuide;
    tireGuide = tire.tireGuide;
  }
  public String getNom() {
    return nom;
  }
  public TypeTire getType() {
    return type;
  }
  public double getTmin() {
    return tmin;
  }
  public double getTmax() {
    return tmax;
  }
  public String[] getMots() {
    return mots;
  }
  public double[] getBornes() {
    return bornes;
  }
  public Tire getTireGuide() {
    return tireGuide;
  }
  void initTireGuide() {
    tireGuide = null;
    if (nomGuide.isEmpty()) return;
    Tire tire1 = donnees.trouverTireParNom(nomGuide);
    if (tire1==null) return;
    solidariser(tire1);
  }
  public boolean isLibre() {
    return tireGuide==null;
  }
  public void remplacerTire(Tire tire) {
    type = tire.type;
    tmin = tire.tmin;
    tmax = tire.tmax;
    mots = tire.mots;
    bornes = tire.bornes;
    nomGuide = tire.nomGuide;
    tireGuide = tire.tireGuide;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
  }
  public double[][] rechercher(String mot) {
    String m = mot.toLowerCase();
    ArrayList<Integer> nos = new ArrayList<Integer>();
    for (int i = 0; i<mots.length; i++)
      if (mots[i].toLowerCase().contains(m)) nos.add(i);
    double[][] intervalles = new double[nos.size()][2];
    for (int i = 0; i<intervalles.length; i++) {
      intervalles[i][0] = bornes[nos.get(i)];
      intervalles[i][1] = bornes[nos.get(i)+1];     
    }
    return intervalles;
  }
  public void renommer() {
    String newNom = (String) JOptionPane.showInputDialog(null,
            "Nouveau nom de la tire "+"\""+nom+"\"", "Renommer une tire",
            JOptionPane.PLAIN_MESSAGE, null, null, nom);
    if (newNom==null) return;
    newNom = newNom.trim();
    if (nom.equals(newNom)||newNom.isEmpty()) return;
    if (donnees.trouverTireParNom(newNom)!=null) {
      GMessages.impossibiliteAction("Renommage impossible : il existe "
              +"déjà une tire de nom "+"\""+newNom+"\"");
      return;
    }
    nom = newNom;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_NOM_TIRE));
  }
  public void supprimer() {
    donnees.supprimerTire(this);
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.SUP_TIRE));

  }
  public void liberer() {
    tireGuide = null;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
  }
  public void solidariser(Tire tire1) {
    if (tire1.bornes.length==0&&bornes.length>0) {
      GMessages.impossibiliteAction("Alignement de la tire \""+nom
              +"\" sur la tire \""+tire1.getNom()
              +"\" impossible \n sans perte d'annotations");
      return;
    }
    double[] newbornes = alignerBornes(tire1);
    ArrayList<Integer> doublons = new ArrayList<Integer>();
    for (int i = 1; i<newbornes.length; i++)
      if (newbornes[i]==newbornes[i-1]) doublons.add(i);
    if (doublons.isEmpty()) bornes = newbornes;
    else if (!dedoublonner(newbornes, doublons)) {
      GMessages.impossibiliteAction("Alignement de la tire \""+nom
              +"\" sur la tire \""+tire1.getNom()
              +"\" impossible \n sans perte d'annotations");
      return;
    }
    tmin = tire1.tmin;
    tmax = tire1.tmax;
    tireGuide = tire1;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
  }
  private double[] alignerBornes(Tire tire1) {
    double[] newbornes = new double[bornes.length];
    double[] bornes1 = tire1.bornes;
    int i1 = 0;
    switch (type) {
      case INTERVALLE:
        newbornes[0] = tire1.tmin;
        newbornes[newbornes.length-1] = tire1.tmax;
        for (int i = 1; i<newbornes.length-1; i++) {
          double borne = bornes[i];
          if (borne<=tire1.tmin) newbornes[i] = tire1.tmin;
          else if (borne>=tire1.tmax) newbornes[i] = tire1.tmax;
          else {
            while (i1<bornes1.length&&bornes[i]>bornes1[i1]) i1++;
            if (i1==bornes1.length)
              newbornes[i] = (tire1.tmax-bornes[i]<bornes[i]-bornes[i1-1])
                      ? tire1.tmax : bornes1[i1-1];
            else if (i1==0)
              newbornes[i] = (bornes1[0]-bornes[i]<bornes[i]-tire1.tmin)
                      ? bornes1[0] : tire1.tmin;
            else
              newbornes[i] = (bornes1[i1]-bornes[i]<bornes[i]-bornes1[i1-1])
                      ? bornes1[i1] : bornes1[i1-1];
          }
        }
        break;
      case POINT:
        for (int i = 0; i<newbornes.length; i++) {
          double borne = bornes[i];
          if (borne<=tire1.tmin) newbornes[i] = bornes1[0];
          else if (borne>=tire1.tmax) newbornes[i] = bornes1[bornes1.length-1];
          else {
            while (i1<bornes1.length&&borne>bornes1[i1]) i1++;
            if (i1==bornes1.length)
              newbornes[i] = bornes1[i1-1];
            else if (i1==0)
              newbornes[i] = bornes1[0];
            else
              newbornes[i] = (bornes1[i1]-borne<borne-bornes1[i1-1])
                      ? bornes1[i1] : bornes1[i1-1];
          }
        }
        break;
    }
    return newbornes;
  }
  private boolean dedoublonner(double[] newbornes0, ArrayList<Integer> doublons) {
    if (doublons.isEmpty()) return true;
    double[] newbornes = new double[bornes.length-doublons.size()];
    String[] newmots = new String[mots.length-doublons.size()];
    switch (type) {
      case INTERVALLE:
        int d1 = doublons.get(0);
        if (!mots[d1-1].isEmpty()) return false;
        int nb = d1;
        System.arraycopy(newbornes0, 0, newbornes, 0, nb);
        int ind = d1;
        System.arraycopy(mots, 0, newmots, 0, nb-1);
        for (int k = 1; k<doublons.size(); k++) {
          int d0 = d1;
          d1 = doublons.get(k);
          if (!mots[d1-1].isEmpty()) return false;
          nb = d1-d0-1;
          System.arraycopy(newbornes0, d0+1, newbornes, ind, nb);
          System.arraycopy(mots, d0, newmots, ind-1, nb);
          ind += nb;
        }
        if (d1<newbornes0.length-1) {
          nb = newbornes0.length-d1-1;
          System.arraycopy(newbornes0, d1+1, newbornes, ind, nb);
          System.arraycopy(mots, d1, newmots, ind-1, nb);
        }
        bornes = newbornes;
        mots = newmots;
        return true;
      case POINT:
        int ind0 = 0;
        int ind1 = 0;
        for (Integer d : doublons) {
          if (!mots[d-1].isEmpty()) {
            if (!mots[d].isEmpty()) return false;
          } else mots[d-1] = mots[d];
          nb = d-ind0;
          System.arraycopy(newbornes0, ind0, newbornes, ind1, nb);
          System.arraycopy(mots, ind0, newmots, ind1, nb);
          ind0 += nb+1;
          ind1 += nb;
        }
        System.arraycopy(newbornes0, ind0, newbornes, ind1, newbornes0.length-ind0);
        System.arraycopy(mots, ind0, newmots, ind1, mots.length-ind0);
        bornes = newbornes;
        mots = newmots;
        return true;
    }
    throw new IllegalArgumentException("Cas non prévu dans un switch");
  }
  public double[] localiserIntervalle(double t) {
    if (type!=TypeTire.INTERVALLE) return null;
    if (t<tmin) return null;
    for (int i = 1; i<bornes.length; i++) if (t<bornes[i])
        return new double[]{bornes[i-1], bornes[i]};
    return null;
  }
  public int trouverNoAnnotation(double t, double seuil) {
    switch (type) {
      case INTERVALLE:
        if (t<tmin) return -1;
        for (int i = 1; i<bornes.length; i++) if (t<bornes[i]) return i-1;
        return -1;
      case POINT:
        if (t<tmin) return -1;
        if (t<bornes[0]) {
          if (bornes[0]-t<seuil) return 0;
          else return -1;
        }
        for (int i = 1; i<bornes.length; i++) if (t<bornes[i]) {
            if (t-bornes[i-1]<bornes[i]-t) {
              if (t-bornes[i-1]<seuil) return i-1;
              else return -1;
            } else {
              if (bornes[i]-t<seuil) return i;
              else return -1;
            }
          }
        if (t<tmax&&t-bornes[bornes.length-1]<seuil) return bornes.length-1;
        return -1;
    }
    throw new IllegalArgumentException("Cas non prévu dans un switch");
  }
  public int trouverNoBorne(double t, double seuil) {
    if (t<tmin) return Integer.MIN_VALUE;
    if (bornes.length==0) return -1;
    if (t<bornes[0]-seuil) return -1;
    if (t<bornes[0]) return 0;
    for (int i = 1; i<bornes.length; i++) if (t<bornes[i]) {
        if (t<bornes[i-1]+seuil) return i-1;
        if (t<bornes[i]-seuil) return -i-1;
        return i;
      }
    if (t<bornes[bornes.length-1]+seuil) return bornes.length-1;
    if (t<tmax) return -bornes.length-1;
    return Integer.MIN_VALUE;
  }
  public void deplacerBorne(int no, double t, double seuil) {
    switch (type) {
      case POINT:
        if (no==0) {
          if (t<tmin+seuil) return;
        } else if (no==bornes.length-1) {
          if (t>tmax-seuil) return;
        } else {
          if (t<bornes[no-1]+seuil||t>bornes[no+1]-seuil) return;
        }
        break;
      case INTERVALLE:
        if (t<bornes[no-1]+seuil||t>bornes[no+1]-seuil) return;
    }
    bornes[no] = t;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
  }
  public double alignerBorne(int no, double t) {
    double[] bornesGuide = tireGuide.bornes;
    int iGuide = indiceAlignement(t, bornesGuide);
    if (no>0&&bornesGuide[iGuide]==bornes[no-1]) iGuide++;
    if (no<bornes.length-1&&bornesGuide[iGuide]==bornes[no+1]) iGuide--;
    bornes[no] = bornesGuide[iGuide];
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
    return bornes[no];
  }
  private int indiceAlignement(double t, double[] bornes) {
    if (t<=bornes[0]) return 0;
    for (int i = 1; i<bornes.length; i++) if (bornes[i]>t)
        return bornes[i]-t<t-bornes[i-1] ? i : i-1;
    return bornes.length-1;
  }
  public void prevenirDeplacementBorne(double oldT, double newT) {
    donnees.deplacementBorneTire(this, oldT, newT);
  }
  void realignerBorne(double oldT, double newT) {
    for (int i = 0; i<bornes.length; i++) if (bornes[i]==oldT) {
        bornes[i] = newT;
        donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
      }
  }
  void reagirSuppressionBorne(double supT) {
    for (int i = 0; i<bornes.length; i++) if (bornes[i]==supT) liberer();
  }
  public void creerBorne(double t, int no) {
    if (tireGuide!=null) {
      t = tireGuide.bornes[indiceAlignement(t, tireGuide.bornes)];
      if (no<bornes.length&&t==bornes[no]||(no>0&&t==bornes[no-1])) {
        GMessages.impossibiliteAction("L'alignement avec la tire \""
                +tireGuide.nom+"\" interdit une nouvelle borne à cet endroit");
        return;
      }
    }
    double[] newbornes = null;
    String[] newmots = null;
    switch (type) {
      case POINT:
        newbornes = new double[bornes.length+1];
        System.arraycopy(bornes, 0, newbornes, 0, no);
        newbornes[no] = t;
        System.arraycopy(bornes, no, newbornes, no+1, bornes.length-no);
        newmots = new String[mots.length+1];
        System.arraycopy(mots, 0, newmots, 0, no);
        newmots[no] = "";
        System.arraycopy(mots, no, newmots, no+1, mots.length-no);
        break;
      case INTERVALLE:
        if (no==0||no==bornes.length) return;
        newbornes = new double[bornes.length+1];
        System.arraycopy(bornes, 0, newbornes, 0, no);
        newbornes[no] = t;
        System.arraycopy(bornes, no, newbornes, no+1,
                bornes.length-no);
        newmots = new String[mots.length+1];
        System.arraycopy(mots, 0, newmots, 0, no-1);
        if (newbornes[no]-newbornes[no-1]>newbornes[no+1]-newbornes[no]) {
          newmots[no-1] = mots[no-1];
          newmots[no] = "";
        } else {
          newmots[no-1] = "";
          newmots[no] = mots[no-1];
        }
        System.arraycopy(mots, no, newmots, no+1, mots.length-no);

    }
    bornes = newbornes;
    mots = newmots;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));

  }
  public void supprimerBorne(int no) {
    double supT = bornes[no];
    double[] newbornes = null;
    String[] newmots = null;
    switch (type) {
      case POINT:
        if (!mots[no].isEmpty()&&!GMessages.confirmationAction(
                "La suppression de cette borne va conduire\nà supprimer "
                +"l'annotation sous-jacente")) return;
        newbornes = new double[bornes.length-1];
        System.arraycopy(bornes, 0, newbornes, 0, no);
        System.arraycopy(bornes, no+1, newbornes, no, newbornes.length-no);
        newmots = new String[mots.length-1];
        System.arraycopy(mots, 0, newmots, 0, no);
        System.arraycopy(mots, no+1, newmots, no,
                newmots.length-no);
        break;
      case INTERVALLE:
        if (no==0||no==bornes.length-1) return;
        if (!mots[no].isEmpty()&&!mots[no-1].isEmpty()
                &&!GMessages.confirmationAction("La suppression de cette borne "
                +"va conduire\nà concaténer les deux annotations adjacentes"))
          return;
        newbornes = new double[bornes.length-1];
        System.arraycopy(bornes, 0, newbornes, 0, no);
        System.arraycopy(bornes, no+1, newbornes, no, newbornes.length-no);

        newmots = new String[mots.length-1];
        System.arraycopy(mots, 0, newmots, 0, no);
        if (newmots[no-1].isEmpty()) newmots[no-1] = mots[no];
        else if (!mots[no].isEmpty()) newmots[no-1] += " "+mots[no];
        System.arraycopy(mots, no+1, newmots, no, newmots.length-no);

    }
    bornes = newbornes;
    mots = newmots;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
    donnees.suppressionBorneTire(this, supT);
  }
  public void editerAnnotation(int no) {
    String newMot = (String) JOptionPane.showInputDialog(null,
            "Annotation", "Editer une annotation",
            JOptionPane.PLAIN_MESSAGE, null, null, mots[no]);
    if (newMot==null) return;
    mots[no] = newMot.trim();
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
  }
  final static double ECART = 0.01;  // taille de l'intervalle pour un point
  public void convertir() {
    if (donnees.isTireGuide(this)) {
      GMessages.impossibiliteAction("Impossible de convertir une tire "
              +"qui en commande d'autres\nDésolidariser d'abord ces tires");
      return;
    }
    int nb = bornes.length;
    double[] newbornes = null;
    String[] newmots = null;
    switch (type) {
      case POINT:
        for (int i = 1; i<nb; i++) {
          if (bornes[i]-bornes[i-1]<2*ECART) {
            GMessages.impossibiliteAction("Impossible de convertir cette tire :\n"
                    +"certains points sont trop proches les uns des autres");
            return;
          }
        }
        bornes[0] = Math.max(bornes[0], tmin+ECART);
        bornes[nb-1] = Math.min(bornes[nb-1], tmax-ECART);
        newbornes = new double[2*nb+2];
        newmots = new String[2*nb+1];
        newbornes[0] = tmin;
        newbornes[newbornes.length-1] = tmax;
        newmots[0] = "";
        for (int i = 0; i<nb; i++) {
          newbornes[2*i+1] = bornes[i]-ECART/2;
          newbornes[2*i+2] = bornes[i]+ECART/2;
          newmots[2*i+1] = mots[i];
          newmots[2*i+2] = "";
        }
        type = TypeTire.INTERVALLE;
        break;
      case INTERVALLE:
        if (nb%2!=0) {
          GMessages.impossibiliteAction("Impossible de convertir cette tire :\n"
                  +"le nombre de bornes est impair");
          return;
        }
        if (!mots[0].isEmpty()) {
          GMessages.impossibiliteAction("Impossible de convertir cette tire :\n"
                  +"sans perdre certaines annotations");
          return;
        }
        for (int i = 1; i<nb-2; i += 2) {
          if (bornes[i+1]-bornes[i]>2*ECART) {
            GMessages.impossibiliteAction("Impossible de convertir cette tire :\n"
                    +"certains intervalles sont trop grands pour des points");
            return;
          }
          if (!mots[i+1].isEmpty()) {
            GMessages.impossibiliteAction("Impossible de convertir cette tire :\n"
                    +"sans perdre certaines annotations");
            return;
          }
        }
        newbornes = new double[nb/2-1];
        newmots = new String[nb/2-1];
        for (int i = 1; i<nb-2; i += 2) {
          newbornes[(i-1)/2] = (bornes[i]+bornes[i+1])/2;
          newmots[(i-1)/2] = mots[i];
        }
        type = TypeTire.POINT;
        break;
    }
    bornes = newbornes;
    mots = newmots;
    tireGuide = null;
    donnees.fireMessage(new MessageDonnes(this, TypeMessage.MODIF_TIRE));
  }
  private String[] copierMotsTireGuide() {
    String[] mots1 = new String[mots.length];
    int indGuide = 0;
    switch (type) {
      case INTERVALLE: // FUSION
        // Ne fonctionne que pour les tireGuides de type INTERVALLE
        if (tireGuide==null||tireGuide.type!=TypeTire.INTERVALLE)
          throw new IllegalArgumentException("Tireguide incorrect");
        for (int i = 0; i<mots1.length; i++) {
          mots1[i] = "";
          while (tireGuide.bornes[indGuide]<bornes[i+1])
            mots1[i] += (mots1[i].isEmpty() ? "" : " ")
                    +tireGuide.mots[indGuide++];
        }
        break;
      case POINT:  // SELECTION
        // Ne fonctionne que pour les tireGuides de type POINT
        if (tireGuide==null||tireGuide.type!=TypeTire.POINT)
          throw new IllegalArgumentException("Tireguide incorrect");
        for (int i = 0; i<mots1.length; i++) {
          mots1[i] = "";
          for (; indGuide<tireGuide.mots.length; indGuide++)
            if (tireGuide.bornes[indGuide]==bornes[i]) {
              mots1[i] = tireGuide.mots[indGuide];
              break;
            }
        }
    }
    return mots1;
  }
  public void ecrireJava(ObjectOutputStream oos) throws IOException {
    oos.writeObject(new double[]{tmin, tmax});
    oos.writeObject(type.name());
    oos.writeObject(nom);
    oos.writeObject(mots);
    oos.writeObject(bornes);
    oos.writeObject(tireGuide==null ? "" : tireGuide.nom);
  }
  public static Tire lireJava(Donnees donnees, ObjectInputStream ois)
          throws IOException, ClassNotFoundException {
    double[] tlims = (double[]) ois.readObject();
    TypeTire type = TypeTire.valueOf((String) ois.readObject());
    String nom = (String) ois.readObject();
    String[] mots = (String[]) ois.readObject();
    double[] bornes = (double[]) ois.readObject();
    String nomGuide = (String) ois.readObject();
    return new Tire(donnees, type, nom, tlims, mots, bornes, nomGuide);
  }
  public void exporterTableur() {
    FichiersTableur.exporterTable(new TableTire(this),
            Fichiers.TypeFichier.TABLEUR_TIRE);
  }
  public void exporterPraat() {
    FichiersPraat.exporterPraatTire(this);
  }

  private static class TableTire extends AbstractTableModel {
    final private static String[] nomsChampsFixes =
            new String[]{"N°", "Début", "Fin", "Position"};
    String[] champs;
    String[][] valeurs;
    TableTire(Tire tire) {
      Tire tireGuide = tire.getTireGuide();
      switch (tire.getType()) {
        case INTERVALLE:
          if (tireGuide==null||tireGuide.getType()!=TypeTire.INTERVALLE) {
            champs = new String[]{nomsChampsFixes[0], tire.getNom(),
              nomsChampsFixes[1], nomsChampsFixes[2]};
            valeurs = new String[tire.getMots().length][4];
            for (int i = 0; i<tire.getMots().length; i++) {
              valeurs[i][0] = String.valueOf(i+1);
              valeurs[i][1] = tire.getMots()[i];
              valeurs[i][2] = String.valueOf(tire.getBornes()[i]);
              valeurs[i][3] = String.valueOf(tire.getBornes()[i+1]);
            }
          } else {
            String[] valsChampGuide = tire.copierMotsTireGuide();
            champs = new String[]{nomsChampsFixes[0], tire.getNom(),
              nomsChampsFixes[1], nomsChampsFixes[2], tireGuide.getNom()};
            valeurs = new String[tire.getMots().length][5];
            for (int i = 0; i<tire.getMots().length; i++) {
              valeurs[i][0] = String.valueOf(i+1);
              valeurs[i][1] = tire.getMots()[i];
              valeurs[i][2] = String.valueOf(tire.getBornes()[i]);
              valeurs[i][3] = String.valueOf(tire.getBornes()[i+1]);
              valeurs[i][4] = valsChampGuide[i];
            }
          }
          break;
        case POINT:
          if (tireGuide==null||tireGuide.getType()!=TypeTire.POINT) {
            champs = new String[]{nomsChampsFixes[0], tire.getNom(),
              nomsChampsFixes[3]};
            valeurs = new String[tire.getMots().length][3];
            for (int i = 0; i<tire.getMots().length; i++) {
              valeurs[i][0] = String.valueOf(i+1);
              valeurs[i][1] = tire.getMots()[i];
              valeurs[i][2] = String.valueOf(tire.getBornes()[i]);
            }
          } else {
            String[] valsChampGuide = tire.copierMotsTireGuide();
            champs = new String[]{nomsChampsFixes[0], tire.getNom(),
              nomsChampsFixes[3], tireGuide.getNom()};
            valeurs = new String[tire.getMots().length][4];
            for (int i = 0; i<tire.getMots().length; i++) {
              valeurs[i][0] = String.valueOf(i+1);
              valeurs[i][1] = tire.getMots()[i];
              valeurs[i][2] = String.valueOf(tire.getBornes()[i]);
              valeurs[i][3] = valsChampGuide[i];
            }
          }
          break;
      }
    }
    @Override
    public int getRowCount() {
      return valeurs.length;
    }
    @Override
    public int getColumnCount() {
      return champs.length;
    }
    @Override
    public String getColumnName(int col) {
      return champs[col];
    }
    @Override
    public String getValueAt(int row, int col) {
      return valeurs[row][col];
    }
  }
}
