/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.fichiers;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import visuAnalor.PanneauEditeur;
import visuAnalor.donnees.*;
import visuAnalor.donnees.Tire.*;
import visuAnalor.util.*;
import visuAnalor.util.Fichiers.*;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
public class FichiersPraat {
  private enum MotCle {
    TextGrid,
    PitchTier,
    Pitch_1,
    IntensityTier,
    Intensity_2,
    Sound,
    Sound_2,
    Collection,
    IntervalTier,
    TextTier;
    @Override
    public String toString() {
      return super.toString().replace('_', ' ');
    }
    public static MotCle valeur(String mot) {
      return valueOf(mot.replace(' ', '_'));
    }
    public static MotCle valeur(TypeTire type) {
      switch (type) {
        case INTERVALLE:
          return IntervalTier;
        case POINT:
          return TextTier;
        default:
          throw new IllegalArgumentException("type de tire oublié "
                  +"dans un switch");
      }
    }
  }

  private static class IntervalleAnnote implements Comparable<IntervalleAnnote> {
    private String mot;
    private double borneinf, bornesup;
    @Override
    public int compareTo(IntervalleAnnote o) {
      return Double.compare(borneinf, o.borneinf);
    }
  }

  private static class PointAnnote implements Comparable<PointAnnote> {
    private String mot;
    private double position;
    @Override
    public int compareTo(PointAnnote o) {
      return Double.compare(position, o.position);
    }
  }
  /*
   * IMPORT
   */
  public static void importerPraat(Donnees donnees) {
    File fichier;
    if ((fichier = Fichiers.choisirFichier(Commande.OUVRIR,
            "Choix du fichier Praat", Fichiers.TypeFichier.PRAAT))==null)
      return;
    
    try {
    importerPraat(donnees, fichier);
       } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Erreur de lecture de fichier",
              JOptionPane.ERROR_MESSAGE);
    }
  }
  public static void importerPraat(Donnees donnees, File fichier)
          throws IOException {
    FileInputStream fic;
    try {
      fic = new FileInputStream(fichier);
    } catch (FileNotFoundException ex) {
      throw new IOException(ex.getLocalizedMessage());
    }
    byte[] cars = new byte[2];
    if (fic.read(cars)!=cars.length)
      throw new IOException("Fin de fichier inattendue");
    if (cars[0]=='o'&&cars[1]=='o') lireFichierBinaire(donnees, fic);
    else if (cars[0]==-2&&cars[1]==-1) {
      lireFichierTexte(donnees, fic, "UTF-16");
    } else if (cars[0]=='F'&&cars[1]=='i')
      lireFichierTexte(donnees, fic, "UTF-8");
    else throw new IOException("Format non reconnu d'en-tête Praat");
    fic.close();
  }
  public static void importerPraatCourbeLissee(PanneauEditeur editeur) {
    File fichier;
    FileInputStream fic;
Donnees donnees = new Donnees();
    if ((fichier = Fichiers.choisirFichier(Commande.OUVRIR,
            "Choix du fichier de la courbe lissée",
            Fichiers.TypeFichier.PRAAT_F0))==null) return;
    try {
      importerPraat(donnees, fichier);
    } catch (IOException ex) {
      GMessages.impossibiliteAction("erreur de lecture de fichier : "+
              ex.getLocalizedMessage());
    }
    editeur.newF0Lissee(donnees.getCourbeF0());
  }
  public static void importerPraatProsogramme(PanneauEditeur editeur, Tire syll) {
    File fichier;
    FileInputStream fic;
    double[] tlims;
    double[][] pts;
    if ((fichier = Fichiers.choisirFichier(Commande.OUVRIR,
            "Choix du fichier prosogramme",
            Fichiers.TypeFichier.PRAAT_F0_PITCHTIER))==null) return;
    try {
      try {
        fic = new FileInputStream(fichier);
      } catch (FileNotFoundException ex) {
        throw new IOException(ex.getLocalizedMessage());
      }
      BufferedReader input = new BufferedReader(
              new InputStreamReader(fic, "ISO-8859-1"));
      verifierPremiereLigneTexte(input);
      if (lireMotCleTexte(input)!=MotCle.PitchTier)
        throw new IOException("Le fichier n'est pas de type PitchTier");
      tlims = lireTLimsTexte(input);
      pts = lireTireFonctionTexte(input);
      verifierFinTexte(input);
      fic.close();
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Erreur de lecture de fichier",
              JOptionPane.ERROR_MESSAGE);
      return;
    }
    Prosogramme proso = new Prosogramme(tlims, pts, syll);
    editeur.newProsogramme(proso);
  }
  /*
   * IMPORT - FORMAT TEXTE
   */
  private final static char[] charsBuffer = new char[1000];
  private static int lgBuffer;
  private static void lireFichierTexte(Donnees donnees, FileInputStream fic,
          String codage) throws IOException {
    BufferedReader input = new BufferedReader(new InputStreamReader(fic, codage));
    verifierPremiereLigneTexte(input);
    MotCle classe = lireMotCleTexte(input);
    lireObjetTexte(classe, donnees, input);
    verifierFinTexte(input);
  }
  private static void lireObjetTexte(MotCle classe, Donnees donnees,
          BufferedReader input) throws IOException {
    double[] tlims;
    double[][] pts;
    switch (classe) {
      case Pitch_1:
        tlims = lireTLimsTexte(input);
        pts = lirePitchTexte(input);
        donnees.setCourbeF0(creerCourbeF0(tlims, pts));
        return;
      case PitchTier:
        tlims = lireTLimsTexte(input);
        pts = lireTireFonctionTexte(input);
        donnees.setCourbeF0(creerCourbeF0(tlims, pts));
        return;
      case IntensityTier:
        tlims = lireTLimsTexte(input);
        pts = lireTireFonctionTexte(input);
        donnees.setCourbeIntensite(creerCourbeIntensite(tlims, pts));
        return;
      case Intensity_2:
        tlims = lireTLimsTexte(input);
        pts = lireIntensiteTexte(input);
        donnees.setCourbeIntensite(creerCourbeIntensite(tlims, pts));
        return;
      case TextGrid:
        tlims = lireTLimsTexte(input);
        String exists = lireEnumSuivantTexte(input);
        int nbtires = lireEntierSuivantTexte(input);
        Tire[] tires = new Tire[nbtires];
        for (int i = 0; i<nbtires; i++)
          tires[i] = lireTireTexte(donnees, input);
        donnees.ajouterTires(tires);
        return;
      case Sound:
      case Sound_2:
        tlims = lireTLimsTexte(input);
        int nbtps = lireEntierSuivantTexte(input);
        lireDecimalSuivantTexte(input); // dx 
        lireDecimalSuivantTexte(input); // x1 
        lireDecimalSuivantTexte(input); // ymin 
        lireDecimalSuivantTexte(input); // ymax
        int nbcanaux = lireEntierSuivantTexte(input);
        lireDecimalSuivantTexte(input); // dy 
        lireDecimalSuivantTexte(input); // y1
        int c;
        while (Character.isWhitespace(c = input.read()));
        int nblignes = c=='z' ? (nbtps+1)*nbcanaux+1 : nbtps*nbcanaux;
        for (int i = 0; i<nblignes; i++) if (input.readLine()==null)
            throw new IOException("fin de fichier inattendue");
        GMessages.impossibiliteAction("Attention : les données Praat de type"
                +" \"Sound\" ne sont pas prises en compte\nIl faut charger"
                +" directement le fichier .wav (menu : Ouvrir un fichier son)");
        return;
      case Collection:
        int nbobjs = lireEntierSuivantTexte(input);
        for (int i = 0; i<nbobjs; i++) {
          MotCle classeobji = lireMotCleTexte(input);
          String nomobji = lireChaineSuivanteTexte(input);
          lireObjetTexte(classeobji, donnees, input);
        }
        return;
    }
  }
  private static void verifierPremiereLigneTexte(BufferedReader input)
          throws IOException {
    String ligne = input.readLine();
    if (ligne==null) throw new IOException("fin de fichier inattendue");
    if (!ligne.contains("\"ooTextFile\""))
      throw new IOException("Type de fichier non reconnu");
  }
  private static void verifierFinTexte(BufferedReader input)
          throws IOException {
    while (true) {
      int c = input.read();
      if (c==-1) return;
      if (!Character.isWhitespace(c))
        throw new IOException("Fichier non lu en entier");
    }
  }
  private static MotCle lireMotCleTexte(BufferedReader input)
          throws IOException {
    String mot = lireChaineSuivanteTexte(input);
    try {
      return MotCle.valeur(mot);
    } catch (IllegalArgumentException ex) {
      throw new IOException("Type de fichier Praat non traité\n"+ex);
    }
  }

  enum ClasseToken {
    NOMBRE, CHAINE, ENUM;
  }
  private static String lireChaineSuivanteTexte(BufferedReader input)
          throws IOException {
    ClasseToken classe = lireTokenSuivantTexte(input);
    if (classe!=ClasseToken.CHAINE)
      throw new IOException("Token de classe inattendue : "+classe.toString());
    return new String(charsBuffer, 0, lgBuffer);
  }
  private static String lireEnumSuivantTexte(BufferedReader input)
          throws IOException {
    ClasseToken classe = lireTokenSuivantTexte(input);
    if (classe!=ClasseToken.ENUM)
      throw new IOException("Token de classe inattendue : "+classe.toString());
    return new String(charsBuffer, 0, lgBuffer);
  }
  private static double lireDecimalSuivantTexte(BufferedReader input)
          throws IOException {
    ClasseToken classe = lireTokenSuivantTexte(input);
    if (classe!=ClasseToken.NOMBRE)
      throw new IOException("Token de classe inattendue : "+classe.toString());
    String nb = new String(charsBuffer, 0, lgBuffer);
    try {
      return Double.parseDouble(nb);
    } catch (NumberFormatException ex) {
      throw new IOException("Format numérique incorrect\n"+ex);
    }
  }
  private static int lireEntierSuivantTexte(BufferedReader input)
          throws IOException {
    ClasseToken classe = lireTokenSuivantTexte(input);
    if (classe!=ClasseToken.NOMBRE)
      throw new IOException("Token de classe inattendue : "+classe.toString());
    String nb = new String(charsBuffer, 0, lgBuffer);
    try {
      return Integer.parseInt(nb);
    } catch (NumberFormatException ex) {
      throw new IOException("Format numérique incorrect\n"+ex);
    }
  }
  private static ClasseToken lireTokenSuivantTexte(BufferedReader input)
          throws IOException {
    lgBuffer = 0;
    boolean dansCrochets = false;
    boolean dansMot = false;
    ClasseToken classe = null;
    while (true) {
      int c = input.read();
      if (classe==null) {
        if (c<0) throw new IOException("Fin de fichier inattendue");
        if (dansCrochets) {
          if (c==']') dansCrochets = false;
          continue;
        }
        if (c=='[') dansCrochets = true;
        else if (c=='<') classe = ClasseToken.ENUM;
        else if (!dansMot&&(Character.isDigit(c)||c=='-')) {
          classe = ClasseToken.NOMBRE;
          charsBuffer[lgBuffer++] = (char) c;
        } else if (c=='"') classe = ClasseToken.CHAINE;
        dansMot = !Character.isWhitespace(c);
        continue;
      } else switch (classe) {
          case NOMBRE:
            if (c<0||Character.isWhitespace(c)) return classe;
            break;
          case ENUM:
            if (c=='>') return classe;
            break;
          case CHAINE:
            if (c=='"') return classe;
            break;
        }
      charsBuffer[lgBuffer++] = (char) c;
    }
  }
  private static double[] lireTLimsTexte(BufferedReader input)
          throws IOException {
    double[] tlims = new double[2];
    tlims[0] = lireDecimalSuivantTexte(input);
    tlims[1] = lireDecimalSuivantTexte(input);
    return tlims;
  }
  private static double[][] lireTireFonctionTexte(BufferedReader input)
          throws IOException {
    int nb = lireEntierSuivantTexte(input);
    double[][] points = new double[2][nb];
    for (int i = 0; i<nb; i++) {
      points[0][i] = lireDecimalSuivantTexte(input);
      points[1][i] = lireDecimalSuivantTexte(input);
    }
    return points;
  }
  private static double[][] lirePitchTexte(BufferedReader input)
          throws IOException {
    int nb = lireEntierSuivantTexte(input);
    double delta = lireDecimalSuivantTexte(input);
    double x1 = lireDecimalSuivantTexte(input);
    double plafond = lireDecimalSuivantTexte(input);
    int maxcandidats = lireEntierSuivantTexte(input);
    if (maxcandidats!=1)
      throw new IOException("Format Pitch : un seul candidat autorisé");
    double[][] points = new double[2][nb];
    for (int i = 0; i<nb; i++) {
      points[0][i] = x1+i*delta;
      lireDecimalSuivantTexte(input); // intensity 
      lireEntierSuivantTexte(input);// nCandidates
      points[1][i] = lireDecimalSuivantTexte(input);
      lireDecimalSuivantTexte(input); // strength
    }
    return points;
  }
  private static double[][] lireIntensiteTexte(BufferedReader input)
          throws IOException {
    int nb = lireEntierSuivantTexte(input);
    double delta = lireDecimalSuivantTexte(input);
    double x1 = lireDecimalSuivantTexte(input);
    lireDecimalSuivantTexte(input); // ymin 
    lireDecimalSuivantTexte(input); // ymax
    int nby = lireEntierSuivantTexte(input);
    if (nby!=1)
      throw new IOException("FormatIntensity : une seule dimension autorisée");
    lireDecimalSuivantTexte(input); // dy
    lireDecimalSuivantTexte(input); // y1
    double[][] points = new double[2][nb];
    for (int i = 0; i<nb; i++) {
      points[0][i] = x1+i*delta;
      points[1][i] = lireDecimalSuivantTexte(input);
    }
    return points;
  }
  private static Tire lireTireTexte(Donnees donnees, BufferedReader input)
          throws IOException {
    MotCle type = lireMotCleTexte(input);
    String nom = lireChaineSuivanteTexte(input);
    double[] tlims = lireTLimsTexte(input);
    int nbmots = lireEntierSuivantTexte(input);
    switch (type) {
      case TextTier:
        PointAnnote[] ptannots = lireTirePointTexte(input, nbmots);
        return creerPointTire(donnees, nom, tlims, ptannots);
      case IntervalTier:
        IntervalleAnnote[] intannots = lireTireIntervalleTexte(input, nbmots);
        return creerIntervalleTire(donnees, nom, tlims, intannots);
      default:
        throw new IOException("Type de tire non traité");
    }
  }
  private static PointAnnote[] lireTirePointTexte(BufferedReader input,
          int nbmots) throws IOException {
    PointAnnote[] annots = new PointAnnote[nbmots];
    for (int i = 0; i<nbmots; i++) {
      annots[i] = new PointAnnote();
      annots[i].position = lireDecimalSuivantTexte(input);
      annots[i].mot = lireChaineSuivanteTexte(input);
    }
    return annots;
  }
  private static IntervalleAnnote[] lireTireIntervalleTexte(
          BufferedReader input, int nbmots) throws IOException {
    IntervalleAnnote[] annots = new IntervalleAnnote[nbmots];
    for (int i = 0; i<nbmots; i++) {
      annots[i] = new IntervalleAnnote();
      annots[i].borneinf = lireDecimalSuivantTexte(input);
      annots[i].bornesup = lireDecimalSuivantTexte(input);
      annots[i].mot = lireChaineSuivanteTexte(input);
    }
    return annots;
  }
  /*
   * IMPORT - FORMAT BINAIRE
   */
  private final static byte[] bytesBuffer = new byte[1000];
  private static void lireFichierBinaire(Donnees donnees, FileInputStream fic)
          throws IOException {
    DataInputStream input = new DataInputStream(fic);
    verifierDebutBinaire(input);
    lireObjetBinaire(lireMotCleBinaire(input), donnees, input);
    verifierFinBinaire(input);
  }
  private static void lireObjetBinaire(MotCle classe, Donnees donnees,
          DataInputStream input) throws IOException {
    double[] tlims;
    double[][] pts;
    switch (classe) {
      case Pitch_1:
        tlims = lireTLimsBinaire(input);
        pts = lirePitchBinaire(input);
        donnees.setCourbeF0(creerCourbeF0(tlims, pts));
        return;
      case PitchTier:
        tlims = lireTLimsBinaire(input);
        pts = lireTireFonctionBinaire(input);
        donnees.setCourbeF0(creerCourbeF0(tlims, pts));
        return;
      case IntensityTier:
        tlims = lireTLimsBinaire(input);
        pts = lireTireFonctionBinaire(input);
        donnees.setCourbeIntensite(new CourbeIntensite(tlims, pts));
        return;
      case Intensity_2:
        tlims = lireTLimsBinaire(input);
        pts = lireIntensiteBinaire(input);
        donnees.setCourbeIntensite(new CourbeIntensite(tlims, pts));
        return;
      case TextGrid:
        tlims = lireTLimsBinaire(input);
        byte exists = input.readByte();
        int nbtires = input.readInt();
        Tire[] tires = new Tire[nbtires];
        for (int i = 0; i<nbtires; i++)
          tires[i] = lireTireBinaire(donnees, input);
        donnees.ajouterTires(tires);
        return;
      case Sound:
      case Sound_2:
        tlims = lireTLimsBinaire(input);
        int nbtps = input.readInt();
        input.skipBytes(32); // dx, x1, ymin et ymax
        int nbcanaux = input.readInt();
        input.skipBytes(16); // dy et y1
        input.skipBytes(nbtps*nbcanaux*8);
        GMessages.impossibiliteAction("Attention : les données Praat de type"
                +" \"Sound\" ne sont pas prises en compte\nIl faut charger"
                +" directement le fichier .wav (menu : Ouvrir un fichier son)");
        return;
      case Collection:
        int nbobjs = input.readInt();
        for (int i = 0; i<nbobjs; i++) {
          MotCle classeobji = lireMotCleBinaire(input);
          String nomobji = lireChaineUTFBinaire(input);
          lireObjetBinaire(classeobji, donnees, input);
        }
        return;
    }
  }
  private static void verifierDebutBinaire(DataInputStream input)
          throws IOException {
    String mot = "BinaryFile";
    int lg = mot.length();
    if (input.read(bytesBuffer, 0, lg)!=lg)
      throw new IOException("Fin de fichier inattendue");
    if (!mot.equals(new String(bytesBuffer, 0, lg, "US-ASCII")))
      throw new IOException("Début de fichier binaire inattendu");
  }
  private static void verifierFinBinaire(DataInputStream input)
          throws IOException {
    if (input.read()!=-1)
      throw new IOException("Fichier non lu en entier");
  }
  private static MotCle lireMotCleBinaire(DataInputStream input)
          throws IOException {
    int lg = input.read();
    if (lg<=0) throw new IOException("fin de fichier inattendue");
    if (input.read(bytesBuffer, 0, lg)!=lg)
      throw new IOException("fin de fichier inattendue");
    try {
      return MotCle.valeur(new String(bytesBuffer, 0, lg, "US-ASCII"));
    } catch (IllegalArgumentException ex) {
      throw new IOException("Type de fichier Praat non traité");
    }
  }
  private static String lireChaineUTFBinaire(DataInputStream input)
          throws IOException {
    int lg = input.readShort();
    if (lg==-1) {
      lg = 2*input.readUnsignedShort();
      if (input.read(bytesBuffer, 0, lg)!=lg)
        throw new IOException("fin de fichier inattendue");
      return new String(bytesBuffer, 0, lg, "UTF-16BE");
    } else {
      if (input.read(bytesBuffer, 0, lg)!=lg)
        throw new IOException("fin de fichier inattendue");
      return new String(bytesBuffer, 0, lg, "US-ASCII");
    }
  }
  private static double[] lireTLimsBinaire(DataInputStream input)
          throws IOException {
    double[] tlims = new double[2];
    tlims[0] = input.readDouble();
    tlims[1] = input.readDouble();
    return tlims;
  }
  private static double[][] lireTireFonctionBinaire(DataInputStream input)
          throws IOException {
    int nb = input.readInt();
    double[][] points = new double[2][nb];
    for (int i = 0; i<nb; i++) {
      points[0][i] = input.readDouble();
      points[1][i] = input.readDouble();
    }
    return points;
  }
  private static double[][] lirePitchBinaire(DataInputStream input)
          throws IOException {
    int nb = input.readInt();
    double delta = input.readDouble();
    double x1 = input.readDouble();
    double plafond = input.readDouble();
    int maxcandidats = input.readUnsignedShort();
    if (maxcandidats!=1)
      throw new IOException("Format Pitch : un seul candidat autorisé");
    double[][] points = new double[2][nb];
    for (int i = 0; i<nb; i++) {
      points[0][i] = x1+i*delta;
      input.skipBytes(12); // intensity + nCandidates
      points[1][i] = input.readDouble();
      input.skipBytes(8);  // strength
    }
    return points;
  }
  private static double[][] lireIntensiteBinaire(DataInputStream input)
          throws IOException {
    int nb = input.readInt();
    double delta = input.readDouble();
    double x1 = input.readDouble();
    input.skipBytes(16); // ymin et ymax
    int nby = input.readInt();
    if (nby!=1)
      throw new IOException("FormatIntensity : une seule dimension autorisée");
    input.skipBytes(16); // dy et y1
    double[][] points = new double[2][nb];
    for (int i = 0; i<nb; i++) {
      points[0][i] = x1+i*delta;
      points[1][i] = input.readDouble();
    }
    return points;
  }
  private static Tire lireTireBinaire(Donnees donnees, DataInputStream input)
          throws IOException {
    MotCle type = lireMotCleBinaire(input);
    String nom = lireChaineUTFBinaire(input);
    double[] tlims = lireTLimsBinaire(input);
    int nbmots = input.readInt();
    switch (type) {
      case TextTier:
        PointAnnote[] ptannots = lireTirePointBinaire(input, nbmots);
        return creerPointTire(donnees, nom, tlims, ptannots);
      case IntervalTier:
        IntervalleAnnote[] intannots = lireTireIntervalleBinaire(input, nbmots);
        return creerIntervalleTire(donnees, nom, tlims, intannots);
      default:
        throw new IOException("Type de tire non traité");
    }
  }
  private static PointAnnote[] lireTirePointBinaire(DataInputStream input,
          int nbmots) throws IOException {
    PointAnnote[] annots = new PointAnnote[nbmots];
    for (int i = 0; i<nbmots; i++) {
      annots[i] = new PointAnnote();
      annots[i].position = input.readDouble();
      annots[i].mot = lireChaineUTFBinaire(input);
    }
    return annots;
  }
  private static IntervalleAnnote[] lireTireIntervalleBinaire(
          DataInputStream input, int nbmots) throws IOException {
    IntervalleAnnote[] annots = new IntervalleAnnote[nbmots];
    for (int i = 0; i<nbmots; i++) {
      annots[i] = new IntervalleAnnote();
      annots[i].borneinf = input.readDouble();
      annots[i].bornesup = input.readDouble();
      annots[i].mot = lireChaineUTFBinaire(input);
    }
    return annots;
  }
  /*
   * IMPORT - CREATION DES OBJETS ANALOR
   */
  final static double PRECISION_TEMPORELLE = 0.001;
  private static Tire creerPointTire(Donnees donnees, String nom,
          double[] tlims, PointAnnote[] annots) {
    Arrays.sort(annots);
    ArrayList<Integer> doublons = new ArrayList<Integer>();
    for (int i = 1; i<annots.length; i++)
      if (annots[i].position<annots[i-1].position+PRECISION_TEMPORELLE)
        doublons.add(i);
    String[] mots = new String[annots.length-doublons.size()];
    double[] bornes = new double[annots.length-doublons.size()];
    int i1 = 0, j = 0;
    for (int i2 : doublons) {
      for (int i = i1; i<i2; i++) {
        mots[j] = annots[i].mot.trim();
        bornes[j++] = annots[i].position;
      }
      i1 = i2+1;
    }
    for (int i = i1; i<annots.length; i++) {
      mots[j] = annots[i].mot.trim();
      bornes[j++] = annots[i].position;
    }
    return new Tire(donnees, TypeTire.POINT, nom.trim(), tlims, mots, bornes, "");
  }
  private static Tire creerIntervalleTire(Donnees donnees, String nom,
          double[] tlims, IntervalleAnnote[] annots) {
    Arrays.sort(annots);
    ArrayList<Integer> trous = new ArrayList<Integer>();
    if (annots[0].borneinf>tlims[0]) trous.add(0);
    for (int i = 1; i<annots.length; i++)
      if (annots[i-1].bornesup<annots[i].borneinf-PRECISION_TEMPORELLE)
        trous.add(i);
    if (annots[annots.length-1].bornesup<tlims[1]-PRECISION_TEMPORELLE)
      trous.add(annots.length);
    String[] mots = new String[annots.length+trous.size()];
    double[] bornes = new double[annots.length+trous.size()+1];
    int i1 = 0, j = 0;
    for (int i2 : trous) {
      if (i2==0) {
        mots[j] = "";
        bornes[j++] = tlims[0];
        continue;
      }
      for (int i = i1; i<i2; i++) {
        mots[j] = annots[i].mot.trim();
        bornes[j++] = annots[i].borneinf;
      }
      mots[j] = "";
      bornes[j++] = annots[i2-1].bornesup;
      i1 = i2;
    }
    for (int i = i1; i<annots.length; i++) {
      mots[j] = annots[i].mot.trim();
      bornes[j++] = annots[i].borneinf;
    }
    bornes[mots.length] = tlims[1];
    return new Tire(donnees, TypeTire.INTERVALLE, nom.trim(), tlims, mots, bornes, "");
  }
  private static CourbeF0 creerCourbeF0(double[] tlims, double[][] pts) {
    double[][] points = supprimerZeros(pts);
    points[1] = calculerLog(points[1]);
    double deltaT = tlims[1]-tlims[0];
    for (int i = 1; i<points[0].length; i++)
      deltaT = Math.min(deltaT, points[0][i]-points[0][i-1]);
    ArrayList<Integer> coupes = new ArrayList<Integer>();
    coupes.add(0);
    for (int i = 1; i<points[0].length; i++)
      if (points[0][i]-points[0][i-1]>deltaT*1.25) coupes.add(i);
    coupes.add(points[0].length);
    return new CourbeF0(tlims, deltaT, points, coupes);
  }
  private static CourbeIntensite creerCourbeIntensite(double[] tlims,
          double[][] pts) {
    return new CourbeIntensite(tlims, pts);
  }
  private static final double ZERO_HERTZ = 0.5;
  private static double[][] supprimerZeros(double[][] pts) {
    ArrayList<Integer> zeros = new ArrayList<Integer>();
    int nbpts = pts[0].length;
    for (int i = 0; i<nbpts; i++) if (pts[1][i]<ZERO_HERTZ) zeros.add(i);
    if (zeros.isEmpty()) return pts;
    double[][] points = new double[2][nbpts-zeros.size()];
    int i = 0, z = 0;
    while (z<zeros.size()) {
      int lgc = zeros.get(z)-i;
      System.arraycopy(pts[0], i, points[0], i-z, lgc);
      System.arraycopy(pts[1], i, points[1], i-z, lgc);
      i += lgc+1;
      z++;
    }
    System.arraycopy(pts[0], i, points[0], i-z, nbpts-i);
    System.arraycopy(pts[1], i, points[1], i-z, nbpts-i);
    return points;
  }
  final private static double LOG2 = Math.log(2);
  private static double[] calculerLog(double[] f) {
    for (int i = 0; i<f.length; i++) f[i] = Math.log(f[i])/LOG2;
    return f;
  }
  /*
   * EXPORT
   */
  private static CourbeF0 f0;
  private static CourbeIntensite I;
  private static Tire[] tires;
  public static void exporterPraat(JFrame fenetre, Donnees donnees) {
    if (donnees.isVide()) return;
    ArrayList<TypeFichier> objets = new ArrayList<TypeFichier>();
    if ((f0 = donnees.getCourbeF0())!=null) {
      objets.add(TypeFichier.PRAAT_F0_PITCHTIER);
      objets.add(TypeFichier.PRAAT_F0_PITCH);
    }
    if ((I = donnees.getCourbeIntensite())!=null) {
      objets.add(TypeFichier.PRAAT_INTENSITYTIER);
      if (I.isDeltaTRegulier()) objets.add(TypeFichier.PRAAT_INTENSITY);
    }
    if ((tires = donnees.getTires()).length>0) {
      objets.add(TypeFichier.PRAAT_TEXTGRID);
    }
    VisuObjetsAExporter saisieExport = new VisuObjetsAExporter(fenetre, objets);
    saisieExport.setVisible(true);
  }
  public static void exporterPraatF0Lissee(JFrame fenetre, CourbeF0 f0lissee) {
    if ((f0 = f0lissee)==null) return;
    ArrayList<TypeFichier> objets = new ArrayList<TypeFichier>();
    objets.add(TypeFichier.PRAAT_F0LISSEE_PITCHTIER);
    objets.add(TypeFichier.PRAAT_F0LISSEE_PITCH);
    VisuObjetsAExporter saisieExport = new VisuObjetsAExporter(fenetre, objets);
    saisieExport.setVisible(true);
  }
 public static void exporterPraatTire(Tire tire) {
    tires = new Tire[]{tire};
    ArrayList<TypeFichier> objets = new ArrayList<TypeFichier>();
    objets.add(TypeFichier.PRAAT_TEXTGRID);
    VisuObjetsAExporter saisieExport = new VisuObjetsAExporter(null, objets);
    saisieExport.setVisible(true);
  }
  private static void annulerExporter() {
    f0 = null;
    I = null;
    tires = null;
  }
  private static void exporter(ArrayList<TypeFichier> objets, CodagePraat codage) {
    int nbobjs = objets.size();
    if (nbobjs==0) return;
    TypeFichier typeFichier;
    if (nbobjs>1) typeFichier = TypeFichier.PRAAT_COLLECTION;
    else typeFichier = objets.get(0);
    File fic;
    if ((fic = Fichiers.choisirFichier(Commande.ENREGISTRER,
            "Nom du fichier Praat", typeFichier))==null) return;
    String nomDonnees = Fichiers.extraireNom(fic);
    switch (codage) {
      case BINAIRE:
        exporterBinaire(fic, nomDonnees, objets);
        return;
      case SHORT:
        exporterTexte(fic, nomDonnees, objets, false);
        return;
      case TEXTE:
        exporterTexte(fic, nomDonnees, objets, true);
        return;
    }
  }
  /*
   * EXPORT - FORMAT TEXTE
   */
  private static void exporterTexte(File fic, String nomDonnees,
          ArrayList<TypeFichier> objets, boolean detaille) {
    PrintWriter output = null;
//    String code = objets.contains(TypeFichier.PRAAT_TEXTGRID)
//            ||(objets.size()>1&&!isAscii(nomDonnees)) ? "UTF-16" : "ISO-8859-1";
    try {
      output = new PrintWriter(fic, "UTF-8");
      ecrireFichierTexte(output, nomDonnees, objets, detaille);
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Erreur d'écriture de fichier",
              JOptionPane.ERROR_MESSAGE);
    } finally {
      if (output!=null) output.close();
    }
  }
  private final static String TAB = "    ";
  private static void ecrireFichierTexte(PrintWriter output, String nomDonnees,
          ArrayList<TypeFichier> objets, boolean detaille) throws IOException {
    output.println("File type = \"ooTextFile\"");
    String tabs = "";
    int nbobjs = objets.size();
    if (nbobjs>1) {
      output.println("Object class = \"Collection\"");
      output.println();
      if (detaille) output.print("size = ");
      output.println(nbobjs);
      if (detaille) output.println("item []:");
      if (detaille) tabs = TAB;
    }
    for (int i = 0; i<nbobjs; i++) {
      TypeFichier obj = objets.get(i);
      if (nbobjs>1) {
        if (detaille) {
          output.println(tabs+"item ["+(i+1)+"]:");
          tabs = TAB+TAB;
          output.print(tabs+"class = ");
        }
      } else output.print("Object class = ");
      switch (obj) {
        case PRAAT_F0_PITCH:
        case PRAAT_F0LISSEE_PITCH:
          output.println("\""+MotCle.Pitch_1.toString()+"\"");
          if (nbobjs>1) {
            if (detaille) output.print(tabs+"name = ");
            output.println("\""+nomDonnees+"_Hz\"");
          } else output.println();
          ecrireTLimsTexte(output, detaille, tabs, f0.getTmin(), f0.getTmax());
          double t0 = calculerT0Pitch();
          double[] vals = calculerValsPitch(t0);
          ecrirePitchTexte(output, detaille, tabs, t0, f0.getDeltaT(),
                  calculerPlafond(vals), vals);
          break;
        case PRAAT_F0_PITCHTIER:
        case PRAAT_F0LISSEE_PITCHTIER:
          output.println("\""+MotCle.PitchTier.toString()+"\"");
          if (nbobjs>1) {
            if (detaille) output.print(tabs+"name = ");
            output.println("\""+nomDonnees+"_Hz\"");
          } else output.println();
          ecrireTLimsTexte(output, detaille, tabs, f0.getTmin(), f0.getTmax());
          ecrireTireFonctionTexte(output, detaille, tabs,
                  calculerPtsPitchTire());
          break;
        case PRAAT_INTENSITY:
          output.println("\""+MotCle.Intensity_2.toString()+"\"");
          if (nbobjs>1) {
            if (detaille) output.print(tabs+"name = ");
            output.println("\""+nomDonnees+"_db\"");
          } else output.println();
          ecrireTLimsTexte(output, detaille, tabs, I.getTmin(), I.getTmax());
          ecrireIntensiteTexte(output, detaille, tabs, I.getPoints()[0][0],
                  I.getDeltaT(), I.getPoints()[1]);
          break;
        case PRAAT_INTENSITYTIER:
          output.println("\""+MotCle.IntensityTier.toString()+"\"");
          if (nbobjs>1) {
            if (detaille) output.print(tabs+"name = ");
            output.println("\""+nomDonnees+"_db\"");
          } else output.println();
          ecrireTLimsTexte(output, detaille, tabs, I.getTmin(), I.getTmax());
          ecrireTireFonctionTexte(output, detaille, tabs, I.getPoints());
          break;
        case PRAAT_TEXTGRID:
          output.println("\""+MotCle.TextGrid.toString()+"\"");
          if (nbobjs>1) {
            if (detaille) output.print(tabs+"name = ");
            output.println("\""+nomDonnees+"_Grid\"");
          } else output.println();
          ecrireTLimsTexte(output, detaille, tabs, calculerTminTires(tires),
                  calculerTmaxTires(tires));
          if (detaille) output.print(tabs+"tiers? ");
          output.println("<exists>");
          if (detaille) output.print(tabs+"size = ");
          output.println(tires.length);
          if (detaille) {
            output.println("item[]:");
            tabs += TAB;
          }
          for (int j = 0; j<tires.length; j++) {
            if (detaille) output.println(tabs+"item ["+(j+1)+"]:");
            ecrireTireTexte(output, detaille, tabs+TAB, tires[j]);
          }
          break;
        default:
          throw new IllegalArgumentException("Type de fichier praat oublié"
                  +" dans un switch");
      }
    }
  }
  private static void ecrireTLimsTexte(PrintWriter output,
          boolean detaille, String tabs, double tmin, double tmax)
          throws IOException {
    if (detaille) output.print(tabs+"xmin = ");
    output.println(tmin);
    if (detaille) output.print(tabs+"xmax = ");
    output.println(tmax);
  }
  private static void ecrireTireFonctionTexte(PrintWriter output,
          boolean detaille, String tabs, double[][] pts) throws IOException {
    if (detaille) output.print(tabs+"points: size = ");
    output.println(pts[0].length);
    for (int i = 0; i<pts[0].length; i++) {
      if (detaille) {
        output.println(tabs+"points ["+(i+1)+"]:");
        output.print(tabs+TAB+"time = ");
      }
      output.println(pts[0][i]);
      if (detaille) output.print(tabs+TAB+"value = ");
      output.println(pts[1][i]);
    }
  }
  private static void ecrirePitchTexte(PrintWriter output, boolean detaille,
          String tabs, double t0, double delt, double plafond, double[] vals)
          throws IOException {
    if (detaille) output.print(tabs+"nx = ");
    output.println(vals.length);
    if (detaille) output.print(tabs+"dx = ");
    output.println(delt);
    if (detaille) output.print(tabs+"x1 = ");
    output.println(t0);
    if (detaille) output.print(tabs+"ceiling = ");
    output.println(plafond);
    if (detaille) output.print(tabs+"maxnCandidates = ");
    output.println(1);
    if (detaille) {
      output.println("frame []:");
      tabs += TAB;
    }
    for (int i = 0; i<vals.length; i++) {
      if (detaille) {
        output.println(tabs+"frame ["+(i+1)+"]:");
        output.print(tabs+TAB+"intensity = ");
      }
      output.println(0);
      if (detaille) output.print(tabs+TAB+"nCandidates = ");
      output.println(1);
      if (detaille) {
        output.println(tabs+TAB+"candidate []:");
        output.println(tabs+TAB+TAB+"candidate [1]:");
        output.print(tabs+TAB+TAB+TAB+"frequency = ");
      }
      output.println(vals[i]);
      if (detaille) output.print(tabs+TAB+TAB+TAB+"strength = ");
      output.println(1);
    }
  }
  private static void ecrireIntensiteTexte(PrintWriter output,
          boolean detaille, String tabs, double t0, double delt, double[] vals)
          throws IOException {
//    output.writeInt(vals.length);
//    output.writeDouble(delt);
//    output.writeDouble(t0);
//    output.writeDouble(1); // ymin 
//    output.writeDouble(1); // ymax 
//    output.writeInt(1);  // ny
//    output.writeDouble(1);// dy
//    output.writeDouble(1);  // y1
//    for (double val : vals) output.writeDouble(val);
    if (detaille) output.print(tabs+"nx = ");
    output.println(vals.length);
    if (detaille) output.print(tabs+"dx = ");
    output.println(delt);
    if (detaille) output.print(tabs+"x1 = ");
    output.println(t0);
    if (detaille) output.print(tabs+"ymin = ");
    output.println(1);
    if (detaille) output.print(tabs+"ymax = ");
    output.println(1);
    if (detaille) output.print(tabs+"ny = ");
    output.println(1);
    if (detaille) output.print(tabs+"dy = ");
    output.println(1);
    if (detaille) output.print(tabs+"y1 = ");
    output.println(1);
    if (detaille) {
      output.println("z [] []:");
      tabs += TAB;
      output.println(tabs+"z [1]:");
      tabs += TAB;
    }
    for (int i = 0; i<vals.length; i++) {
      if (detaille) {
        output.print(tabs+"z [1]["+(i+1)+"] = ");
      }
      output.println(vals[i]);
    }
  }
  private static void ecrireTireTexte(PrintWriter output, boolean detaille,
          String tabs, Tire tire) throws IOException {
    MotCle type = MotCle.valeur(tire.getType());
    if (detaille) output.print(tabs+"class = ");
    output.println("\""+type.toString()+"\"");
    if (detaille) output.print(tabs+"name = ");
    output.println("\""+tire.getNom()+"\"");
    ecrireTLimsTexte(output, detaille, tabs, tire.getTmin(), tire.getTmax());
    switch (type) {
      case TextTier:
        ecrireTirePointTexte(output, detaille, tabs, tire.getBornes(),
                tire.getMots());
        return;
      case IntervalTier:
        ecrireTireIntervalleTexte(output, detaille, tabs, tire.getBornes(),
                tire.getMots());
        return;
      default:
        throw new IllegalArgumentException("Type de tire oublié dans un switch");
    }
  }
  private static void ecrireTirePointTexte(PrintWriter output,
          boolean detaille, String tabs, double[] bornes, String[] mots)
          throws IOException {
    if (detaille) output.print(tabs+"points: size = ");
    output.println(mots.length);
    for (int i = 0; i<mots.length; i++) {
      if (detaille) output.println(tabs+"points ["+(i+1)+"]:");
      if (detaille) output.print(tabs+TAB+"time = ");
      output.println(bornes[i]);
      if (detaille) output.print(tabs+TAB+"mark = ");
      output.println("\""+mots[i]+"\"");
    }
  }
  private static void ecrireTireIntervalleTexte(PrintWriter output,
          boolean detaille, String tabs, double[] bornes, String[] mots)
          throws IOException {
    if (detaille) output.print(tabs+"intervals: size = ");
    output.println(mots.length);
    for (int i = 0; i<mots.length; i++) {
      if (detaille) output.println(tabs+"intervals ["+(i+1)+"]:");
      ecrireTLimsTexte(output, detaille, tabs+TAB, bornes[i], bornes[i+1]);
      if (detaille) output.print(tabs+TAB+"text = ");
      output.println("\""+mots[i]+"\"");
    }
  }

  /*
   * EXPORT - FORMAT BINAIRE
   */
  private static void exporterBinaire(File fic, String nomDonnees,
          ArrayList<TypeFichier> objets) {
    DataOutputStream output = null;
    try {
      output = new DataOutputStream(new FileOutputStream(fic));
      ecrireFichierBinaire(output, nomDonnees, objets);
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Erreur d'écriture de fichier",
              JOptionPane.ERROR_MESSAGE);
    } finally {
      if (output!=null)
        try {
          output.close();
        } catch (IOException ex) {
          JOptionPane.showMessageDialog(null, ex, "Erreur de fermeture de fichier",
                  JOptionPane.ERROR_MESSAGE);
        }
    }
  }
  private static void ecrireFichierBinaire(DataOutputStream output,
          String nomDonnees, ArrayList<TypeFichier> objets) throws IOException {
    output.writeBytes("ooBinaryFile");
    int nbobjs = objets.size();
    if (nbobjs>1) {
      ecrireMotCleBinaire(output, MotCle.Collection);
      output.writeInt(nbobjs);
    }
    for (TypeFichier obj : objets) {
      switch (obj) {
        case PRAAT_F0_PITCH:
        case PRAAT_F0LISSEE_PITCH:
          ecrireMotCleBinaire(output, MotCle.Pitch_1);
          if (nbobjs>1) ecrireChaineUTFBinaire(output, nomDonnees+"_Hz");
          ecrireTLimsBinaire(output, f0.getTmin(), f0.getTmax());
          double t0 = calculerT0Pitch();
          double[] vals = calculerValsPitch(t0);
          ecrirePitchBinaire(output, t0, f0.getDeltaT(), calculerPlafond(vals),
                  vals);
          break;
        case PRAAT_F0_PITCHTIER:
        case PRAAT_F0LISSEE_PITCHTIER:
          ecrireMotCleBinaire(output, MotCle.PitchTier);
          if (nbobjs>1) ecrireChaineUTFBinaire(output, nomDonnees+"_Hz");
          ecrireTLimsBinaire(output, f0.getTmin(), f0.getTmax());
          ecrireTireFonctionBinaire(output, calculerPtsPitchTire());
          break;
        case PRAAT_INTENSITY:
          ecrireMotCleBinaire(output, MotCle.Intensity_2);
          if (nbobjs>1) ecrireChaineUTFBinaire(output, nomDonnees+"_db");
          ecrireTLimsBinaire(output, I.getTmin(), I.getTmax());
          ecrireIntensiteBinaire(output, I.getPoints()[0][0], I.getDeltaT(),
                  I.getPoints()[1]);
          break;
        case PRAAT_INTENSITYTIER:
          ecrireMotCleBinaire(output, MotCle.IntensityTier);
          if (nbobjs>1) ecrireChaineUTFBinaire(output, nomDonnees+"_db");
          ecrireTLimsBinaire(output, I.getTmin(), I.getTmax());
          ecrireTireFonctionBinaire(output, I.getPoints());
          break;
        case PRAAT_TEXTGRID:
          ecrireMotCleBinaire(output, MotCle.TextGrid);
          if (nbobjs>1) ecrireChaineUTFBinaire(output, nomDonnees+"_Grid");
          ecrireTLimsBinaire(output, calculerTminTires(tires),
                  calculerTmaxTires(tires));
          output.writeByte(1); //exists
          output.writeInt(tires.length);
          for (Tire tire : tires)
            ecrireTireBinaire(output, tire);
          break;
        default:
          throw new IllegalArgumentException("Type de fichier praat oublié"
                  +" dans un switch");
      }
    }
  }
  private static void ecrireMotCleBinaire(DataOutputStream output,
          MotCle motcle) throws IOException {
    String mot = motcle.toString();
    output.writeByte(mot.length());
    output.writeBytes(mot);
  }
  private static void ecrireChaineUTFBinaire(DataOutputStream output,
          String chaine) throws IOException {
    if (isAscii(chaine)) {
      output.writeShort(chaine.length());
      output.writeBytes(chaine);
    } else {
      output.writeShort(-1);
      output.writeShort(chaine.length());
      output.writeChars(chaine);
    }
  }
  private static void ecrireTLimsBinaire(DataOutputStream output,
          double tmin, double tmax) throws IOException {
    output.writeDouble(tmin);
    output.writeDouble(tmax);
  }
  private static void ecrireTireFonctionBinaire(DataOutputStream output,
          double[][] pts) throws IOException {
    output.writeInt(pts[0].length);
    for (int i = 0; i<pts[0].length; i++) {
      output.writeDouble(pts[0][i]);
      output.writeDouble(pts[1][i]);
    }
  }
  private static void ecrirePitchBinaire(DataOutputStream output,
          double t0, double deltaT, double plafond, double[] vals)
          throws IOException {
    output.writeInt(vals.length);
    output.writeDouble(deltaT);
    output.writeDouble(t0);
    output.writeDouble(plafond);
    output.writeShort(1); // maxnCandidates
    for (double val : vals) {
      output.writeDouble(0); // intensity 
      output.writeInt(1); //  nCandidates  
      output.writeDouble(val);
      output.writeDouble(1); // strength
    }
  }
  private static void ecrireIntensiteBinaire(DataOutputStream output,
          double t0, double delt, double[] vals) throws IOException {
    output.writeInt(vals.length);
    output.writeDouble(delt);
    output.writeDouble(t0);
    output.writeDouble(1); // ymin 
    output.writeDouble(1); // ymax 
    output.writeInt(1);  // ny
    output.writeDouble(1);// dy
    output.writeDouble(1);  // y1
    for (double val : vals) output.writeDouble(val);
  }
  private static void ecrireTireBinaire(DataOutputStream output, Tire tire)
          throws IOException {
    MotCle type = MotCle.valeur(tire.getType());
    ecrireMotCleBinaire(output, type);
    ecrireChaineUTFBinaire(output, tire.getNom());
    ecrireTLimsBinaire(output, tire.getTmin(), tire.getTmax());
    output.writeInt(tire.getMots().length);
    switch (type) {
      case TextTier:
        ecrireTirePointBinaire(output, tire.getBornes(), tire.getMots());
        return;
      case IntervalTier:
        ecrireTireIntervalleBinaire(output, tire.getBornes(), tire.getMots());
        return;
      default:
        throw new IllegalArgumentException("Type de tire oublié dans un switch");
    }
  }
  private static void ecrireTirePointBinaire(DataOutputStream output,
          double[] bornes, String[] mots) throws IOException {
    for (int i = 0; i<mots.length; i++) {
      output.writeDouble(bornes[i]);
      ecrireChaineUTFBinaire(output, mots[i]);
    }
  }
  private static void ecrireTireIntervalleBinaire(DataOutputStream output,
          double[] bornes, String[] mots) throws IOException {
    for (int i = 0; i<mots.length; i++) {
      output.writeDouble(bornes[i]);
      output.writeDouble(bornes[i+1]);
      ecrireChaineUTFBinaire(output, mots[i]);
    }
  }
  private static boolean isAscii(String chaine) {
    for (int c = 0; c<chaine.length(); c++) if (chaine.charAt(c)>127)
        return false;
    return true;
  }
  /*
   * EXPORT - CALCULS SUR LES OBJETS ANALOR
   */
  private static double[][] calculerPtsPitchTire() {
    double[][] pts = f0.getPoints();
    double[] pts1 = new double[pts[1].length];
    for (int i = 0; i<pts1.length; i++) pts1[i] = Math.pow(2, pts[1][i]);
    return new double[][]{pts[0], pts1};
  }
  private static double calculerT0Pitch() {
    double delT = f0.getDeltaT();
    double tmin = f0.getTmin();
    double t0 = f0.getPoints()[0][0];
    while (t0-delT>=tmin) t0 -= delT;
    return t0;
  }
  private static double[] calculerValsPitch(double t0) {
    double[][] pts = f0.getPoints();
    int nbpts = pts[0].length;
    double[] pts1 = new double[nbpts];
    for (int i = 0; i<pts1.length; i++) pts1[i] = Math.pow(2, pts[1][i]);
    ArrayList<Integer> coupes = f0.getCoupes();
    double delt = f0.getDeltaT();
    int[] nbptsinterp = new int[coupes.size()];
    int nbptsinterptotal = 0;
    double t = t0;
    int nomax = coupes.size()-1;
    for (int no = 0; no<=nomax; no++) {
      double debportion = no==nomax ? f0.getTmax() : pts[0][coupes.get(no)];
      while (t<debportion-delt/2) {
        nbptsinterp[no]++;
        t += delt;
      }
      nbptsinterptotal += nbptsinterp[no];
      if (no<nomax) t += (coupes.get(no+1)-coupes.get(no))*delt;
    }
    double[] vals = new double[nbpts+nbptsinterptotal];
    int i = 0;
    for (int no = 0; no<=nomax; no++) {
      Arrays.fill(vals, i, i+nbptsinterp[no], 0);
      i += nbptsinterp[no];
      if (no<nomax) {
        int nbptsportion = coupes.get(no+1)-coupes.get(no);
        System.arraycopy(pts1, coupes.get(no), vals, i, nbptsportion);
        i += nbptsportion;
      }
    }
    return vals;
  }
  final static private double PLAFOND_STANDARD = 300;
  private static double calculerPlafond(double[] vals) {
    double plafond = Double.NEGATIVE_INFINITY;
    for (double val : vals) if (plafond<val) plafond = val;
    plafond *= 1.1;
    return Math.max(PLAFOND_STANDARD, plafond);

  }
  private static double calculerTminTires(Tire[] tires) {
    double min = Double.POSITIVE_INFINITY;
    for (Tire tire : tires) if (tire.getTmin()<min) min = tire.getTmin();
    return min;
  }
  private static double calculerTmaxTires(Tire[] tires) {
    double max = Double.NEGATIVE_INFINITY;
    for (Tire tire : tires) if (tire.getTmax()>max) max = tire.getTmax();
    return max;
  }
  /*
   * SAISIE EXPORT
   */

  private enum CodagePraat {
    BINAIRE("Binaire"), SHORT("Texte court"), TEXTE("Texte long");
    private String nom;
    CodagePraat(String s) {
      nom = s;
    }
  }
  private static final CodagePraat[] codagesPraat = new CodagePraat[]{
    CodagePraat.BINAIRE, CodagePraat.SHORT, CodagePraat.TEXTE};

  static private class VisuObjetsAExporter extends JDialog {
    private EnumMap<TypeFichier, JCheckBox> saisieObjets;
    private JComboBox saisieCodage;
    private VisuObjetsAExporter(JFrame fenetre, ArrayList<TypeFichier> objets) {
      super(fenetre, "Export format Praat", true);
      JPanel panneauSaisie = new JPanel(new GridLayout(0, 1, 0, 10));
      panneauSaisie.add(new JLabel("Données à exporter :"));
      saisieObjets = new EnumMap<TypeFichier, JCheckBox>(TypeFichier.class);
      for (TypeFichier obj : objets) {
        JCheckBox saisieObj;
        switch (obj) {
          case PRAAT_F0_PITCH:
            saisieObj = new JCheckBox("Fondamentale (format Pitch)", false);
            break;
          case PRAAT_F0_PITCHTIER:
            saisieObj = new JCheckBox("Fondamentale (format PitchTier)", true);
            break;
          case PRAAT_F0LISSEE_PITCH:
            saisieObj = new JCheckBox("Fondamentale lissée (format Pitch)", false);
            break;
          case PRAAT_F0LISSEE_PITCHTIER:
            saisieObj = new JCheckBox("Fondamentale lissée (format PitchTier)", true);
            break;
          case PRAAT_INTENSITY:
            saisieObj = new JCheckBox("Intensité (format Intensity)", false);
            break;
          case PRAAT_INTENSITYTIER:
            saisieObj = new JCheckBox("Intensité (format IntensityTier)", true);
            break;
          case PRAAT_TEXTGRID:
            saisieObj = new JCheckBox("Tires (format TextGrid)", true);
            break;
          default:
            throw new IllegalArgumentException("Type de fichier praat non prévu dans un switch");
        }
        saisieObjets.put(obj, saisieObj);
        panneauSaisie.add(saisieObj);
      }
      Box panneauCodage = Box.createHorizontalBox();
      panneauCodage.add(new JLabel("Codage Praat : "));
      saisieCodage = new JComboBox(new String[]{codagesPraat[0].nom,
                codagesPraat[1].nom, codagesPraat[2].nom});
      saisieCodage.setSelectedIndex(0);
      panneauCodage.add(saisieCodage);
      panneauSaisie.add(panneauCodage);
      Box commandes = Box.createHorizontalBox();
      commandes.add(Box.createHorizontalGlue());
      commandes.add(
              new JButton(new GAction("Annuler") {
        @Override
        public void executer() {
          annulerExporter();
          dispose();
        }
      }));
      commandes.add(
              new JButton(new GAction("Valider") {
        @Override
        public void executer() {
          ArrayList<TypeFichier> objs = new ArrayList<TypeFichier>();
          for (TypeFichier obj : saisieObjets.keySet())
            if (saisieObjets.get(obj).isSelected()) objs.add(obj);
          CodagePraat codage = codagesPraat[saisieCodage.getSelectedIndex()];
          dispose();
          exporter(objs, codage);
        }
      }));
      commandes.add(Box.createHorizontalGlue());
      panneauSaisie.add(commandes);
      setContentPane(panneauSaisie);
      setLocationRelativeTo(fenetre);
      pack();
    }
  }
}
