/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.statistiques;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import org.jfree.chart.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.data.statistics.*;
import org.jfree.data.xy.*;
import org.jfree.ui.*;
import visuAnalor.*;
import visuAnalor.BandeauFonctions.*;
import visuAnalor.donnees.*;
import visuAnalor.donnees.Donnees.*;
import visuAnalor.util.*;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
@SuppressWarnings("serial")
public class VisuStatsF0 extends JFrame implements DonneesListener {
  private Donnees donnees;
  private PanneauEditeur editeur;
  private XYPlot plot;
  private F0Axis axeF0;
  private SimpleHistogramDataset dataset;

  public VisuStatsF0(Container fenetre, Donnees donnees, PanneauEditeur editeur) {
    super("Statistiques f0");
    if (donnees.getCourbeF0()==null) return;
    Point pos = new Point(fenetre.getX()+200, fenetre.getY()+50);
    Dimension dim = new Dimension(Math.max(200, fenetre.getWidth()-400),
            fenetre.getHeight()-30);
    setLocation(pos);
    setSize(dim);
    this.donnees = donnees;
    this.editeur = editeur;
    donnees.addEventListener(this);
    dataset = new SimpleHistogramDataset("");
    dataset.setAdjustForBinSize(false);
    axeF0 = new F0Axis();
    NumberAxis axeFreq = new NumberAxis();
    XYBarRenderer rendu = new XYBarRenderer();
    plot = new XYPlot();
    plot.setDomainAxis(axeF0);
    plot.setRangeAxis(axeFreq);
    plot.setOrientation(PlotOrientation.HORIZONTAL);
    plot.setDataset(dataset);
    plot.setRenderer(rendu);
    rendu.setDrawBarOutline(true);
    rendu.setAutoPopulateSeriesOutlinePaint(false);
    rendu.setBaseOutlinePaint(Color.BLACK);
    rendu.setShadowVisible(false);
    rendu.setAutoPopulateSeriesFillPaint(false);
    rendu.setSeriesPaint(0, Color.BLUE);
    rendu.setBaseSeriesVisibleInLegend(false);
    rendu.setBarPainter(new StandardXYBarPainter());
    rendu.setMargin(0.1);
    JFreeChart chart = new JFreeChart("Statistiques F0", plot);
    ChartStatsF0 graphique = 
            new ChartStatsF0(chart, editeur, donnees.getCourbeF0());
    setContentPane(graphique);
    // Fermeture de la fenêtre
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent evt) {
        fermerFenetre();
      }
    });
    setVisible(true);
    newDonnees();
  }

  private void fermerFenetre() {
    donnees.removeEventListener(this);
    dispose();
  }

  @Override
  public void traiterEventDonnees(MessageDonnes evt) {
    switch (evt.getType()) {
      case CLEAR_DONNEES:
      case NEW_DONNEES:
        fermerFenetre();
        return;
      case MODIF_F0:
      case NEW_F0:
        newDonnees();
        return;
      case NEW_INTENSITE:
      case MODIF_TIRE:
      case MODIF_NOM_TIRE:
      case NEW_SON:
      case ADD_TIRES:
      case SUP_TIRE:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }

  private void newDonnees() {
    dataset.clearObservations();
    dataset.removeAllBins();
    CourbeF0 f0 = donnees.getCourbeF0();
    int nodataf0 = f0.getNoDataF0();
    double[] bornes = f0.calculerBornesEtMoyenne();
    if (bornes==null) {
      fermerFenetre();
      return;
    }
    double borneinf = bornes[0];
    double bornesup = bornes[1];
    double moyenne = bornes[2];
    int inf = (int) Math.ceil((borneinf-moyenne)*12);
    int sup = (int) Math.ceil((bornesup-moyenne)*12);
    for (int i = inf-1; i<sup+1; i++) {
      SimpleHistogramBin bin = new SimpleHistogramBin(moyenne+i/12.,
              moyenne+(i+1)/12., true, false);
      dataset.addBin(bin);
    }
    XYDataset f0dataset = f0.getDataSet();
    for (int i = 0; i<f0dataset.getItemCount(nodataf0); i++) {
      double f = f0dataset.getYValue(nodataf0, i);
      if (!Double.isNaN(f)) dataset.addObservation(f);
    }
    plot.getDomainAxis().setLowerBound(moyenne+(inf-2)/12.);
    plot.getDomainAxis().setUpperBound(moyenne+(sup+1)/12.);
    axeF0.configurerTicks(inf, sup, moyenne);

  }

  private static class ChartStatsF0 extends ChartPanel {
    SimpleHistogramDataset dataset;
    XYPlot plot;
    PopupStatsF0 popup;

    public ChartStatsF0(JFreeChart chart, PanneauEditeur editeur, 
            CourbeF0 f0) {
      super(chart);
      this.popup = new PopupStatsF0(editeur, f0);
      this.plot = (XYPlot) chart.getPlot();
      this.dataset = (SimpleHistogramDataset) plot.getDataset();
      setMouseZoomable(false);
      setPopupMenu(null);
    }

    @Override
    public void mousePressed(MouseEvent e) {
      try {
        if (e.isPopupTrigger()) {
          showPopup(e);
          return;
        }
      } catch (Throwable ex) {
        GMessages.erreurFatale(ex);
      }
//    System.out.println(x1Pressed);
    }

    @Override
    public void mouseReleased(MouseEvent e) {
      try {
        if (e.isPopupTrigger()) {
          showPopup(e);
          return;
        }
      } catch (Throwable ex) {
        GMessages.erreurFatale(ex);
      }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
    }

    @Override
    public void mouseMoved(MouseEvent e) {
    }

    private void showPopup(MouseEvent e) {
      Point2D pt = translateScreenToJava2D(e.getPoint());
      PlotRenderingInfo info = getChartRenderingInfo().getPlotInfo();
      double y = pt.getY();
      Rectangle2D area = info.getDataArea();
      if (!area.contains(pt)) return;
      double f = plot.getDomainAxis().java2DToValue(y, area, RectangleEdge.LEFT);
      if (f<dataset.getStartXValue(0, 0)) return;
      for (int i = 0; i<dataset.getItemCount(0); i++) {
        if (f>=dataset.getEndXValue(0, i)) continue;
        popup.init(dataset.getStartXValue(0, i), dataset.getEndXValue(0, i));
        popup.show(e.getComponent(), e.getX(), e.getY());
        return;
      }
    }
  }

  private static class PopupStatsF0 extends JPopupMenu {
    double borneinf, bornesup;
    PanneauEditeur editeur;
    CourbeF0 f0;

    private PopupStatsF0(PanneauEditeur edit, CourbeF0 f) {
      super();
      
       this.editeur = edit;
      this.f0 = f;
      add("<html><I>Parcourir les valeurs...").setEnabled(false);
      add(new GAction("...dans l'intervalle") {
        @Override
        public void executer() {
          editeur.newParcoursF0(f0.getTIntervalles(borneinf, bornesup));
        }
      });
     add(new GAction("...dans et au-dessus") {
        @Override
        public void executer() {
          editeur.newParcoursF0(f0.getTIntervalles(borneinf, Double.MAX_VALUE));
        }
      });
     add(new GAction("...dans et au-dessous") {
        @Override
        public void executer() {
          editeur.newParcoursF0(f0.getTIntervalles(-Double.MAX_VALUE, bornesup));
        }
      });
    }
    private void init( double borneinf, double bornesup)  {
      this.borneinf = borneinf;
      this.bornesup = bornesup;
    }
  }
}