/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.statistiques;

/**
 *
 * @author Bernard
 */

  import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import org.jfree.chart.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.data.statistics.*;
import org.jfree.data.xy.*;
import org.jfree.ui.*;
import visuAnalor.*;
import visuAnalor.BandeauFonctions.*;
import visuAnalor.donnees.*;
import visuAnalor.donnees.Donnees.*;
import visuAnalor.util.*;
import visuAnalor.util.GMessages.*;

/**
 *
 * @author Bernard
 */
@SuppressWarnings("serial")
public class VisuStatsIntensite extends JFrame implements DonneesListener {
  private Donnees donnees;
  private PanneauEditeur editeur;
  private XYPlot plot;
  private NumberAxis axeI;
  private SimpleHistogramDataset dataset;
    private final static Paint COULEUR_INTENSITE = new Color(0.3f, 0f, 0.3f);


  public VisuStatsIntensite(Container fenetre, Donnees donnees, PanneauEditeur editeur) {
    super("Statistiques Intensité");
    if (donnees.getCourbeIntensite()==null) return;
    Point pos = new Point(fenetre.getX()+200, fenetre.getY()+50);
    Dimension dim = new Dimension(Math.max(200, fenetre.getWidth()-400),
            fenetre.getHeight()-30);
    setLocation(pos);
    setSize(dim);
    this.donnees = donnees;
    this.editeur = editeur;
    donnees.addEventListener(this);
    dataset = new SimpleHistogramDataset("");
    dataset.setAdjustForBinSize(false);
    axeI = new NumberAxis();
    NumberAxis axeFreq = new NumberAxis();
    XYBarRenderer rendu = new XYBarRenderer();
    plot = new XYPlot();
    plot.setDomainAxis(axeI);
    plot.setRangeAxis(axeFreq);
    plot.setOrientation(PlotOrientation.HORIZONTAL);
    plot.setDataset(dataset);
    plot.setRenderer(rendu);
    rendu.setDrawBarOutline(true);
    rendu.setAutoPopulateSeriesOutlinePaint(false);
    rendu.setBaseOutlinePaint(Color.BLACK);
    rendu.setShadowVisible(false);
    rendu.setAutoPopulateSeriesFillPaint(false);
    rendu.setSeriesPaint(0, COULEUR_INTENSITE);
    rendu.setBaseSeriesVisibleInLegend(false);
    rendu.setBarPainter(new StandardXYBarPainter());
    rendu.setMargin(0.1);
    JFreeChart chart = new JFreeChart("Statistiques Intensité", plot);
    ChartStatsIntensite graphique = 
            new ChartStatsIntensite(chart, editeur, donnees.getCourbeIntensite());
    setContentPane(graphique);
    // Fermeture de la fenêtre
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent evt) {
        fermerFenetre();
      }
    });
    setVisible(true);
    newDonnees();
  }

  private void fermerFenetre() {
    donnees.removeEventListener(this);
    dispose();
  }

  @Override
  public void traiterEventDonnees(MessageDonnes evt) {
    switch (evt.getType()) {
      case CLEAR_DONNEES:
      case NEW_DONNEES:
        fermerFenetre();
        return;
      case MODIF_F0:
      case NEW_F0:
         return;
      case NEW_INTENSITE:
       newDonnees();
        return;
      case MODIF_TIRE:
      case MODIF_NOM_TIRE:
      case NEW_SON:
      case ADD_TIRES:
      case SUP_TIRE:
        return;
      default:
        throw new UnsupportedOperationException("Cas "+evt.getType()
                +" oublié dans un switch");
    }
  }

  private void newDonnees() {
    dataset.clearObservations();
    dataset.removeAllBins();
    CourbeIntensite I = donnees.getCourbeIntensite();
    double[] bornes = I.calculerBornes();
    if (bornes==null) {
      fermerFenetre();
      return;
    }
    double borneinf = bornes[0];
    double bornesup = bornes[1];
    int inf = (int) Math.floor(borneinf);
    int sup = (int) Math.ceil(bornesup);
    for (int i = inf-1; i<sup+1; i++) {
      SimpleHistogramBin bin = new SimpleHistogramBin(i, i+1, true, false);
      dataset.addBin(bin);
    }
    XYDataset Idataset = I.getDataSet();
   for (int i = 0; i<Idataset.getItemCount(0); i++) {
      dataset.addObservation(Idataset.getYValue(0, i));
    }
    plot.getDomainAxis().setLowerBound(inf-2);
    plot.getDomainAxis().setUpperBound(sup+1);
    
//   axeI.configurerTicks(inf, sup, moyenne);

  }

  private static class ChartStatsIntensite extends ChartPanel {
    SimpleHistogramDataset dataset;
    XYPlot plot;
    PopupStatsIntensite popup;

    public ChartStatsIntensite(JFreeChart chart, PanneauEditeur editeur, 
            CourbeIntensite I) {
      super(chart);
      this.popup = new PopupStatsIntensite(editeur, I);
      this.plot = (XYPlot) chart.getPlot();
      this.dataset = (SimpleHistogramDataset) plot.getDataset();
      setMouseZoomable(false);
      setPopupMenu(null);
    }

    @Override
    public void mousePressed(MouseEvent e) {
      try {
        if (e.isPopupTrigger()) {
          showPopup(e);
          return;
        }
      } catch (Throwable ex) {
        GMessages.erreurFatale(ex);
      }
//    System.out.println(x1Pressed);
    }

    @Override
    public void mouseReleased(MouseEvent e) {
      try {
        if (e.isPopupTrigger()) {
          showPopup(e);
          return;
        }
      } catch (Throwable ex) {
        GMessages.erreurFatale(ex);
      }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
    }

    @Override
    public void mouseMoved(MouseEvent e) {
    }

    private void showPopup(MouseEvent e) {
      Point2D pt = translateScreenToJava2D(e.getPoint());
      PlotRenderingInfo info = getChartRenderingInfo().getPlotInfo();
      double y = pt.getY();
      Rectangle2D area = info.getDataArea();
      if (!area.contains(pt)) return;
      double val = plot.getDomainAxis().java2DToValue(y, area, RectangleEdge.LEFT);
      if (val<dataset.getStartXValue(0, 0)) return;
      for (int i = 0; i<dataset.getItemCount(0); i++) {
        if (val>=dataset.getEndXValue(0, i)) continue;
        popup.init(dataset.getStartXValue(0, i), dataset.getEndXValue(0, i));
        popup.show(e.getComponent(), e.getX(), e.getY());
        return;
      }
    }
  }

  private static class PopupStatsIntensite extends JPopupMenu {
    double borneinf, bornesup;
    PanneauEditeur editeur;
    CourbeIntensite I;

    private PopupStatsIntensite(PanneauEditeur edit, CourbeIntensite Int) {
      super();
      
       this.editeur = edit;
      this.I = Int;
      add("<html><I>Parcourir les valeurs...").setEnabled(false);
      add(new GAction("...dans l'intervalle") {
        @Override
        public void executer() {
          editeur.newParcoursI(I.getTIntervalles(borneinf, bornesup));
        }
      });
     add(new GAction("...dans et au-dessus") {
        @Override
        public void executer() {
          editeur.newParcoursI(I.getTIntervalles(borneinf, Double.MAX_VALUE));
        }
      });
     add(new GAction("...dans et au-dessous") {
        @Override
        public void executer() {
          editeur.newParcoursI(I.getTIntervalles(-Double.MAX_VALUE, bornesup));
        }
      });
    }
    private void init( double borneinf, double bornesup)  {
      this.borneinf = borneinf;
      this.bornesup = bornesup;
    }
  }
}
