/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package visuAnalor.util;

import java.awt.*;
import java.beans.*;
import java.io.*;
import java.util.*;
import java.util.prefs.Preferences;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import org.jfree.ui.FilesystemFilter;

/**
 *
 * @author Bernard
 */
public abstract class Fichiers {
  public enum TypeFichier {
    CORPUS_ANALOR("Fichiers Analor", "or"),
    DONNEES("Donnees Analor", "or"),
    PRAAT("Fichier Praat", "PitchTier", "Pitch", "IntensityTier",
    "Intensity", "TextGrid", "Sound", "Collection"),
    PRAAT_F0("Courbe intonative", "PitchTier",  "Pitch"),   
    PRAAT_F0_PITCHTIER("Courbe intonative (.PitchTier)", "PitchTier"),
    PRAAT_F0_PITCH("Courbe intonative (.Pitch - format rhapsodie)", "Pitch"),
    PRAAT_F0LISSEE_PITCHTIER("Courbe intonative (.PitchTier)", "PitchTier"),
    PRAAT_F0LISSEE_PITCH("Courbe intonative (.Pitch - format rhapsodie)", "Pitch"),
    PRAAT_INTENSITYTIER("Courbe d'intensité (.IntensityTier)", "IntensityTier"),
    PRAAT_INTENSITY("Courbe d'intensité (.Intensity)", "Intensity"),
    PRAAT_TEXTGRID("Tires d'annotation (.TextGrid)", "TextGrid"),
    PRAAT_COLLECTION("Ensemble de données Praat (.Collection)", "Collection"),
    PROSOGRAMME("Prosogramme format texte (séparateur : tabulation)", "txt"),
    TABLEUR_STATS("Tableur format texte (séparateur : tabulation)", "txt"),
    TABLEUR_TIRE("Tire format texte (séparateur : tabulation)", "txt"),
    SON("Fichier son", "wav"),
    PARAMETRES("Fichier de paramétrage", "orp"),
    TABLEUR_PARAMS("Paramètres format texte (séparateur : tabulation)", "txt"),
    TEXTE_PERF("Performances format texte", "txt"),
    GRAPHIQUE_PNG("Graphique (.png)", "png");
    String[] ext;
    String nom;
    TypeFichier(String nom, String... ext) {
      this.nom = nom;
      this.ext = ext;
    }
    public String getNom() {
      return nom;
    }
    public String getExtension() {
      return ext[0];
    }
    public String[] getExtensions() {
      return ext;
    }
    public boolean isPerf() {
      return this==TEXTE_PERF;
    }
   public boolean isResultats() {
      return this==TABLEUR_STATS;
    }
    public boolean isParams() {
      return this==TABLEUR_PARAMS;
    }
    public boolean isTire() {
      return this==TABLEUR_TIRE;
    }
    public boolean isImportDonnees() {
      return name().startsWith("PRAAT");
    }
  }

  public enum Commande {
    OUVRIR("Ouvrir"),
    ENREGISTRER("Enregistrer");
    String label;
    Commande(String label) {
      this.label = label;
    }
    public String getLabel() {
      return label;
    }
  }
  // Variables de classe
  private static Preferences pref =
          Preferences.userNodeForPackage(Fichiers.class);
  private static File repertoireCourantDonnees =
          creerRepertoireCourantDonnees();
  private static EnumMap<TypeFichier, File> fichiersCourants =
          creerFichiersCourants();
  private static File creerRepertoireCourantDonnees() {
    return new File(pref.get("repertoire", System.getProperty("user.dir")));
  }
  private static EnumMap<TypeFichier, File> creerFichiersCourants() {
    EnumMap<TypeFichier, File> fics =
            new EnumMap<TypeFichier, File>(TypeFichier.class);
    String nomfic = pref.get("nomDonnees", "");
    if (!nomfic.isEmpty())
      fics.put(TypeFichier.DONNEES,
              new File(repertoireCourantDonnees,
              nomfic+'.'+TypeFichier.DONNEES.getExtension()));
    return fics;
  }
  public static File trouverFichier(TypeFichier type, String nomFichier) {
    File fichier = new File(repertoireCourantDonnees, nomFichier);
    if (!getExtension(fichier).equals(type.getExtension()))
      fichier = changerExtension(fichier, type.getExtension());
    if (!fichier.exists()) return null;
    majFichiersCourants(type, fichier);
    return fichier;
  }
  public static File trouverFichier(TypeFichier type, File oldfichier) {
    File fichier = changerExtension(oldfichier, type.getExtension());
    if (!fichier.exists()) return null;
    majFichiersCourants(type, fichier);
    return fichier;
  }
  public static File getRepertoireCourant(TypeFichier type) {
    if (fichiersCourants.containsKey(type))
      return fichiersCourants.get(type).getParentFile();
    return repertoireCourantDonnees;
  }
  public static File getFichierCourant(TypeFichier type) {
    if (fichiersCourants.containsKey(type)) return fichiersCourants.get(type);
    return null;
  }
  public static String getNomFichierCourant(TypeFichier type) {
    if (fichiersCourants.containsKey(type))
      return extraireNom(fichiersCourants.get(type));
    return "";
  }
  public static void removeFichierCourant(TypeFichier type) {
    fichiersCourants.remove(type);
  }
  private static void majFichiersCourants(TypeFichier type, File fichier) {
    fichiersCourants.put(type, fichier);
//    if (type == Type.GLOZZ_CORPUS) {
//      fichiersCourants.put(Type.GLOZZ_ANNOT, changerExtension(fichier, Type.GLOZZ_ANNOT.ext));
//      fichiersCourants.put(Type.GLOZZ_MODELE, changerExtension(fichier, Type.GLOZZ_MODELE.ext));
//    }
    if (type==TypeFichier.DONNEES) {
      repertoireCourantDonnees = fichier.getParentFile();
      pref.put("repertoire", repertoireCourantDonnees.getPath());
      pref.put("nomDonnees", extraireNom(fichier));
    }

  }
  // Utilitaires de manipulation de noms de fichiers
  private static File ajouterExtension(File fichier, String extension) {
    String nom = fichier.getName();
    return new File(fichier.getParentFile(), nom+"."+extension);
  }
  private static File changerExtension(File fichier, String extension) {
    String nom = fichier.getName();
    int ind = nom.lastIndexOf('.');
    if (ind>=0) nom = nom.substring(0, ind);
    return new File(fichier.getParentFile(), nom+"."+extension);
  }
  public static String extraireNom(File fichier) {
    if (fichier==null) return null;
    String nom = fichier.getName();
    int ind = nom.lastIndexOf('.');
    if (ind<0) return nom;
    else return nom.substring(0, ind);
  }
  private static String getExtension(File fichier) {
    String nom = fichier.getName();
    int ind = nom.lastIndexOf('.');
    if (ind>=0) return nom.substring(ind+1);
    return "";
  }  // Choix des fichiers
  public static File choisirFichier(Commande commande, String titre, 
          TypeFichier type) {
    return choisirFichier(commande, titre, type, null);
  }
  public static File choisirFichier(Commande commande, String titre, 
          TypeFichier type, JComponent complement) {
    JFileChooser choixFichier = new JFileChooser();
    choixFichier.resetChoosableFileFilters();
    choixFichier.setFileFilter(new FileNameExtensionFilter(type.getNom(), type.getExtensions()));
    choixFichier.setDialogTitle(titre);
    if (complement!=null) choixFichier.setAccessory(complement);
    choixFichier.setCurrentDirectory(getRepertoireCourant(type));
    File fichierPropose;
    String nom;
    if(type.isPerf()) {
      fichierPropose = new File(getRepertoireCourant(type), titre+"_perf");
    }
    else if (type.isTire()) {
      choixFichier.setDialogTitle("Nom du fichier pour la tire \""+titre+"\"");
      nom = extraireNom(getFichierCourant(TypeFichier.DONNEES));
      if (nom==null) fichierPropose = null;
      else
        fichierPropose = new File(getRepertoireCourant(type), nom+"_tire_"+titre);
    } else if (type.isResultats()) {
      nom = extraireNom(getFichierCourant(TypeFichier.DONNEES));
      if (nom==null) fichierPropose = null;
      else fichierPropose = new File(getRepertoireCourant(type), nom+"_res");
    } else if (type.isParams()) {
      nom = extraireNom(getFichierCourant(TypeFichier.PARAMETRES));
      if (nom==null) fichierPropose = null;
      else fichierPropose = new File(getRepertoireCourant(type), nom+"_param");
    } else {
      fichierPropose = getFichierCourant(type);
    }
    if (fichierPropose!=null) {
      if (commande==Commande.OUVRIR&&!fichierPropose.exists()) {
        fichierPropose = new File(getRepertoireCourant(type), "*."
                +type.getExtension());
      }
      choixFichier.setSelectedFile(fichierPropose);
    }
    int reponse = choixFichier.showDialog(null, commande.getLabel());
    if (reponse!=JFileChooser.APPROVE_OPTION) return null;
    File fichierChoisi = choixFichier.getSelectedFile();
    if (commande==Commande.ENREGISTRER) {
      if (!getExtension(fichierChoisi).equals(type.getExtension()))
        fichierChoisi = ajouterExtension(fichierChoisi, type.getExtension());
      if (fichierChoisi.exists()) {
        int rep = JOptionPane.showOptionDialog(null, "Remplacer "+fichierChoisi.getName()+" ?",
                "Fichier existant", JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,
                null, new Object[]{"Remplacer", "Annuler"}, "Remplacer");
        if (rep==1) return null;
      }
    }
    majFichiersCourants(type, fichierChoisi);
    if (commande==Commande.OUVRIR&&type.isImportDonnees())
      fichiersCourants.remove(TypeFichier.DONNEES);
    return fichierChoisi;
  }

  public static File[] choisirCorpus(String titre, TypeFichier type) {
    JFileChooser choixFichier = new JFileChooser();
    FilesystemFilter filtre = new FilesystemFilter(type.getExtensions(), "", false);
    choixFichier.addPropertyChangeListener(new SelectFics(choixFichier, filtre));
     choixFichier.resetChoosableFileFilters();
    choixFichier.setFileFilter(new FileNameExtensionFilter(
            type.getNom(), type.getExtension()));
    choixFichier.setDialogTitle(titre);
    choixFichier.setMultiSelectionEnabled(true);
    File rep = getRepertoireCourant(type);
    choixFichier.setCurrentDirectory(rep);
    int reponse = choixFichier.showDialog(null, "Choisir");
    if (reponse!=JFileChooser.APPROVE_OPTION) return new File[0];
     File[] fics = choixFichier.getSelectedFiles();
     if(fics==null||fics.length==0) return new File[0];
   majFichiersCourants(TypeFichier.CORPUS_ANALOR, fics[0]);
    return fics;
 }

  public static class GCodage {
    JPanel contenant;
    JComboBox saisieCodage;
    public GCodage() {
      saisieCodage = new JComboBox(new String[]{
                "Microsoft (windows-1252)",
                "UTF-8 (unicode 8 bits)",
                "UTF-16 (unicode 16 bits)",
                "ISO-8859-1 (latin-1)",
                "<autre (à saisir)>"
              });
      saisieCodage.setEditable(true);
      saisieCodage.setSelectedIndex(0);
      contenant = new JPanel(new BorderLayout());
      contenant.setBorder(new TitledBorder("Codage des caractères"));
      contenant.add(saisieCodage, BorderLayout.NORTH);
      contenant.add(new JPanel(), BorderLayout.CENTER);
    }
    public JComponent getContenant() {
      return contenant;
    }
    public String getCode() {
      switch (saisieCodage.getSelectedIndex()) {
        case 0:
          return "windows-1252";
        case 1:
          return "UTF-8";
        case 2:
          return "UTF-16";
        case 3:
          return "ISO-8859-1";
        default:
          return (String) saisieCodage.getSelectedItem();
      }
    }
  }
  public static class SelectFics implements PropertyChangeListener{
    JFileChooser choix;
    FilesystemFilter filtre;
    SelectFics(JFileChooser choix, FilesystemFilter filtre) {
      this.choix = choix;
      this.filtre = filtre;
    }
    @Override
       public void propertyChange(PropertyChangeEvent e) {
        if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(e.getPropertyName())) {
           choix.setSelectedFiles(choix.getCurrentDirectory().listFiles(filtre));
        }
    }
  }
}
